
with Ada.Command_Line;                  use Ada.Command_Line;
with Ada.Text_IO;                       use Ada.Text_IO;
with Allegro;                           use Allegro;
with Allegro.Audio;                     use Allegro.Audio;
with Allegro.Audio.Codecs;              use Allegro.Audio.Codecs;
with Allegro.System;                    use Allegro.System;
with Allegro.Time;                      use Allegro.Time;
with GNAT.OS_Lib;
with Interfaces.C;                      use Interfaces.C;

--
-- Ryan Dickie
-- Audio example that loads a series of files and puts them through the mixer.
-- Originlly derived from the audio example on the wiki.
--
package body Ex_Acodec is

    procedure Ada_Main is
        voice          : A_Allegro_Voice;
        mixer          : A_Allegro_Mixer;
        sample         : A_Allegro_Sample_Instance;
        sample_data    : A_Allegro_Sample;
        sample_time    : Float;

        type A_String is access all String;
        type String_Array is array (Integer range <>) of A_String;
        type A_String_Array is access all String_Array;
        filenames : A_String_Array;
    begin
        if Argument_Count < 1 then
            filenames := new String_Array(1..1);
            filenames(1) := new String'("data/testing.ogg");
        else
            filenames := new String_Array(1..Argument_Count);
            for i in 1..Argument_Count loop
                filenames(i) := new String'(Argument(i));
            end loop;
        end if;

        if not Al_Initialize then
            Put_Line( "Could not init Allegro." );
            GNAT.OS_Lib.OS_Exit( 1 );
        end if;

        if not Al_Init_Acodec_Addon then
            Put_Line( "Could not init audio codec addon." );
            GNAT.OS_Lib.OS_Exit( 1 );
        end if;

        if not Al_Install_Audio then
            Put_Line( "Could not init sound!" );
            GNAT.OS_Lib.OS_Exit( 1 );
        end if;

        voice := Al_Create_Voice( 44100, ALLEGRO_AUDIO_DEPTH_INT16, ALLEGRO_CHANNEL_CONF_2 );
        if voice = null then
            Put_Line( "Could not create ALLEGRO_VOICE." );
            GNAT.OS_Lib.OS_Exit( 1 );
        end if;

        mixer := Al_Create_Mixer( 44100, ALLEGRO_AUDIO_DEPTH_FLOAT32, ALLEGRO_CHANNEL_CONF_2 );
        if mixer = null then
            Put_Line( "al_create_mixer failed." );
            GNAT.OS_Lib.OS_Exit( 1 );
        end if;

        if not Al_Attach_Mixer_To_Voice( mixer, voice ) then
            Put_Line( "al_attach_mixer_to_voice failed." );
            GNAT.OS_Lib.OS_Exit( 1 );
        end if;

        sample := Al_Create_Sample_Instance( null );
        if sample = null then
            Put_Line( "al_create_sample failed." );
            GNAT.OS_Lib.OS_Exit( 1 );
        end if;

        for i in filenames'Range loop
            sample_time := 0.0;

            -- Load the entire sound file from disk. */
            sample_data := Al_Load_Sample( filenames(i).all );
            if sample_data = null then
                Put_Line( "Could not load sample from '" & filenames(i).all & "'!" );
            end if;

            if sample_data /= null and then not Al_Set_Sample( sample, sample_data ) then
                Put_Line( "al_set_sample_instance_ptr failed." );
            end if;

            if sample /= null and then not Al_Attach_Sample_Instance_To_Mixer( sample, mixer ) then
                Put_Line( "al_attach_sample_instance_to_mixer failed." );
                GNAT.OS_Lib.OS_Exit( 1 );
            end if;

            -- Play sample in looping mode.
            if sample /= null then
               if not Al_Set_Sample_Instance_Playmode( sample, ALLEGRO_PLAYMODE_LOOP ) then
                    Put_Line( "Al_Set_Sample_Instance_Playmode failed" );
                    GNAT.OS_Lib.OS_Exit( 1 );
                end if;
                if not Al_Play_Sample_Instance( sample ) then
                    Put_Line( "Al_Play_Sample_Instance failed" );
                    GNAT.OS_Lib.OS_Exit( 1 );
                end if;

                sample_time := Al_Get_Sample_Instance_Time( sample );
                Put_Line( "Playing '" & filenames(i).all & "' (" & sample_time'Img & " seconds) 3 times" );

                Al_Rest( double(sample_time) );

                if not Al_Set_Sample_Instance_Gain( sample, 0.5 ) then
                    Put_Line( "Failed to set gain." );
                end if;
                Al_Rest( double(sample_time) );

                if not Al_Set_Sample_Instance_Gain( sample, 0.25 ) then
                    Put_Line( "Failed to set gain." );
                end if;
                Al_Rest( double(sample_time) );

                if not Al_Stop_Sample_Instance( sample ) then
                    Put_Line( "Al_Stop_Sample_Instance failed" );
                    GNAT.OS_Lib.OS_Exit( 1 );
                end if;
                Put_Line( "" );

                -- Free the memory allocated.
                if not Al_Set_Sample( sample, null ) then
                    Put_Line( "Al_Set_Sample failed" );
                    GNAT.OS_Lib.OS_Exit( 1 );
                end if;
            end if;
            if sample_data /= null then
                Al_Destroy_Sample( sample_data );
            end if;
        end loop;

        Al_Destroy_Sample_Instance( sample );
        Al_Destroy_Mixer( mixer );
        Al_Destroy_Voice( voice );

        Al_Uninstall_Audio;
    end Ada_Main;

begin
    Set_Ada_Main( Ada_Main'Access );
end Ex_Acodec;
