
with Ada.Command_Line;                  use Ada.Command_Line;
with Ada.Text_IO;                       use Ada.Text_IO;
with Allegro;                           use Allegro;
with Allegro.Bitmaps;                   use Allegro.Bitmaps;
with Allegro.Bitmaps.Image_IO;          use Allegro.Bitmaps.Image_IO;
with Allegro.Color;                     use Allegro.Color;
with Allegro.Image_Addon;               use Allegro.Image_Addon;
with Allegro.System;                    use Allegro.System;
with Allegro.Time;                      use Allegro.Time;
with GNAT.OS_Lib;
with Interfaces;                        use Interfaces;
with Interfaces.C;                      use Interfaces.C;

-- Image conversion example
package body Ex_Convert is

    procedure Ada_Main is
        bitmap : A_Allegro_Bitmap;
        t0, t1 : double;
    begin
        if Argument_Count < 2 then
            Put_Line( "Usage: exnew_convert <infile> <outfile>" );
            Put_Line( "    Possible file types: BMP PCX PNG TGA" );
            GNAT.OS_Lib.OS_Exit( 1 );
        end if;

        if not Al_Initialize then
            Put_Line( "Could not init Allegro." );
            GNAT.OS_Lib.OS_Exit( 1 );
        end if;

        if not Al_Init_Image_Addon then
            Put_Line( "Could not init Allegro IIO addon." );
            GNAT.OS_Lib.OS_Exit( 1 );
        end if;

        Al_Set_New_Bitmap_Format( ALLEGRO_PIXEL_FORMAT_ARGB_8888 );
        Al_Set_New_Bitmap_Flags( ALLEGRO_MEMORY_BITMAP or
                                 ALLEGRO_NO_PREMULTIPLIED_ALPHA );

        bitmap := Al_Load_Bitmap( Argument(1) );
        if bitmap = null then
            Put_Line( "Error loading input file" );
            GNAT.OS_Lib.OS_Exit( 1 );
        end if;

        t0 := Al_Get_Time;
        if not Al_Save_Bitmap( Argument(2), bitmap ) then
            Put_Line( "Error saving bitmap" );
            GNAT.OS_Lib.OS_Exit( 1 );
        end if;
        t1 := Al_Get_Time;
        Put_Line( "Saving took" & Integer'Image( Integer( (t1 - t0) * 1000.0 ) ) &
                  " milliseconds" );

        Al_Destroy_Bitmap( bitmap );
    end Ada_Main;

begin
    Set_Ada_Main( Ada_Main'Access );
end Ex_Convert;
