
with Ada.Numerics.Elementary_Functions; use Ada.Numerics.Elementary_Functions;
with Allegro;                           use Allegro;
with Allegro.Bitmaps;                   use Allegro.Bitmaps;
with Allegro.Bitmaps.Drawing;           use Allegro.Bitmaps.Drawing;
with Allegro.Bitmaps.Image_IO;          use Allegro.Bitmaps.Image_IO;
with Allegro.Blending;                  use Allegro.Blending;
with Allegro.Color;                     use Allegro.Color;
with Allegro.Displays;                  use Allegro.Displays;
with Allegro.Displays.Monitors;         use Allegro.Displays.Monitors;
with Allegro.Drawing;                   use Allegro.Drawing;
with Allegro.Events;                    use Allegro.Events;
with Allegro.Fonts;                     use Allegro.Fonts;
with Allegro.Image_Addon;               use Allegro.Image_Addon;
with Allegro.Keyboard;                  use Allegro.Keyboard;
with Allegro.Mouse;                     use Allegro.Mouse;
with Allegro.System;                    use Allegro.System;
with Allegro.Time;                      use Allegro.Time;
with Allegro.Timers;                    use Allegro.Timers;
with Common;                            use Common;
with GNAT.Random_Numbers;               use GNAT.Random_Numbers;
with Interfaces;                        use Interfaces;
with Interfaces.C;                      use Interfaces.C;

package body Ex_Draw_Bitmap is

    procedure Ada_Main is

        FPS         : constant := 60;
        MAX_SPRITES : constant := 1024;

        type Sprite is
            record
                x,   y : Float;
                dx, dy : Float;
            end record;

        type A_String is access all String;
        type String_Array is array (Integer range <>) of A_String;

        text : constant String_Array := String_Array'(
            0 => new String'("B - toggle alpha blending"),
            1 => new String'("Left/Right - change bitmap size"),
            2 => new String'("Up/Down - change bitmap count"),
            3 => new String'("F1 - toggle help text")
        );

        target_info : constant String_Array := String_Array'(
            0 => new String'("textures"),
            1 => new String'("memory buffers")
        );

        blend_info : constant String_Array := String_Array'(
            0 => new String'("alpha"),
            1 => new String'("additive"),
            2 => new String'("tinted"),
            3 => new String'("solid")
        );

        type Sprite_Array is array (Integer range <>) of Sprite;

        type double_array is array (Integer range <>) of double;

        type Example_Rec is
            record
                sprites              : Sprite_Array(0..MAX_SPRITES-1);
                use_memory_bitmaps   : Boolean := False;
                blending             : Integer := 0;
                display              : A_Allegro_Display;
                mysha                : A_Allegro_Bitmap;
                bitmap               : A_Allegro_Bitmap;

                bitmap_size          : Integer := 0;
                sprite_count         : Integer := 0;
                show_help            : Boolean := True;
                font                 : A_Allegro_Font;

                mouse_down           : Boolean := False;
                last_x,
                last_y               : Integer := 0;

                white,
                half_white,
                dark,
                red                  : Allegro_Color;

                direct_speed_measure : double := 0.0;

                ftpos                : Integer := 0;
                frame_times          : double_array(0..FPS-1) := (others => 0.0);
            end record;

        example : Example_Rec;

        ------------------------------------------------------------------------

        procedure Add_Time is
        begin
            example.frame_times(example.ftpos) := Al_Get_Time;
            example.ftpos := example.ftpos + 1;
            if example.ftpos >= FPS then
                example.ftpos := 0;
            end if;
        end Add_Time;

        ------------------------------------------------------------------------

        procedure Get_fps( average, minmax : out Integer ) is
            prev : Integer := FPS - 1;
            min_dt : double := 1.0;
            max_dt : double := 1.0 / 1_000_000.0;
            av     : double := 0.0;
            d      : double;
            dt     : double;
        begin
            for i in 0..FPS-1 loop
                if i /= example.ftpos then
                    dt := example.frame_times(i) - example.frame_times(prev);
                    if dt < min_dt then
                        min_dt := dt;
                        if min_dt < 0.000_001 then
                            min_dt := 0.000_001;
                        end if;
                    elsif dt > max_dt then
                        max_dt := dt;
                    end if;
                    av := av + dt;
                end if;
                prev := i;
            end loop;

            av := av / double(FPS - 1);
            if av < 0.000_001 then
                -- prevent overflow on startup
                average := 1_000_000;
            else
                average := Integer(double'Ceiling( 1.0 / av ));
            end if;
            d := 1.0 / min_dt - 1.0 / max_dt;
            minmax := Integer(double'Floor( d / 2.0 ));
        end Get_fps;

        ------------------------------------------------------------------------

        procedure Add_Sprite is
            w : constant Integer := Al_Get_Display_Width( example.display );
            h : constant Integer := Al_Get_Display_Height( example.display );
            g : GNAT.Random_Numbers.Generator;
            a : Float;
            s : Integer;
        begin
            if example.sprite_count < MAX_SPRITES then
                s := example.sprite_count;
                example.sprite_count := example.sprite_count + 1;

                Reset( g );
                a := Float(Integer'(Random( g )) mod 360);
                example.sprites(s).x := Float(Integer'(Random( g )) mod (w - example.bitmap_size));
                example.sprites(s).y := Float(Integer'(Random( g )) mod (h - example.bitmap_size));
                example.sprites(s).dx := Cos( a ) * Float(FPS) * 2.0;
                example.sprites(s).dy := Sin( a ) * Float(FPS) * 2.0;
            end if;
        end Add_Sprite;

        ------------------------------------------------------------------------

        procedure Add_Sprites( n : Integer ) is
        begin
            for i in 1..n loop
                Add_Sprite;
            end loop;
        end Add_Sprites;

        ------------------------------------------------------------------------

        procedure Remove_Sprites( n : Integer ) is
        begin
            example.sprite_count := Integer'Max( example.sprite_count - n, 0 );
        end Remove_Sprites;

        ------------------------------------------------------------------------

        procedure Change_Size( size : Integer ) is
            bw, bh : Integer;
        begin
            if size < 0 then
                return;
            end if;

            if example.bitmap /= null then
                Al_Destroy_Bitmap( example.bitmap );
            end if;
            if example.use_memory_bitmaps then
                Al_Set_New_Bitmap_Flags( ALLEGRO_MEMORY_BITMAP );
            else
                Al_Set_New_Bitmap_Flags( ALLEGRO_VIDEO_BITMAP );
            end if;

            example.bitmap := Al_Create_Bitmap( size, size );
            example.bitmap_size := size;
            Al_Set_Target_Bitmap( example.bitmap );
            Al_Set_Blender( ALLEGRO_ADD, ALLEGRO_ONE, ALLEGRO_ZERO );
            Al_Clear_To_Color( Al_Map_RGBA_f( 0.0, 0.0, 0.0, 0.0 ) );
            bw := Al_Get_Bitmap_Width( example.mysha );
            bh := Al_Get_Bitmap_Height( example.mysha );
            Al_Draw_Scaled_Bitmap( example.mysha, 0.0, 0.0, Float(bw), Float(bh), 0.0, 0.0, Float(size), Float(size), 0 );
            Al_Set_Target_Backbuffer( example.display );
        end Change_Size;

        ------------------------------------------------------------------------

        procedure Sprite_Update( s : in out Sprite ) is
            w : constant Integer := Al_Get_Display_Width( example.display );
            h : constant Integer := Al_Get_Display_Height( example.display );
        begin
            s.x := s.x + s.dx / Float(FPS);
            s.y := s.y + s.dy / Float(FPS);

            if s.x < 0.0 then
                s.x := -s.x;
                s.dx := -s.dx;
            elsif Integer(s.x) + example.bitmap_size > w then
                s.x := -s.x + 2.0 * Float(w - example.bitmap_size);
                s.dx := -s.dx;
            end if;
            if s.y < 0.0 then
                s.y := -s.y;
                s.dy := -s.dy;
            elsif Integer(s.y) + example.bitmap_size > h then
                s.y := -s.y + 2.0 * Float(h - example.bitmap_size);
                s.dy := -s.dy;
            end if;

            if example.bitmap_size > w then
                s.x := Float(w / 2 - example.bitmap_size / 2);
            end if;
            if example.bitmap_size > h then
                s.y := Float(h / 2 - example.bitmap_size / 2);
            end if;
        end Sprite_Update;

        ------------------------------------------------------------------------

        procedure Update is
        begin
            for i in 0..example.sprite_count-1 loop
                Sprite_Update( example.sprites(i) );
            end loop;
        end Update;

        ------------------------------------------------------------------------

        procedure Redraw is
            w      : constant Integer := Al_Get_Display_Width( example.display );
            h      : constant Integer := Al_Get_Display_Height( example.display );
            fh     : constant Integer := Al_Get_Font_Line_Height( example.font );
            f1, f2 : Integer;
            tint   : Allegro_Color := example.white;
        begin
            if example.blending = 0 then
                Al_Set_Blender( ALLEGRO_ADD, ALLEGRO_ONE, ALLEGRO_INVERSE_ALPHA );
                tint := example.half_white;
            elsif example.blending = 1 then
                Al_Set_Blender( ALLEGRO_ADD, ALLEGRO_ONE, ALLEGRO_ONE );
                tint := example.dark;
            elsif example.blending = 2 then
                Al_Set_Blender( ALLEGRO_ADD, ALLEGRO_ONE, ALLEGRO_ZERO );
                tint := example.red;
            elsif example.blending = 3 then
                Al_Set_Blender( ALLEGRO_ADD, ALLEGRO_ONE, ALLEGRO_ZERO );
            end if;

            for i in 0..example.sprite_count-1 loop
                Al_Draw_Tinted_Bitmap( example.bitmap, tint, example.sprites(i).x, example.sprites(i).y, 0 );
            end loop;

            Al_Set_Blender( ALLEGRO_ADD, ALLEGRO_ONE, ALLEGRO_INVERSE_ALPHA );
            if example.show_help then
                for i in 0..3 loop
                    Al_Draw_Text( example.font, example.white, 0.0, Float(h - 5 * fh + i * fh), 0, text(i).all );
                end loop;
            end if;

            Al_Draw_Text( example.font, example.white, 0.0, 0.0, 0, "count:" & Integer'Image( example.sprite_count ) );
            Al_Draw_Text( example.font, example.white, 0.0, Float(fh), 0, "size:" & Integer'Image( example.bitmap_size ) );
            Al_Draw_Text( example.font, example.white, 0.0, Float(fh * 2), 0, target_info(Boolean'Pos( example.use_memory_bitmaps )).all );
            Al_Draw_Text( example.font, example.white, 0.0, Float(fh * 3), 0, blend_info(example.blending).all );

            Get_fps( f1, f2 );
            Al_Draw_Text( example.font, example.white, Float(w), 0.0, ALLEGRO_ALIGN_RIGHT, "FPS:" & Integer'Image( f1 ) & " +- " & Integer'Image( f2 ) );
            if example.direct_speed_measure < 0.000_001 then
                -- prevent overflow on startup
                Al_Draw_Text( example.font, example.white, Float(w), Float(fh), ALLEGRO_ALIGN_RIGHT, " --- / sec" );
            else
                Al_Draw_Text( example.font, example.white, Float(w), Float(fh), ALLEGRO_ALIGN_RIGHT, Integer'Image( Integer(1.0 / example.direct_speed_measure) ) & " / sec" );
            end if;
        end Redraw;

        ------------------------------------------------------------------------

        timer       : A_Allegro_Timer;
        queue       : A_Allegro_Event_Queue;
        info        : aliased Allegro_Monitor_Info;
        event       : aliased Allegro_Event;
        w           : Integer := 640;
        h           : Integer := 480;
        done        : Boolean := False;
        need_redraw : Boolean := True;
        adapters    : Integer := 0;
            pragma Warnings( Off, adapters );
    begin
        if not Al_Initialize then
            Abort_Example( "Could not init Allegro." );
        end if;

        if not Al_Init_Image_Addon then
            Abort_Example( "Could not init Allegro IIO addon." );
        end if;

        Al_Init_Font_Addon;

        adapters := Al_Get_Num_Video_Adapters;

        Al_Get_Monitor_Info( 0, info );
        if info.x2 - info.x1 < w then
            w := info.x2 - info.x1;
        end if;
        if info.y2 - info.y1 < h then
            h := info.y2 - info.y1;
        end if;
        example.display := Al_Create_Display( w, h );
        if example.display = null then
            Abort_Example( "Error creating display." );
        end if;

        if not Al_Install_Keyboard then
            Abort_Example( "Could not install keyboard." );
        end if;

        if not Al_Install_Mouse then
            Abort_Example( "Could not install mouse." );
        end if;

        example.font := Al_Load_Font( "data/fixed_font.tga", 0, 0 );
        if example.font = null then
            Abort_Example( "Error loading data/fixed_font.tga" );
        end if;

        example.mysha := Al_Load_Bitmap( "data/mysha256x256.png" );
        if example.mysha = null then
            Abort_Example( "Error loading data/mysha256x256.png" );
        end if;

        example.white := Al_Map_RGB_f( 1.0, 1.0, 1.0 );
        example.half_white := Al_Map_RGBA_f( 1.0, 1.0, 1.0, 0.5 );
        example.dark := Al_Map_RGB( 15, 15, 15 );
        example.red := Al_Map_RGB_f( 1.0, 0.2, 0.1 );
        Change_Size( 256 );
        Add_Sprite;
        Add_Sprite;

        timer := Al_Create_Timer( 1.0 / double(FPS) );

        queue := Al_Create_Event_Queue;
        Al_Register_Event_Source( queue, Al_Get_Keyboard_Event_Source );
        Al_Register_Event_Source( queue, Al_Get_Mouse_Event_Source );
        Al_Register_Event_Source( queue, Al_Get_Timer_Event_Source( timer ) );
        Al_Register_Event_Source( queue, Al_Get_Display_Event_Source( example.display ) );

        Al_Start_Timer( timer );

        while not done loop
            if need_redraw and then Al_Is_Event_Queue_Empty( queue ) then
                declare
                    t : double := -Al_Get_Time;
                begin
                    Add_Time;
                    Al_Clear_To_Color( Al_Map_RGB_f( 0.0, 0.0, 0.0) );
                    Redraw;
                    t := t + Al_Get_Time;
                    example.direct_speed_measure := t;
                    Al_Flip_Display;
                    need_redraw := False;
                end;
            end if;

            Al_Wait_For_Event( queue, event );
            case event.any.typ is
                when ALLEGRO_EVENT_KEY_CHAR => -- includes repeats
                    case event.keyboard.keycode is
                        when ALLEGRO_KEY_ESCAPE =>
                            done := True;
                        when ALLEGRO_KEY_UP =>
                            Add_Sprites( 1 );
                        when ALLEGRO_KEY_DOWN =>
                            Remove_Sprites( 1 );
                        when ALLEGRO_KEY_LEFT =>
                            Change_Size( example.bitmap_size - 1 );
                        when ALLEGRO_KEY_RIGHT =>
                            Change_Size( example.bitmap_size + 1 );
                        when ALLEGRO_KEY_F1 =>
                            example.show_help := not example.show_help;
                        when ALLEGRO_KEY_SPACE =>
                            example.use_memory_bitmaps := not example.use_memory_bitmaps;
                            Change_Size( example.bitmap_size );
                        when ALLEGRO_KEY_B =>
                            example.blending := (example.blending + 1) mod 4;
                        when others =>
                            null;
                    end case;

                when ALLEGRO_EVENT_DISPLAY_CLOSE =>
                    done := True;

                when ALLEGRO_EVENT_TIMER =>
                    Update;
                    need_redraw := True;

                when ALLEGRO_EVENT_MOUSE_BUTTON_DOWN =>
                    example.mouse_down := True;
                    example.last_x := event.mouse.x;
                    example.last_y := event.mouse.y;

                when ALLEGRO_EVENT_MOUSE_BUTTON_UP =>
                    declare
                        fh     : constant Integer := Al_Get_Font_Line_Height( example.font );
                        button : Integer;
                    begin
                        example.mouse_down := False;
                        if event.mouse.x < 40 and then event.mouse.y >= h - fh * 5 then
                            button := (event.mouse.y - (h - fh * 5)) / fh;
                            if button = 0 then
                                example.use_memory_bitmaps := not example.use_memory_bitmaps;
                                Change_Size( example.bitmap_size );
                            end if;
                            if button = 1 then
                                example.blending := (example.blending + 1) mod 4;
                            end if;
                            if button = 4 then
                                example.show_help := not example.show_help;
                            end if;
                        end if;
                    end;

                when ALLEGRO_EVENT_MOUSE_AXES =>
                    if example.mouse_down then
                        declare
                            dx : constant Integer := event.mouse.x - example.last_x;
                            dy : constant Integer := event.mouse.y - example.last_y;
                        begin
                            if dy > 4 then
                                Add_Sprites( dy / 4 );
                            elsif dy < -4 then
                                Remove_Sprites( -dy / 4 );
                            end if;
                            if dx > 4 then
                                Change_Size( example.bitmap_size + dx - 4 );
                            elsif dx < -4 then
                                Change_Size( example.bitmap_size + dx + 4 );
                            end if;

                            example.last_x := event.mouse.x;
                            example.last_y := event.mouse.y;
                        end;
                    end if;

                when others =>
                    null;

            end case;
        end loop;

        Al_Destroy_Event_Queue( queue );
        Al_Destroy_Timer( timer );
        Al_Destroy_Bitmap( example.mysha );
        Al_Destroy_Font( example.font );
        AL_Destroy_Display( example.display );
        Al_Destroy_Bitmap( example.bitmap );
    end Ada_Main;

begin
    Set_Ada_Main( Ada_Main'Access );
end Ex_Draw_Bitmap;
