
with Ada.Command_Line;                  use Ada.Command_Line;
with Ada.Text_IO;                       use Ada.Text_IO;
with Allegro;                           use Allegro;
with Allegro.Audio;                     use Allegro.Audio;
with Allegro.Audio.Codecs;              use Allegro.Audio.Codecs;
with Allegro.System;                    use Allegro.System;
with Allegro.Time;                      use Allegro.Time;
with Interfaces;                        use Interfaces;
with Interfaces.C;                      use Interfaces.C;

-- Shows the ability to play a sample without a mixer.
package body Ex_Kcm_Direct is

    procedure Ada_Main is
        voice  : A_Allegro_Voice;
        sample : A_Allegro_Sample_Instance;
    begin
        if Argument_Count < 1 then
            Put_Line( "Usage: ex_kcm_direct.exe {audio_files}" );
            return;
        end if;

        if not Al_Initialize then
          Put_Line( "Could not init Allegro." );
          return;
        end if;

        if not Al_Init_Acodec_Addon then
            Put_Line( "Could not init audio codecs" );
        end if;

        if not Al_Install_Audio then
            Put_Line( "Could not init sound!" );
            return;
        end if;

        for i in 1..Argument_Count loop
            declare
                sample_data : A_Allegro_Sample;
                filename    : constant String := Argument( i );
                chan        : Allegro_Channel_Conf;
                depth       : Allegro_Audio_Depth;
                freq        : unsigned;
                sample_time : Float := 0.0;

            begin
                -- Load the entire sound file from disk.
                sample_data := Al_Load_Sample( filename );
                if sample_data = null then
                    Put_Line( "Could not load sample from '" & filename & "'!" );
                else

                    sample := Al_Create_Sample_Instance( null );
                    if sample = null then
                        Put_Line( "al_create_sample failed." );
                        return;
                    end if;

                    if not Al_Set_Sample( sample, sample_data ) then
                        Put_Line( "al_set_sample failed." );
                    else
                        depth := Al_Get_Sample_Instance_Depth( sample );
                        chan := Al_Get_Sample_Instance_Channels( sample );
                        freq := Al_Get_Sample_Instance_Frequency( sample );
                        if depth < ALLEGRO_AUDIO_DEPTH_UINT8 then
                            Put_Line( "Loaded sample:" & Integer(8 + Allegro_Audio_Depth'Pos( depth ) * 8)'Img &
                                      "-bit depth, " & chan'Img & " channels,"
                                      & freq'Img & " Hz" );
                        else
                            Put_Line( "Loaded sample: 0-bit depth, " & chan'Img &
                                      " channels," & freq'Img & " Hz" );
                        end if;
                        Put_Line( "Trying to create a voice with the same specs... " );
                        voice := Al_Create_Voice( freq, depth, chan );
                        if voice = null then
                            Put_Line( "Could not create ALLEGRO_VOICE." );
                            return;
                        end if;
                        Put_Line( "done." );

                        if not Al_Attach_Sample_Instance_To_Voice( sample, voice ) then
                            Put_Line( "al_attach_sample_instance_to_voice failed." );
                            return;
                        end if;

                        -- Play sample in looping mode.
                        if not Al_Set_Sample_Instance_Playmode( sample, ALLEGRO_PLAYMODE_LOOP ) then
                            null;
                        end if;
                        if not Al_Play_Sample_Instance( sample ) then
                            null;
                      end if;

                        sample_time := Al_Get_Sample_Instance_Time( sample );
                        Put_Line( "Playing '" & filename & "' (" & sample_time'Img & " seconds) 3 times" );

                        Al_Rest( double(sample_time * 3.0) );

                        if not Al_Stop_Sample_Instance( sample ) then
                            null;
                        end if;
                        Put_Line( "" );

                        -- Free the memory allocated.
                        if not Al_Set_Sample( sample, null ) then
                            null;
                        end if;
                        Al_Destroy_Sample( sample_data );
                        Al_Destroy_Sample_Instance( sample );
                        Al_Destroy_Voice( voice );
                    end if;
                end if;
            end;
        end loop;

        Al_Uninstall_Audio;
    end Ada_Main;

begin
    Set_Ada_Main( Ada_Main'Access );
end Ex_Kcm_Direct;
