
with Ada.Command_Line;                  use Ada.Command_Line;
with Ada.Strings.Unbounded;             use Ada.Strings.Unbounded;
with Allegro;                           use Allegro;
with Allegro.Bitmaps;                   use Allegro.Bitmaps;
with Allegro.Color;                     use Allegro.Color;
with Allegro.Configuration;             use Allegro.Configuration;
with Allegro.Displays;                  use Allegro.Displays;
with Allegro.Drawing;                   use Allegro.Drawing;
with Allegro.Drawing.Primitives;        use Allegro.Drawing.Primitives;
with Allegro.Events;                    use Allegro.Events;
with Allegro.Fonts;                     use Allegro.Fonts;
with Allegro.Fonts.TTF;                 use Allegro.Fonts.TTF;
with Allegro.Keyboard;                  use Allegro.Keyboard;
with Allegro.Mouse;                     use Allegro.Mouse;
with Allegro.System;                    use Allegro.System;
with Allegro.Time;                      use Allegro.Time;
with Allegro.Timers;                    use Allegro.Timers;
with Allegro.UTF8;                      use Allegro.UTF8;
with Common;                            use Common;
with Interfaces;                        use Interfaces;
with Interfaces.C;                      use Interfaces.C;

package body Ex_TTF is

        procedure Ada_Main is

        type Example is
            record
                fps                : double;
                f1, f2, f3, f4, f5 : A_Allegro_Font;
                config             : A_Allegro_Config;
            end record;

        ex : Example;

        ------------------------------------------------------------------------

        function Get_String( key : String ) return String is
            cfg : constant String := Al_Get_Config_Value( ex.config, "", key );
        begin
            if cfg'Length > 0 then
                return cfg;
            else
                return key;
            end if;
        end Get_String;

        ------------------------------------------------------------------------

        procedure Render is
            white      : constant Allegro_Color := Al_Map_RGBA_f( 1.0, 1.0, 1.0, 1.0 );
            black      : constant Allegro_Color := Al_Map_RGBA_f( 0.0, 0.0, 0.0, 1.0 );
            red        : constant Allegro_Color := Al_Map_RGBA_f( 1.0, 0.0, 0.0, 1.0 );
            green      : constant Allegro_Color := Al_Map_RGBA_f( 0.0, 0.5, 0.0, 1.0 );
            blue       : constant Allegro_Color := Al_Map_RGBA_f( 0.1, 0.2, 1.0, 1.0 );
            x, y,
            w, h,
            as, de,
            xpos, ypos : Float;
            target_w,
            target_h   : Integer;
            info,
            sub_info   : aliased Allegro_Ustr_Info;
            u          : A_Allegro_Ustr;
        begin
            Al_Clear_To_Color( white );

            Al_Hold_Bitmap_Drawing( True );

            Al_Draw_Text( ex.f1, black, 50.0,  50.0, 0, "Tulip (kerning)" );
            Al_Draw_Text( ex.f2, black, 50.0, 100.0, 0, "Tulip (no kerning)" );
            Al_Draw_Text( ex.f3, black, 50.0, 200.0, 0, "This font has a size of 12 pixels, the one above has 48 pixels." );

            Al_Hold_Bitmap_Drawing( False );
            Al_Hold_Bitmap_Drawing( True );

            Al_Draw_Text( ex.f3, red, 50.0, 220.0, 0, "The color can simply be changed." );

            Al_Hold_Bitmap_Drawing( False );
            Al_Hold_Bitmap_Drawing( True );

            Al_Draw_Text( ex.f3, green, 50.0, 240.0, 0, "Some unicode symbols:" );
            Al_Draw_Text( ex.f3, green, 50.0, 260.0, 0, Get_String( "symbols1" ) );
            Al_Draw_Text( ex.f3, green, 50.0, 280.0, 0, Get_String( "symbols2" ) );
            Al_Draw_Text( ex.f3, green, 50.0, 300.0, 0, Get_String( "symbols3" ) );

            u := Al_Ref_Cstr( info'Access, Get_String( "substr1" ) );
            Al_Draw_Ustr( ex.f3, green, 50.0, 320.0, 0, Al_Ref_Ustr( sub_info'Access, u, Al_Ustr_Offset( u, 0 ), Al_Ustr_Offset( u, 6 ) ) );
            u := Al_Ref_Cstr( info'Access, Get_String( "substr2" ) );
            Al_Draw_Ustr( ex.f3, green, 50.0, 340.0, 0, Al_Ref_Ustr( sub_info'Access, u, Al_Ustr_Offset( u, 7 ), Al_Ustr_Offset( u, 11 ) ) );
            u := Al_Ref_Cstr( info'Access, Get_String( "substr3" ) );
            Al_Draw_Ustr( ex.f3, green, 50.0, 360.0, 0, Al_Ref_Ustr( sub_info'Access, u, Al_Ustr_Offset( u, 4 ), Al_Ustr_Offset( u, 11 ) ) );
            u := Al_Ref_Cstr( info'Access, Get_String( "substr4" ) );
            Al_Draw_Ustr( ex.f3, green, 50.0, 380.0, 0, Al_Ref_Ustr( sub_info'Access, u, Al_Ustr_Offset( u, 0 ), Al_Ustr_Offset( u, 11 ) ) );

            Al_Draw_Text( ex.f5, black, 50.0, 420.0, 0, "forced monochrome" );

            Al_Hold_Bitmap_Drawing( False );

            target_w := Al_Get_Bitmap_Width( Al_Get_Target_Bitmap );
            target_h := Al_Get_Bitmap_Height( Al_Get_Target_Bitmap );

            xpos := Float(target_w) - 10.0;
            ypos := Float(target_h) - 10.0;
            declare
                xi, yi, wi, hi : Integer;
            begin
                Al_Get_Text_Dimensions( ex.f4, "Allegro", xi, yi, wi, hi );
                x := Float(xi);
                y := Float(yi);
                w := Float(wi);
                h := Float(hi);
            end;
            as := Float(Al_Get_Font_Ascent( ex.f4 ));
            de := Float(Al_Get_Font_Descent( ex.f4 ));
            xpos := xpos - w;
            ypos := ypos - h;
            x := x + xpos;
            y := y + ypos;

            Al_Draw_Rectangle( x, y, x + w, y + h, black, 0.0 );
            Al_Draw_Line( x, y + as, x + w, y + as, black, 0.0 );
            Al_Draw_Line( x, y + as + de, x + w, y + as + de, black, 0.0 );

            Al_Hold_Bitmap_Drawing( True );
            Al_Draw_Text( ex.f4, blue, xpos, ypos, 0, "Allegro" );
            Al_Hold_Bitmap_Drawing( False );

            Al_Hold_Bitmap_Drawing( True );

            Al_Draw_Text( ex.f3, black, Float(target_w), 0.0, ALLEGRO_ALIGN_RIGHT,
                          ex.fps'Img & " FPS" );

            Al_Hold_Bitmap_Drawing( False );
        end Render;

        ------------------------------------------------------------------------

        font_file : Unbounded_String := To_Unbounded_String( "data/DejaVuSans.ttf" );
        display   : A_Allegro_Display;
        timer     : A_Allegro_Timer;
        queue     : A_Allegro_Event_Queue;
        redraw    : Integer := 0;
        t         : double := 0.0;
        dt        : double;
        event     : Allegro_Event;
    begin
        if not Al_Initialize then
            Abort_Example( "Could not init Allegro." );
        end if;

        if not Al_Init_Primitives_Addon then
            Abort_Example( "Could not init primitives addon." );
        end if;
        if not Al_Install_Mouse then
            Abort_Example( "Could not install mouse." );
        end if;
        Al_Init_Font_Addon;
        if not Al_Init_TTF_Addon then
            Abort_Example( "Could not init TTF addon." );
        end if;

        display := Al_Create_Display( 640, 480 );
        if display = null then
            Abort_Example( "Could not create display." );
        end if;
        if not Al_Install_Keyboard then
            Abort_Example( "Could not install keyboard." );
        end if;

        if Argument_Count = 1 then
            font_file := To_Unbounded_String( Argument(1) );
        end if;

        ex.f1 := Al_Load_Font( To_String( font_file ), 48, 0 );
        ex.f2 := Al_Load_Font( To_String( font_file ), 48, ALLEGRO_TTF_NO_KERNING );
        ex.f3 := Al_Load_Font( To_String( font_file ), 12, 0 );
        -- Specifying negative values means we specify the glyph height
        -- in pixels, not the usual font size.
        --
        ex.f4 := Al_Load_Font( To_String( font_file ), -140, 0 );
        ex.f5 := Al_Load_Font( To_String( font_file ), 12, ALLEGRO_TTF_MONOCHROME );

        if ex.f1 = null or else
           ex.f2 = null or else
           ex.f3 = null or else
           ex.f4 = null
        then
            Abort_Example( "Could not load font: " & To_String( font_file ) );
        end if;

        ex.config := Al_Load_Config_File( "data/ex_ttf.ini" );
        if ex.config = null then
            Abort_Example( "Could not data/ex_ttf.ini" );
        end if;

        timer := Al_Create_Timer( 1.0 / 60.0 );

        queue := Al_Create_Event_Queue;
        Al_Register_Event_Source( queue, Al_Get_Keyboard_Event_Source );
        Al_Register_Event_Source( queue, Al_Get_Display_Event_Source( display ) );
        Al_Register_Event_Source( queue, Al_Get_Timer_Event_Source( timer ) );

        Al_Start_Timer( timer );
        loop
            Al_Wait_For_Event( queue, event );
            if event.any.typ = ALLEGRO_EVENT_DISPLAY_CLOSE then
                exit;
            elsif event.any.typ = ALLEGRO_EVENT_KEY_DOWN and then
                  event.keyboard.keycode = ALLEGRO_KEY_ESCAPE
            then
                exit;
            elsif event.any.typ = ALLEGRO_EVENT_TIMER then
                redraw := redraw + 1;
            end if;

            while redraw > 0 and then Al_Is_Event_Queue_Empty( queue ) loop
                redraw := redraw - 1;

                dt := Al_Get_Time;
                Render;
                dt := Al_Get_Time - dt;

                t := 0.99 * t + 0.01 * dt;

                ex.fps := 1.0 / t;
                Al_Flip_Display;
            end loop;
        end loop;

        Al_Destroy_Font( ex.f1 );
        Al_Destroy_Font( ex.f2 );
        Al_Destroy_Font( ex.f3 );
        Al_Destroy_Font( ex.f4 );
        Al_Destroy_Font( ex.f5 );
        Al_Destroy_Config( ex.config );
    end Ada_Main;

begin
    Set_Ada_Main( Ada_Main'Access );
end Ex_TTF;
