--
-- Copyright (c) 2013 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

-- Allegro 5.0.9 - Color addon
package Allegro.Color.Spaces is

    function Al_Get_Allegro_Color_Version return Unsigned_32;
    pragma Import( C, Al_Get_Allegro_Color_Version, "al_get_allegro_color_version" );

    -- CMYK - cyan, magenta, yellow, key

    function Al_Color_CMYK( c, m, y, k : Float ) return Allegro_Color;
    pragma Import( C, Al_Color_CMYK, "al_color_cmyk" );

    procedure Al_Color_CMYK_To_RGB( cyan, magenta, yellow, key : Float;
                                    red, green, blue           : in out Float );
    pragma Import( C, Al_Color_CMYK_To_RGB, "al_color_cmyk_to_rgb" );

    -- HSL - hue, saturation, lightness

    function Al_Color_HSL( h, s, l : Float ) return Allegro_Color;
    pragma Import( C, Al_Color_HSL, "al_color_hsl" );

    procedure Al_Color_HSL_To_RGB( hue, saturation, lightness : Float;
                                   red, green, blue           : in out Float );
    pragma Import( C, Al_Color_HSL_To_RGB, "al_color_hsl_to_rgb" );

    -- HSV - hue, saturation, value

    function Al_Color_HSV( h, s, v : Float ) return Allegro_Color;
    pragma Import( C, Al_Color_HSV, "al_color_hsv" );

    procedure Al_Color_HSV_To_RGB( hue, saturation, value : Float;
                                   red, green, blue       : in out Float );
    pragma Import( C, Al_Color_HSV_To_RGB, "al_color_hsv_to_rgb" );

    -- HTML colors

    function Al_Color_HTML( str : String ) return Allegro_Color;

    procedure Al_Color_HTML_To_RGB( str : String; red, green, blue : in out Float );

    function Al_Color_RGB_To_HTML( red, green, blue : Float ) return String;

    -- Color names

    function Al_Color_Name( name : String ) return Allegro_Color;

    function Al_Color_Name_To_RGB( name : String; r, g, b : access Float ) return Boolean;

    -- RGB - red, green, blue

    procedure Al_Color_RGB_To_CMYK( red, green, blue           : Float;
                                    cyan, magenta, yellow, key : in out Float );
    pragma Import( C, Al_Color_RGB_To_CMYK, "al_color_rgb_to_cmyk" );

    procedure Al_Color_RGB_To_HSL( red, green, blue           : Float;
                                   hue, saturation, lightness : in out Float );
    pragma Import( C, Al_Color_RGB_To_HSL, "al_color_rgb_to_hsl" );

    procedure Al_Color_RGB_To_HSV( red, green, blue       : Float;
                                   hue, saturation, value : in out Float );
    pragma Import( C, Al_Color_RGB_To_HSV, "al_color_rgb_to_hsv" );

    function Al_Color_RGB_To_Name( r, g, b : Float ) return String;

    procedure Al_Color_RGB_To_YUV( red, green, blue : Float;
                                   y, u, v          : in out Float );
    pragma Import( C, Al_Color_RGB_To_YUV, "al_color_rgb_to_yuv" );

    -- Y'UV - luminance, chrominance

    function Al_Color_YUV( y, u, v : Float ) return Allegro_Color;
    pragma Import( C, Al_Color_YUV, "al_color_yuv" );

    procedure Al_Color_YUV_To_RGB( y, u, v          : Float;
                                   red, green, blue : in out Float );
    pragma Import( C, Al_Color_YUV_To_RGB, "al_color_yuv_to_rgb" );

end Allegro.Color.Spaces;
