--
-- Copyright (c) 2013 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

package body Allegro.Drawing.Primitives is

    function Al_Init_Primitives_Addon return Boolean is

        function C_Al_Init_Primitives_Addon return Bool;
        pragma Import( C, C_Al_Init_Primitives_Addon, "al_init_primitives_addon" );

    begin
        return To_Ada( C_Al_Init_Primitives_Addon );
    end Al_Init_Primitives_Addon;

    ----------------------------------------------------------------------------

    procedure Al_Calculate_Arc( dest         : Point_2D_Array;
                                cx, cy       : Float;
                                rx, ry       : Float;
                                start_theta  : Float;
                                delta_theta  : Float;
                                thickness    : Float ) is
        segments : Integer;
    begin
        if thickness <= 0.0 then
            segments := dest'Length;
        else
            -- thickness > 0 requires twice as many points per segment
            segments := (dest'Length - 1) / 2;
        end if;
        Al_Calculate_Arc( dest(dest'First)'Address, Point_2D'Size / 8,
                          cx, cy, rx, ry, start_theta, delta_theta,
                          thickness, segments );
    end Al_Calculate_Arc;
    pragma Inline_Always( Al_Calculate_Arc );

    ----------------------------------------------------------------------------

    procedure Al_Calculate_Spline( dest      : Point_2D_Array;
                                   points    : Point_2D_Array;
                                   thickness : Float ) is
        segments : Integer;
    begin
        if thickness <= 0.0 then
            segments := dest'Length;
        else
            -- thickness > 0 requires twice as many points per segment
            segments := (dest'Length - 1) / 2;
        end if;
        Al_Calculate_Spline( dest(dest'First)'Address, Point_2D'Size / 8,
                             points(points'First)'Address, thickness, segments );
    end Al_Calculate_Spline;
    pragma Inline_Always( Al_Calculate_Spline );

    ----------------------------------------------------------------------------

    procedure Al_Draw_Spline( points    : Point_2D_Array;
                              color     : Allegro_Color;
                              thickness : Float ) is
    begin
        Al_Draw_Spline( points(points'First)'Address, color, thickness );
    end Al_Draw_Spline;
    pragma Inline_Always( Al_Draw_Spline );

    ----------------------------------------------------------------------------

    procedure Al_Calculate_Ribbon( dest      : Point_2D_Array;
                                   points    : Point_2D_Array;
                                   thickness : Float ) is
    begin
        Al_Calculate_Ribbon( dest(dest'First)'Address, Point_2D'Size / 8,
                             points(points'First)'Address, Point_2D'Size / 8,
                             thickness, points'Length );
    end Al_Calculate_Ribbon;
    pragma Inline_Always( Al_Calculate_Ribbon );

    ----------------------------------------------------------------------------

    procedure Al_Draw_Ribbon( points    : Point_2D_Array;
                              color     : Allegro_Color;
                              thickness : Float ) is
    begin
        Al_Draw_Ribbon( points(points'First)'Address, Point_2D'Size / 8,
                        color, thickness, points'Length );
    end Al_Draw_Ribbon;
    pragma Inline_Always( Al_Draw_Ribbon );

    ----------------------------------------------------------------------------

    function Al_Create_Vertex_Decl( elements : Allegro_Vertex_Element_Array;
                                    stride   : Positive ) return A_Allegro_Vertex_Decl is
        elems : aliased Allegro_Vertex_Element_Array(elements'First..elements'Last + 1);
    begin
        elems(elems'First..elems'Last - 1) := elements;

        -- zero out the last element to indicate the end of the array
        elems(elems'Last).attribute := Allegro_Prim_Attr'First;
        elems(elems'Last).storage := Allegro_Prim_Storage'First;
        elems(elems'Last).offset := 0;

        return Al_Create_Vertex_Decl( elems(elems'First)'Address, stride );
    end Al_Create_Vertex_Decl;

    ----------------------------------------------------------------------------

    procedure Al_Destroy_Vertex_Decl( decl : in out A_Allegro_Vertex_Decl ) is

        procedure C_Al_Destroy_Vertex_Decl( decl : A_Allegro_Vertex_Decl );
        pragma Import( C, C_Al_Destroy_Vertex_Decl, "al_destroy_vertex_decl" );

    begin
        if decl /= null then
            C_Al_Destroy_Vertex_Decl( decl );
            decl := null;
        end if;
    end Al_Destroy_Vertex_Decl;

    ----------------------------------------------------------------------------

    procedure Al_Draw_Prim( vertices : Allegro_Vertex_Array;
                            texture  : A_Allegro_Bitmap;
                            start    : Integer;
                            stop     : Integer;
                            primType : Allegro_Prim_Type ) is
    begin
        Al_Draw_Prim( vertices(vertices'First)'Address, null, texture, start,
                      stop, primType );
    end Al_Draw_Prim;

    ----------------------------------------------------------------------------

    function Al_Draw_Indexed_Prim( vertices : Address;
                                   decl     : A_Allegro_Vertex_Decl;
                                   texture  : A_Allegro_Bitmap;
                                   indices  : Vertex_Index_Array;
                                   primType : Allegro_Prim_Type ) return Integer is

        function C_Al_Draw_Indexed_Prim( vertices : Address;
                                         decl     : A_Allegro_Vertex_Decl;
                                         texture  : A_Allegro_Bitmap;
                                         indices  : Address;
                                         num_vtx  : Integer;
                                         primType : Allegro_Prim_Type ) return Integer;
        pragma Import( C, C_Al_Draw_Indexed_Prim, "al_draw_indexed_prim" );

    begin
        return C_Al_Draw_Indexed_Prim( vertices, decl, texture,
                                       indices(indices'First)'Address,
                                       indices'Length, primType );
    end Al_Draw_Indexed_Prim;

    ----------------------------------------------------------------------------

    procedure Al_Draw_Indexed_Prim( vertices : Address;
                                    decl     : A_Allegro_Vertex_Decl;
                                    texture  : A_Allegro_Bitmap;
                                    indices  : Vertex_Index_Array;
                                    primType : Allegro_Prim_Type ) is
    begin
        if Al_Draw_Indexed_Prim( vertices, decl, texture, indices, primType ) = 0 then
            null;
        end if;
    end Al_Draw_Indexed_Prim;

    ----------------------------------------------------------------------------

    procedure Al_Draw_Indexed_Prim( vertices : Allegro_Vertex_Array;
                                    texture  : A_Allegro_Bitmap;
                                    indices  : Vertex_Index_Array;
                                    primType : Allegro_Prim_Type ) is
    begin
        Al_Draw_Indexed_Prim( vertices(vertices'First)'Address, null, texture,
                              indices, primType );
    end Al_Draw_Indexed_Prim;

end Allegro.Drawing.Primitives;
