--
-- Copyright (c) 2013 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

with System;                            use System;

package body Allegro.Fonts is

    function Al_Load_Font( filename : String;
                           size     : Integer;
                           flags    : Font_Loading_Flags ) return A_Allegro_Font is

        function C_Al_Load_Font( filename : C.char_array;
                                 size     : Integer;
                                 flags    : Font_Loading_Flags ) return A_Allegro_Font;
        pragma Import( C, C_Al_Load_Font, "al_load_font" );

    begin
        return C_Al_Load_Font( To_C( filename ), size, flags );
    end Al_Load_Font;

    ----------------------------------------------------------------------------

    procedure Al_Destroy_Font( f : in out A_Allegro_Font ) is

        procedure C_Al_Destroy_Font( f : A_Allegro_Font );
        pragma Import( C, C_Al_Destroy_Font, "al_destroy_font" );

    begin
        if f /= null then
            C_Al_Destroy_Font( f );
            f := null;
        end if;
    end Al_Destroy_Font;

    ----------------------------------------------------------------------------

    function Al_Register_Font_Loader( ext : String; load : A_Loader_Proc ) return Boolean is

        function C_Al_Register_Font_Loader( ext  : C.char_array;
                                            load : A_Loader_Proc ) return Bool;
        pragma Import( C, C_Al_Register_Font_Loader, "al_register_font_loader" );

    begin
        return To_Ada( C_Al_Register_Font_Loader( To_C( ext ), load ) );
    end Al_Register_Font_Loader;

    ----------------------------------------------------------------------------

    function Al_Get_Text_Width( f : A_Allegro_Font; str : String ) return Integer is

        function C_Al_Get_Text_Width( f : A_Allegro_Font; str : C.char_array ) return Integer;
        pragma Import( C, C_Al_Get_Text_Width, "al_get_text_width" );

    begin
        return C_Al_Get_Text_Width( f, To_C( str ) );
    end Al_Get_Text_Width;

    ----------------------------------------------------------------------------

    procedure Al_Draw_Text( font  : A_Allegro_Font;
                            color : Allegro_Color;
                            x, y  : Float;
                            flags : Font_Draw_Flags;
                            text  : String ) is

        procedure C_Al_Draw_Text( font  : A_Allegro_Font;
                                  color : Allegro_Color;
                                  x, y  : Float;
                                  flags : Font_Draw_Flags;
                                  text  : C.char_array );
        pragma Import( C, C_Al_Draw_Text, "al_draw_text" );

    begin
        C_Al_Draw_Text( font, color, x, y, flags, To_C( text ) );
    end Al_Draw_Text;

    ----------------------------------------------------------------------------

    procedure Al_Draw_Justified_Text( font   : A_Allegro_Font;
                                      color  : Allegro_Color;
                                      x1, x2 : Float;
                                      y      : Float;
                                      diff   : Float;
                                      flags  : Font_Draw_Flags;
                                      text   : String ) is

        procedure C_Al_Draw_Justified_Text( font   : A_Allegro_Font;
                                  color  : Allegro_Color;
                                  x1, x2 : Float;
                                  y      : Float;
                                  diff   : Float;
                                  flags  : Font_Draw_Flags;
                                  text   : C.char_array );
        pragma Import( C, C_Al_Draw_Justified_Text, "al_draw_justified_text" );

    begin
        C_Al_Draw_Justified_Text( font, color, x1, x2, y, diff, flags, To_C( text ) );
    end Al_Draw_Justified_Text;

    ----------------------------------------------------------------------------

    procedure Al_Get_Text_Dimensions( f    : A_Allegro_Font;
                                      text : String;
                                      bbx  : out Integer;
                                      bby  : out Integer;
                                      bbw  : out Integer;
                                      bbh  : out Integer ) is

        procedure C_Al_Get_Text_Dimensions( f    : A_Allegro_Font;
                                            text : C.char_array;
                                            bbx  : access Integer;
                                            bby  : access Integer;
                                            bbw  : access Integer;
                                            bbh  : access Integer );
        pragma Import( C, C_Al_Get_Text_Dimensions, "al_get_text_dimensions" );

        abbx, abby, abbw, abbh : aliased Integer;
    begin
        C_Al_Get_Text_Dimensions( f, To_C( text ), abbx'Access, abby'Access,
                                  abbw'Access, abbh'Access );
        bbx := abbx;
        bby := abby;
        bbw := abbw;
        bbh := abbh;
    end Al_Get_Text_Dimensions;

    ----------------------------------------------------------------------------

    procedure Al_Get_Ustr_Dimensions( f    : A_Allegro_Font;
                                      text : A_Allegro_Ustr;
                                      bbx  : out Integer;
                                      bby  : out Integer;
                                      bbw  : out Integer;
                                      bbh  : out Integer ) is

        procedure C_Al_Get_Ustr_Dimensions( f    : A_Allegro_Font;
                                            text : A_Allegro_Ustr;
                                            bbx  : access Integer;
                                            bby  : access Integer;
                                            bbw  : access Integer;
                                            bbh  : access Integer );
        pragma Import( C, C_Al_Get_Ustr_Dimensions, "al_get_ustr_dimensions" );

        abbx, abby, abbw, abbh : aliased Integer;
    begin
        C_Al_Get_Ustr_Dimensions( f, text, abbx'Access, abby'Access, abbw'Access,
                                  abbh'Access );
        bbx := abbx;
        bby := abby;
        bbw := abbw;
        bbh := abbh;
    end Al_Get_Ustr_Dimensions;

    ----------------------------------------------------------------------------

    function Al_Grab_Font_From_Bitmap( bmp    : A_Allegro_Bitmap;
                                       ranges : Unicode_Range_Array
                                     ) return A_Allegro_Font is

        function C_Al_Grab_Font_From_Bitmap( bmp    : A_Allegro_Bitmap;
                                             n      : Integer;
                                             ranges : Address ) return A_Allegro_Font;
        pragma Import( C, C_Al_Grab_Font_From_Bitmap, "al_grab_font_from_bitmap" );

    begin
        return C_Al_Grab_Font_From_Bitmap( bmp, ranges'Length, ranges(ranges'First)'Address );
    end Al_Grab_Font_From_Bitmap;

    ----------------------------------------------------------------------------

    function Al_Load_Bitmap_Font( filename : String ) return A_Allegro_Font is

        function C_Al_Load_Bitmap_Font( filename : C.char_array ) return A_Allegro_Font;
        pragma Import( C, C_Al_Load_Bitmap_Font, "al_load_bitmap_font" );

    begin
        return C_Al_Load_Bitmap_Font( To_C( filename ) );
    end Al_Load_Bitmap_Font;

end Allegro.Fonts;
