--
-- Copyright (c) 2013 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

with Interfaces.C;                      use Interfaces.C;
with Interfaces.C.Strings;              use Interfaces.C.Strings;
with System;                            use System;

package body Allegro.Native_Dialogs is

    function Al_Init_Native_Dialog_Addon return Boolean is

        function C_Al_Init_Native_Dialog_Addon return Bool;
        pragma Import( C, C_Al_Init_Native_Dialog_Addon, "al_init_native_dialog_addon" );

    begin
        return To_Ada( C_Al_Init_Native_Dialog_Addon );
    end Al_Init_Native_Dialog_Addon;

    ----------------------------------------------------------------------------

    function Al_Create_Native_File_Dialog( initial_path : String;
                                           title        : String;
                                           patterns     : String;
                                           mode         : Filechooser_Flags
                                         ) return A_Allegro_Filechooser is

        function C_Al_Create_Native_File_Dialog( initial_path : C.char_array;
                                                 title        : C.char_array;
                                                 patterns     : C.char_array;
                                                 mode         : Filechooser_Flags
                                               ) return A_Allegro_Filechooser;
        pragma Import( C, C_Al_Create_Native_File_Dialog, "al_create_native_file_dialog" );

    begin
        return C_Al_Create_Native_File_Dialog( To_C( initial_path ),
                                               To_C( title ),
                                               To_C( patterns ),
                                               mode );
    end Al_Create_Native_File_Dialog;

    ----------------------------------------------------------------------------

    function Al_Show_Native_File_Dialog( display : A_Allegro_Display;
                                         dialog  : A_Allegro_Filechooser ) return Boolean is

        function C_Al_Show_Native_File_Dialog( display : A_Allegro_Display;
                                               dialog  : A_Allegro_Filechooser ) return Bool;
        pragma Import( C, C_Al_Show_Native_File_Dialog, "al_show_native_file_dialog" );

    begin
        return To_Ada( C_Al_Show_Native_File_Dialog( display, dialog ) );
    end Al_Show_Native_File_Dialog;

    ----------------------------------------------------------------------------

    function Al_Get_Native_File_Dialog_Path( dialog : A_Allegro_Filechooser;
                                             index  : Natural ) return String is

        function C_Al_Get_Native_File_Dialog_Path( dialog : A_Allegro_Filechooser;
                                                   index  : C.size_t
                                                 ) return C.Strings.chars_ptr;
        pragma Import( C, C_Al_Get_Native_File_Dialog_Path, "al_get_native_file_dialog_path" );

    begin
        return To_Ada( Value( C_Al_Get_Native_File_Dialog_Path( dialog, C.size_t(index) ) ) );
    end Al_Get_Native_File_Dialog_Path;

    ----------------------------------------------------------------------------

    procedure Al_Destroy_Native_File_Dialog( dialog : in out A_Allegro_Filechooser ) is

        procedure C_Al_Destroy_Native_File_Dialog( dialog : A_Allegro_Filechooser );
        pragma Import( C, C_Al_Destroy_Native_File_Dialog, "al_destroy_native_file_dialog" );

    begin
        if dialog /= null then
            C_Al_Destroy_Native_File_Dialog( dialog );
            dialog := null;
        end if;
    end Al_Destroy_Native_File_Dialog;

    ----------------------------------------------------------------------------

    function Al_Show_Native_Message_Box( display : A_Allegro_Display;
                                         title   : String;
                                         heading : String;
                                         text    : String;
                                         buttons : String;
                                         flags   : Messagebox_Flags ) return Integer is

        function C_Al_Show_Native_Message_Box( display : A_Allegro_Display;
                                               title   : C.char_array;
                                               heading : C.char_array;
                                               text    : C.char_array;
                                               buttons : C.char_array;
                                               flags   : Messagebox_Flags ) return Integer;
        pragma Import( C, C_Al_Show_Native_Message_Box, "al_show_native_message_box" );

        function C_Al_Show_Native_Message_Box_Simple( display : A_Allegro_Display;
                                                      title   : C.char_array;
                                                      heading : C.char_array;
                                                      text    : C.char_array;
                                                      buttons : Address;    -- pass null
                                                      flags   : Messagebox_Flags ) return Integer;
        pragma Import( C, C_Al_Show_Native_Message_Box_Simple, "al_show_native_message_box" );

    begin
        if buttons'Length > 0 then
            return C_Al_Show_Native_Message_Box( display, To_C( title ),
                                                 To_C( heading ), To_C( text ),
                                                 To_C( buttons ), flags );
        else
            return C_Al_Show_Native_Message_Box_Simple( display, To_C( title ),
                                                        To_C( heading ), To_C( text ),
                                                        Null_Address, flags );
        end if;
    end Al_Show_Native_Message_Box;

    ----------------------------------------------------------------------------

    function Al_Open_Native_Text_Log( title : String; flags : Textlog_Flags ) return A_Allegro_Textlog is

        function C_Al_Open_Native_Text_Log( title : C.char_array;
                                            flags : Textlog_Flags ) return A_Allegro_Textlog;
        pragma Import( C, C_Al_Open_Native_Text_Log, "al_open_native_text_log" );

    begin
        return C_Al_Open_Native_Text_Log( To_C( title ), flags );
    end Al_Open_Native_Text_Log;

    ----------------------------------------------------------------------------

    procedure Al_Close_Native_Text_Log( textlog : in out A_Allegro_Textlog ) is

        procedure C_Al_Close_Native_Text_Log( textlog : A_Allegro_Textlog );
        pragma Import( C, C_Al_Close_Native_Text_Log, "al_close_native_text_log" );

    begin
        if textlog /= null then
            C_Al_Close_Native_Text_Log( textlog );
            textlog := null;
        end if;
    end Al_Close_Native_Text_Log;

    ----------------------------------------------------------------------------

    procedure Al_Append_Native_Text_Log( textlog : A_Allegro_Textlog; str : String ) is

        procedure C_Al_Append_Native_Text_Log( textlog : A_Allegro_Textlog;
                                               str     : C.char_array );
        pragma Import( C, C_Al_Append_Native_Text_Log, "al_append_native_text_log_s" );

    begin
        C_Al_Append_Native_Text_Log( textlog, To_C( str ) );
    end Al_Append_Native_Text_Log;

end Allegro.Native_Dialogs;
