--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

with Ada.Text_IO;
with Allegro.System;                    use Allegro.System;
with Debugging;                         use Debugging;
with Engine_Debugging;                  use Engine_Debugging;
with Version;

pragma Elaborate_All( Debugging );

package body Applications.Shell is

    not overriding
    procedure Construct( this          : access Shell_Application;
                         company       : String;
                         name          : String;
                         configDir     : String;
                         enableAllegro : Boolean ) is
    begin
        Application(this.all).Construct( company, name, configDir );
        this.allegroEnabled := enableAllegro;
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure Finalize( this : access Shell_Application ) is
    begin
        if this.allegroEnabled and then Al_Is_System_Installed then
            pragma Debug( Dbg( "Shutting down Allegro", D_INIT, INFO ) );
            Al_Uninstall_System;
        end if;
        pragma Debug( Dbg( "Application finalized", D_INIT, INFO ) );

        Application(this.all).Finalize;
    end Finalize;

    ----------------------------------------------------------------------------

    function Get_Announce( this : not null access Shell_Application'Class ) return String is
    begin
        return this.Get_Name & " v" & Version.Release & " - " &
               this.Get_Company & " " &
               "Copyright (C) " & Version.Copyright_Year;
    end Get_Announce;

    ----------------------------------------------------------------------------

    overriding
    function Initialize( this : access Shell_Application ) return Boolean is
    begin
        if not Application(this.all).Initialize then
            return False;
        end if;

        if this.allegroEnabled and then not Al_Initialize then
            this.Show_Error( "Error initializing Allegro" );
            Dbg( "Error: Al_Initialize failed",  D_INIT, ERROR );
            raise INIT_EXCEPTION;
        end if;

        return True;
    exception
        when e : others =>
            Dbg( "Shell_Application.Initialize Exception: ...", D_INIT, ERROR );
            Dbg( e, D_INIT, ERROR );

            this.Finalize;
            return False;
    end Initialize;

begin

    -- Use Ada.Text_IO as the default debug output for shell applications
    -- This can be overridden by calling Set_Output in the elaboration section
    -- of a specific application's Project package body.
    Debugging.Set_Output( Ada.Text_IO.Put_Line'Access );

end Applications.Shell;
