--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

package Applications.Shell is

    -- An abstract command-line application class without a window.
    type Shell_Application is abstract new Application with private;
    type A_Shell_Application is access all Shell_Application'Class;

    -- Returns a string announcing the product. It is formatted as:
    -- [APPNAME] & " v[VERSION] - [COMPANY] Copyright (C) [BUILD_YEAR]
    function Get_Announce( this : not null access Shell_Application'Class ) return String;

private

    type Shell_Application is abstract new Application with
        record
            allegroEnabled : Boolean := False;      -- allegro library enabled?
        end record;

    -- Constructs a shell/command line application without a window. When
    -- Allegro is not enabled, the library will not be initialized and may only
    -- perform some basic tasks. Unless the application needs to take advantage
    -- of the power of Allegro (e.g. do image processing), set it to False.
    procedure Construct( this          : access Shell_Application;
                         company       : String;
                         name          : String;
                         configDir     : String;
                         enableAllegro : Boolean );

    -- Finalizes the application and releases all resources. Do not call this if
    -- the application didn't successfully initialize. This should be called
    -- last by an overriding implementation.
    procedure Finalize( this : access Shell_Application );

    -- Initializes the shell application without a graphics window. Returns True
    -- on success. If initialization fails, Finalize should not be called. No
    -- exceptions will be raised. This should be called first by an overriding
    -- implementation.
    function Initialize( this : access Shell_Application ) return Boolean;

end Applications.Shell;
