--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

with Entities.Players;                  use Entities.Players;
with Resources;                         use Resources;
with Tiles;                             use Tiles;

package body Entities.Enemies is

    not overriding
    procedure Construct( this     : access Enemy;
                         width,
                         height   : Natural;
                         libName  : String;
                         iconName : String ) is
    begin
        Entity(this.all).Construct( width, height, libName );
        this.icon := this.lib.Get_Id( iconName );
        this.frame := this.icon;
        if this.icon = 0 then
            raise RESOURCE_ERROR with "Icon <" & iconName & "> not found in " &
                                      "tile library <" & libName & ">";
        end if;
    end Construct;

    ----------------------------------------------------------------------------

    not overriding
    procedure Die( this : access Enemy ) is
        pragma Unreferenced( this );
    begin
        null;
    end Die;

    ----------------------------------------------------------------------------

    function Get_Icon( this : not null access Enemy'Class ) return A_Allegro_Bitmap is
        tile : constant A_Tile := this.lib.Get_Tile( this.icon );
        bmp  : A_Allegro_Bitmap := null;
    begin
        if tile /= null then
            if not tile.Is_Loaded then
                -- the icon exists in the tile library but the bitmap was not
                -- returned because the bitmaps have not been loaded. normally
                -- entities don't load the bitmaps with their libraries because
                -- they're only used by the game thread. however, if the
                -- entity's bitmap icon is requested of it, it's because the
                -- view is using the template entity instances in the entity
                -- factory to draw icons and the bitmaps need to be loaded.
                this.lib.Load_Bitmaps;
            end if;
            bmp := tile.Get_Bitmap;
        end if;
        return bmp;
    end Get_Icon;

    ----------------------------------------------------------------------------

    function Is_Shootable( this : not null access Enemy'Class ) return Boolean is
    begin
        return this.shootable;
    end Is_Shootable;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Read( stream : access Root_Stream_Type'Class; obj : out Enemy ) is
    begin
        Entity'Read( stream, Entity(obj) );
        obj.deadlyTouch := Boolean'Input( stream );
        obj.shootable := Boolean'Input( stream );
    end Object_Read;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Write( stream : access Root_Stream_Type'Class; obj : Enemy ) is
    begin
        Entity'Write( stream, Entity(obj) );
        Boolean'Output( stream, obj.deadlyTouch );
        Boolean'Output( stream, obj.shootable );
    end Object_Write;

    ----------------------------------------------------------------------------

    overriding
    procedure On_Collide( this : access Enemy; e : not null A_Entity ) is
    begin
        if e.all in Player'Class then
            if this.deadlyTouch and then not this.editingMode then
                A_Player(e).Die;
            end if;
        end if;
    end On_Collide;

    ----------------------------------------------------------------------------

    procedure Delete( this : in out A_Enemy ) is
    begin
        Delete( A_Object(this) );
    end Delete;

end Entities.Enemies;
