--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

with Entities.Players;                  use Entities.Players;
with Resources;                         use Resources;
with Tiles;                             use Tiles;

package body Entities.Items is

    not overriding
    procedure Construct( this       : access Item;
                         width,
                         height     : Natural;
                         libName    : String;
                         firstFrame : String;
                         maxFrames  : Positive;
                         frameDelay : Time_Span ) is
    begin
        Entity(this.all).Construct( width, height, libName );
        this.firstFrame := this.lib.Get_Id( firstFrame );
        if this.firstFrame = 0 then
            raise RESOURCE_ERROR with "Image <" & firstFrame & "> not found in " &
                                      "tile library <" & libName & ">";
        end if;
        this.maxFrames := maxFrames;
        this.frameDelay := frameDelay;
        this.frame := this.firstFrame;
        this.physical := False;
        this.clipped := False;
    end Construct;

    ----------------------------------------------------------------------------

    not overriding
    function Get_Icon( this : access Item ) return A_Allegro_Bitmap is
        tile : constant A_Tile := this.lib.Get_Tile( this.firstFrame );
        bmp  : A_Allegro_Bitmap := null;
    begin
        if tile /= null then
            if not tile.Is_Loaded then
                -- the icon exists in the tile library but the bitmap was not
                -- returned because the bitmaps have not been loaded. normally
                -- entities don't load the bitmaps with their libraries because
                -- they're only used by the game thread. however, if the
                -- entity's bitmap icon is requested of it, it's because the
                -- view is using the template entity instances in the entity
                -- factory to draw icons and the bitmaps need to be loaded.
                this.lib.Load_Bitmaps;
            end if;
            bmp := tile.Get_Bitmap;
        end if;
        return bmp;
    end Get_Icon;

    ----------------------------------------------------------------------------

    overriding
    procedure On_Collide( this : access Item; e : not null A_Entity ) is
    begin
        if e.all in Player'Class and then not this.editingMode then
            A_Item(this).Give_Item;
        end if;
    end On_Collide;

    ----------------------------------------------------------------------------

    overriding
    procedure Update( this : access Item; time : Tick_Time ) is
    begin
        -- Tick is overridden here instead of Update_Frame because access to
        -- 'time.total' is required.
        if this.maxFrames > 1 then
            this.Set_Frame( this.firstFrame + ((time.total / this.frameDelay) mod this.maxFrames) );
        end if;
    end Update;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Read( stream : access Root_Stream_Type'Class; obj : out Item ) is
    begin
        Entity'Read( stream, Entity(obj) );
    end Object_Read;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Write( stream : access Root_Stream_Type'Class; obj : Item ) is
    begin
        Entity'Write( stream, Entity(obj) );
    end Object_Write;

end Entities.Items;
