--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

--with Debugging;                         use Debugging;
with Events.Entities;                   use Events.Entities;

package body Entities.Sprites is

    not overriding
    procedure Construct( this       : access Sprite;
                         width,
                         height     : Natural;
                         libName    : String;
                         frames     : Tile_Id_Array;
                         lifeSpan   : Time_Span := Time_Span_Zero;
                         frameDelay : Time_Span := Time_Span_Zero ) is
    begin
        Entity(this.all).Construct( width, height, libName );
        this.clipped := False;
        this.lifeSpan := lifeSpan;
        this.frameDelay := frameDelay;
        this.frames := new Tile_Id_Array'(frames);
        this.frame := this.frames(this.frames'First);
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure Delete( this : in out Sprite ) is
    begin
        Delete( this.frames );
        Entity(this).Delete;
    end Delete;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Read( stream : access Root_Stream_Type'Class; obj : out Sprite ) is
    begin
        Entity'Read( stream, Entity(obj) );
        obj.lifeSpan := Time_Span'Input( stream );
        obj.frameDelay := Time_Span'Input( stream );
        Delete( obj.frames );
        obj.frames := A_Tile_Id_Array'Input( stream );
    end Object_Read;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Write( stream : access Root_Stream_Type'Class; obj : Sprite ) is
    begin
        Entity'Write( stream, Entity(obj) );
        Time_Span'Output( stream, obj.lifeSpan );
        Time_Span'Output( stream, obj.frameDelay );
        A_Tile_Id_Array'Output( stream, obj.frames );
    end Object_Write;

    ----------------------------------------------------------------------------

    overriding
    procedure Update( this : access Sprite; time : Tick_Time ) is
        pragma Unreferenced( time );
    begin
        if this.lifeSpan > Time_Span_Zero and then this.age >= this.lifeSpan then
            Queue_Delete_Entity( this.id );
        end if;
    end Update;

    ----------------------------------------------------------------------------

    overriding
    procedure Update_Frame( this   : access Sprite;
                            notify : Boolean := True ) is
    begin
        if this.frameDelay /= Time_Span_Zero then
            -- the frame changes every frameDelay in a looped sequence
            this.Set_Frame( this.frames(this.frames'First +
                                        (this.age / this.frameDelay) mod this.frames'Length),
                            notify );
        elsif this.lifeSpan /= Time_Span_Zero then
            -- each frame is displayed once over the lifespan of the sprite
            this.Set_Frame( this.frames(this.frames'First +
                                        Integer'Min( (this.age / (this.lifeSpan / this.frames'Length)),
                                                     this.frames'Length - 1 )),
                            notify );
        else
            -- no animation without lifeSpan or frameDelay
            null;
        end if;
    end Update_Frame;

end Entities.Sprites;
