--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

with Events;

pragma Elaborate_All( Events );

package Events.Application is

    -- A notification that the application window has lost focus.
    APP_BLUR_ID : constant Event_Id := To_Event_Id( "App_Blur" );

    -- A notification that the application window has regained focus.
    APP_FOCUS_ID : constant Event_Id := To_Event_Id( "App_Focus" );

    -- A notification that the user is attempting to close the window of the
    -- application. This does not have to be acted on, for instance, if the
    -- player hasn't saved his game yet.
    CLOSE_WINDOW_ID : constant Event_Id := To_Event_Id( "Close_Window" );

    -- A command to close the application, no questions asked. This event is
    -- queued when it's time for the application to quit.
    EXIT_APPLICATION_ID  : constant Event_Id := To_Event_Id( "Exit_Application" );

    ----------------------------------------------------------------------------

    -- Queues an App_Blur_Event.
    procedure Queue_App_Blur;

    -- Queues an App_Focus_Event.
    procedure Queue_App_Focus;

    -- Queues a Close_Window_Event.
    procedure Queue_Close_Window;

    -- Queues a Exit_Application_Event.
    procedure Queue_Exit_Application;

end Events.Application;
