--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

with Events;

pragma Elaborate_All( Events );

package Events.Audio is

    PLAY_MUSIC_ID : constant Event_Id := To_Event_Id( "Play_Music" );

    -- A command to begin playing a looped music track.
    type Play_Music_Event is new Event with private;
    type A_Play_Music_Event is access all Play_Music_Event'Class;

    -- Returns the filename of the music to play.
    function Get_Music_Name( this : not null access Play_Music_Event'Class ) return String;
    pragma Postcondition( Get_Music_Name'Result'Length > 0 );

    ----------------------------------------------------------------------------

    PLAY_SOUND_ID : constant Event_Id := To_Event_Id( "Play_Sound" );

    -- A command to play a sound effect once through.
    type Play_Sound_Event is new Event with private;
    type A_Play_Sound_Event is access all Play_Sound_Event'Class;

    -- Returns the name of the sound effect to play.
    function Get_Sound_Name( this : not null access Play_Sound_Event'Class ) return String;
    pragma Postcondition( Get_Sound_Name'Result'Length > 0 );

    ----------------------------------------------------------------------------

    -- A command to stop any music currently playing.
    STOP_MUSIC_ID : constant Event_Id := To_Event_Id( "Stop_Music" );

    ----------------------------------------------------------------------------

    -- Queues a Play_Music_Event.
    procedure Queue_Play_Music( name : String );
    pragma Precondition( name'Length > 0 );

    -- Queues a Play_Sound_Event.
    procedure Queue_Play_Sound( name : String );
    pragma Precondition( name'Length > 0 );

    -- Queues a Stop_Music_Event.
    procedure Queue_Stop_Music;

private

    type Play_Sound_Event is new Event with
        record
            name : Unbounded_String;
        end record;

    procedure Construct( this : access Play_Sound_Event; name : String );
    pragma Precondition( name'Length > 0 );

    function To_String( this : access Play_Sound_Event ) return String;

    ----------------------------------------------------------------------------

    type Play_Music_Event is new Event with
        record
            name : Unbounded_String;
        end record;

    procedure Construct( this : access Play_Music_Event; name : String );
    pragma Precondition( name'Length > 0 );

    function To_String( this : access Play_Music_Event ) return String;

end Events.Audio;
