--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

--with Debugging;                         use Debugging;
with Events.Manager;
with Objects;

pragma Warnings( Off, Objects );
pragma Elaborate_All( Objects );

package body Events.Entities is

    not overriding
    procedure Construct( this : access Entity_Event;
                         name : String;
                         id   : Entity_Id ) is
    begin
        Event(this.all).Construct( name );
        this.id := id;
    end Construct;

    ----------------------------------------------------------------------------

    function Get_Id( this : not null access Entity_Event'Class ) return Entity_Id is
    begin
        return this.id;
    end Get_Id;

    ----------------------------------------------------------------------------

    overriding
    function To_String( this : access Entity_Event ) return String is
    begin
        return '<' & this.Get_Name & " - id: " & Image( this.id ) & '>';
    end To_String;

    --==========================================================================

    not overriding
    procedure Construct( this : access Entities_Event;
                         name : String;
                         a,b  : Entity_Id ) is
    begin
        Event(this.all).Construct( name );
        this.a := a;
        this.b := b;
    end Construct;

    ----------------------------------------------------------------------------

    function Get_A( this : not null access Entities_Event'Class ) return Entity_Id is
    begin
        return this.a;
    end Get_A;

    ----------------------------------------------------------------------------

    function Get_B( this : not null access Entities_Event'Class ) return Entity_Id is
    begin
        return this.b;
    end Get_B;

    ----------------------------------------------------------------------------

    overriding
    function To_String( this : access Entities_Event ) return String is
    begin
        return '<' & this.Get_Name & " - a: " & Image( this.a ) &
               ", b: " & Image( this.b ) & '>';
    end To_String;

    --==========================================================================

    not overriding
    procedure Construct( this : access Accelerate_Event;
                         id   : Entity_Id;
                         dir  : Cardinal_Direction;
                         vel  : Float;
                         acc  : Float ) is
    begin
        Entity_Event(this.all).Construct( "Accelerate", id );
        this.dir := dir;
        this.vel := vel;
        this.acc := acc;
    end Construct;

    ----------------------------------------------------------------------------

    function Get_Acceleration( this : not null access Accelerate_Event'Class ) return Float is
    begin
        return this.acc;
    end Get_Acceleration;

    ----------------------------------------------------------------------------

    function Get_Direction( this : not null access Accelerate_Event'Class ) return Cardinal_Direction is
    begin
        return this.dir;
    end Get_Direction;

    ----------------------------------------------------------------------------

    function Get_Velocity( this : not null access Accelerate_Event'Class ) return Float is
    begin
        return this.vel;
    end Get_Velocity;

    --==========================================================================

    overriding
    procedure Adjust( this : access Entity_Attribute_Event ) is
    begin
        Entity_Event(this.all).Adjust;
        this.attribute := To_Unbounded_String( To_String( this.attribute ) );
        this.val := this.val.Clone;
    end Adjust;

    ----------------------------------------------------------------------------

    not overriding
    procedure Construct( this      : access Entity_Attribute_Event;
                         name      : String;
                         id        : Entity_Id;
                         attribute : String;
                         val       : Value_Ptr'Class ) is
    begin
        Entity_Event(this.all).Construct( name, id );
        this.attribute := To_Unbounded_String( attribute );
        this.val := val.Clone;
    end Construct;

    ----------------------------------------------------------------------------

    not overriding
    function Get_Attribute( this : access Entity_Attribute_Event ) return String is
    begin
        return To_String( this.attribute );
    end Get_Attribute;

    ----------------------------------------------------------------------------

    not overriding
    function Get_Value( this : access Entity_Attribute_Event ) return Value_Ptr is
    begin
        return this.val.Clone;
    end Get_Value;

    ----------------------------------------------------------------------------

    overriding
    procedure Delete( this : in out Entity_Attribute_Event ) is
    begin
        this.val := Values.Nul;
        Entity_Event(this).Delete;
    end Delete;

    --==========================================================================

    overriding
    procedure Adjust( this : access Entity_Created_Event ) is
    begin
       Entity_Event(this.all).Adjust;
       this.class := To_Unbounded_String( To_String( this.class ) );
       this.libName := To_Unbounded_String( To_String( this.libName ) );
       this.attributes := As_Assoc( this.attributes.Clone );
    end Adjust;

    ----------------------------------------------------------------------------

    not overriding
    procedure Construct( this         : access Entity_Created_Event;
                         id           : Entity_Id;
                         class        : String;
                         physical,
                         metaphysical,
                         clipped      : Boolean;
                         width,
                         height       : Natural;
                         x, y         : Float;
                         xv, yv       : Float;
                         libName      : String;
                         frame        : Natural;
                         attributes   : Assoc_Ptr ) is
    begin
        Entity_Event(this.all).Construct( "Entity_Created", id );
        this.class := To_Unbounded_String( class );
        this.physical := physical;
        this.metaphysical := metaphysical;
        this.clipped := clipped;
        this.width := width;
        this.height := height;
        this.x := x;
        this.y := y;
        this.xv := xv;
        this.yv := yv;
        this.libName := To_Unbounded_String( libName );
        this.frame := frame;
        this.attributes := As_Assoc( attributes.Clone );
    end Construct;

    ----------------------------------------------------------------------------

    function Get_Attributes( this : not null access Entity_Created_Event'Class ) return Assoc_Ptr is
    begin
        return As_Assoc( this.attributes.Clone );
    end Get_Attributes;

    ----------------------------------------------------------------------------

    overriding
    procedure Delete( this : in out Entity_Created_Event ) is
    begin
        this.attributes := Associations.Nul;
        Entity_Event(this).Delete;
    end Delete;

    ----------------------------------------------------------------------------

    function Get_Class( this : not null access Entity_Created_Event'Class ) return String is
    begin
        return To_String( this.class );
    end Get_Class;

    ----------------------------------------------------------------------------

    function Get_Frame( this : not null access Entity_Created_Event'Class ) return Natural is
    begin
        return this.frame;
    end Get_Frame;

    ----------------------------------------------------------------------------

    function Get_Height( this : not null access Entity_Created_Event'Class ) return Natural is
    begin
        return this.height;
    end Get_Height;

    ----------------------------------------------------------------------------

    function Get_Lib_Name( this : not null access Entity_Created_Event'Class ) return String is
    begin
        return To_String( this.libName );
    end Get_Lib_Name;

    ----------------------------------------------------------------------------

    function Get_Width( this : not null access Entity_Created_Event'Class ) return Natural is
    begin
        return this.width;
    end Get_Width;

    ----------------------------------------------------------------------------

    function Get_X( this : not null access Entity_Created_Event'Class ) return Float is
    begin
        return this.x;
    end Get_X;

    ----------------------------------------------------------------------------

    function Get_XV( this : not null access Entity_Created_Event'Class ) return Float is
    begin
        return this.xv;
    end Get_XV;

    ----------------------------------------------------------------------------

    function Get_Y( this : not null access Entity_Created_Event'Class ) return Float is
    begin
        return this.y;
    end Get_Y;

    ----------------------------------------------------------------------------

    function Get_YV( this : not null access Entity_Created_Event'Class ) return Float is
    begin
        return this.yv;
    end Get_YV;

    ----------------------------------------------------------------------------

    function Is_Clipped( this : not null access Entity_Created_Event'Class ) return Boolean is
    begin
        return this.clipped;
    end Is_Clipped;

    ----------------------------------------------------------------------------

    function Is_Metaphysical( this : not null access Entity_Created_Event'Class ) return Boolean is
    begin
        return this.metaphysical;
    end Is_Metaphysical;

    ----------------------------------------------------------------------------

    function Is_Physical( this : not null access Entity_Created_Event'Class ) return Boolean is
    begin
        return this.physical;
    end Is_Physical;

    ----------------------------------------------------------------------------

    overriding
    function To_String( this : access Entity_Created_Event ) return String is
    begin
        return '<' & this.Get_Name & " - id: " & Image( this.id ) &
               ", class: " & To_String( this.class ) & '>';
    end To_String;

    --==========================================================================

    not overriding
    procedure Construct( this : access Entity_Tile_Event;
                         name : String;
                         id   : Entity_Id;
                         x, y : Natural ) is
    begin
        Entity_Event(this.all).Construct( name, id );
        this.x := x;
        this.y := y;
    end Construct;

    ----------------------------------------------------------------------------

    function Get_X( this : not null access Entity_Tile_Event'Class ) return Natural is
    begin
        return this.x;
    end Get_X;

    ----------------------------------------------------------------------------

    function Get_Y( this : not null access Entity_Tile_Event'Class ) return Natural is
    begin
        return this.y;
    end Get_Y;

    --==========================================================================

    not overriding
    procedure Construct( this     : access Entity_Grounded_Event;
                         id       : Entity_Id;
                         grounded : Boolean ) is
    begin
        Entity_Event(this.all).Construct( "Entity_Grounded", id );
        this.grounded := grounded;
    end Construct;

    ----------------------------------------------------------------------------

    function Is_Grounded( this : not null access Entity_Grounded_Event'Class ) return Boolean is
    begin
        return this.grounded;
    end Is_Grounded;

    --==========================================================================

    not overriding
    procedure Construct( this         : access Entity_Hit_Wall_Event;
                         id           : Entity_Id;
                         dir          : Cardinal_Direction;
                         firstContact : Boolean ) is
    begin
        Entity_Event(this.all).Construct( "Entity_Hit_Wall", id );
        this.dir := dir;
        this.firstContact := firstContact;
    end Construct;

    ----------------------------------------------------------------------------

    function Get_Direction( this : not null access Entity_Hit_Wall_Event'Class ) return Cardinal_Direction is
    begin
        return this.dir;
    end Get_Direction;

    ----------------------------------------------------------------------------

    function Is_First_Contact( this : not null access Entity_Hit_Wall_Event'Class ) return Boolean is
    begin
        return this.firstContact;
    end Is_First_Contact;

    --==========================================================================

    not overriding
    procedure Construct( this   : access Entity_Moved_Event;
                         id     : Entity_Id;
                         x, y,
                         xv, yv : Float ) is
    begin
        Entity_Event(this.all).Construct( "Entity_Moved", id );
        this.x := x;
        this.y := y;
        this.xv := xv;
        this.yv := yv;
    end Construct;

    ----------------------------------------------------------------------------

    function Get_X( this : not null access Entity_Moved_Event'Class ) return Float is
    begin
        return this.x;
    end Get_X;

    ----------------------------------------------------------------------------

    function Get_XV( this : not null access Entity_Moved_Event'Class ) return Float is
    begin
        return this.xv;
    end Get_XV;

    ----------------------------------------------------------------------------

    function Get_Y( this : not null access Entity_Moved_Event'Class ) return Float is
    begin
        return this.y;
    end Get_Y;

    ----------------------------------------------------------------------------

    function Get_YV( this : not null access Entity_Moved_Event'Class ) return Float is
    begin
        return this.yv;
    end Get_YV;

    --==========================================================================

    not overriding
    procedure Construct( this   : access Entity_Resized_Event;
                         id     : Entity_Id;
                         width,
                         height : Natural ) is
    begin
        Entity_Event(this.all).Construct( "Entity_Resized", id );
        this.width := width;
        this.height := height;
    end Construct;

    ----------------------------------------------------------------------------

    function Get_Height( this : not null access Entity_Resized_Event'Class ) return Natural is
    begin
        return this.height;
    end Get_Height;

    ----------------------------------------------------------------------------

    function Get_Width( this : not null access Entity_Resized_Event'Class ) return Natural is
    begin
        return this.width;
    end Get_Width;

    --==========================================================================

    not overriding
    procedure Construct( this  : access Frame_Changed_Event;
                         id    : Entity_Id;
                         frame : Natural ) is
    begin
        Entity_Event(this.all).Construct( "Frame_Changed", id );
        this.frame := frame;
    end Construct;

    ----------------------------------------------------------------------------

    function Get_Frame( this : not null access Frame_Changed_Event'Class ) return Natural is
    begin
        return this.frame;
    end Get_Frame;

    --==========================================================================

    not overriding
    procedure Construct( this      : access Entity_Directive_Event;
                         id        : Entity_Id;
                         directive : Hashed_String;
                         mode      : Directive_Mode ) is
    begin
        Entity_Event(this.all).Construct( "Entity_Directive", id );
        this.directive := directive;
        this.mode := mode;
    end Construct;

    ----------------------------------------------------------------------------

    function Get_Directive( this : not null access Entity_Directive_Event'Class ) return Hashed_String is
    begin
        return this.directive;
    end Get_Directive;

    ----------------------------------------------------------------------------

    function Get_Mode( this : not null access Entity_Directive_Event'Class ) return Directive_Mode is
    begin
        return this.mode;
    end Get_Mode;

    ----------------------------------------------------------------------------

    overriding
    function To_String( this : access Entity_Directive_Event ) return String is
    begin
        return '<' & this.Get_Name & " - id: " & Image( this.id ) &
               ", directive: " & To_String( this.directive ) &
               ", mode: " & Directive_Mode'Image( this.mode ) & '>';
    end To_String;

    --==========================================================================

    not overriding
    procedure Construct( this : access Move_Entity_Event; id : Entity_Id; x, y : Float ) is
    begin
        Entity_Event(this.all).Construct( "Move_Entity", id );
        this.x := x;
        this.y := y;
    end Construct;

    ----------------------------------------------------------------------------

    function Get_X( this : not null access Move_Entity_Event'Class ) return Float is
    begin
        return this.x;
    end Get_X;

    ----------------------------------------------------------------------------

    function Get_Y( this : not null access Move_Entity_Event'Class ) return Float is
    begin
        return this.y;
    end Get_Y;

    --==========================================================================

    not overriding
    procedure Construct( this   : access Resize_Entity_Event;
                         id     : Entity_Id;
                         width,
                         height : Natural ) is
    begin
        Entity_Event(this.all).Construct( "Resize_Entity", id );
        this.width := width;
        this.height := height;
    end Construct;

    ----------------------------------------------------------------------------

    function Get_Height( this : not null access Resize_Entity_Event'Class ) return Natural is
    begin
        return this.height;
    end Get_Height;

    ----------------------------------------------------------------------------

    function Get_Width( this : not null access Resize_Entity_Event'Class ) return Natural is
    begin
        return this.width;
    end Get_Width;

    --==========================================================================

    procedure Construct( this   : access Spawn_Entity_Event;
                         id     : String;
                         x, y   : Float;
                         width,
                         height : Natural;
                         xv, yv : Float ) is
    begin
        Event(this.all).Construct( "Spawn_Entity" );
        this.id := To_Unbounded_String( id );
        this.x := x;
        this.y := y;
        this.width := width;
        this.height := height;
        this.xv := xv;
        this.yv := yv;
    end Construct;

    ----------------------------------------------------------------------------

    function Get_Height( this : access Spawn_Entity_Event ) return Integer is
    begin
        return this.height;
    end Get_Height;

    ----------------------------------------------------------------------------

    function Get_Id( this : access Spawn_Entity_Event ) return String is
    begin
        return To_String( this.id );
    end Get_Id;

    ----------------------------------------------------------------------------

    function Get_Width( this : access Spawn_Entity_Event ) return Integer is
    begin
        return this.width;
    end Get_Width;

    ----------------------------------------------------------------------------

    function Get_X( this : access Spawn_Entity_Event ) return Float is
    begin
        return this.x;
    end Get_X;

    ----------------------------------------------------------------------------

    function Get_XV( this : access Spawn_Entity_Event ) return Float is
    begin
        return this.xv;
    end Get_XV;

    ----------------------------------------------------------------------------

    function Get_Y( this : access Spawn_Entity_Event ) return Float is
    begin
        return this.y;
    end Get_Y;

    ----------------------------------------------------------------------------

    function Get_YV( this : access Spawn_Entity_Event ) return Float is
    begin
        return this.yv;
    end Get_YV;

    ----------------------------------------------------------------------------

    function To_String( this : access Spawn_Entity_Event ) return String is
    begin
        return '<' & this.Get_Name & " - class: " & To_String( this.id ) & '>';
    end To_String;

    --==========================================================================

    procedure Queue_Accelerate( id  : Entity_Id;
                                dir : Cardinal_Direction;
                                vel : Float;
                                acc : Float ) is
        evt : A_Accelerate_Event := new Accelerate_Event;
    begin
        evt.Construct( id, dir, vel, acc );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_Accelerate;

    ----------------------------------------------------------------------------

    procedure Queue_Delete_Entity( id : Entity_Id ) is
        evt : A_Entity_Event := new Entity_Event;
    begin
        evt.Construct( "Delete_Entity", id );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_Delete_Entity;

    ----------------------------------------------------------------------------

    procedure Queue_Entities_Collided( a, b : Entity_Id ) is
        evt : A_Entities_Event := new Entities_Event;
    begin
        evt.Construct( "Entities_Collided", a, b );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_Entities_Collided;

    ----------------------------------------------------------------------------

    procedure Queue_Entities_Separated( a, b : Entity_Id ) is
        evt : A_Entities_Event := new Entities_Event;
    begin
        evt.Construct( "Entities_Separated", a, b );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_Entities_Separated;

    ----------------------------------------------------------------------------

    procedure Queue_Entity_Attribute_Changed( id        : Entity_Id;
                                              attribute : String;
                                              val       : Value_Ptr'Class ) is
        evt : A_Entity_Attribute_Event := new Entity_Attribute_Event;
    begin
        evt.Construct( "Entity_Attribute_Changed", id, attribute, val );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_Entity_Attribute_Changed;

    ----------------------------------------------------------------------------

    procedure Queue_Entity_Created( id           : Entity_Id;
                                    class        : String;
                                    physical,
                                    metaphysical,
                                    clipped      : Boolean;
                                    width,
                                    height       : Natural;
                                    x, y         : Float;
                                    xv, yv       : Float;
                                    libName      : String;
                                    frame        : Natural;
                                    attributes   : Assoc_Ptr ) is
        evt : A_Entity_Created_Event := new Entity_Created_Event;
    begin
        evt.Construct( id, class, physical, metaphysical, clipped, width, height, x, y,
                       xv, yv, libName, frame, attributes );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_Entity_Created;

    ----------------------------------------------------------------------------

    procedure Queue_Entity_Deleted( id : Entity_Id ) is
        evt : A_Entity_Event := new Entity_Event;
    begin
        evt.Construct( "Entity_Deleted", id );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_Entity_Deleted;

    ----------------------------------------------------------------------------

    procedure Queue_Entity_Entered_Tile( id : Entity_Id; x, y : Natural ) is
        evt : A_Entity_Tile_Event := new Entity_Tile_Event;
    begin
        evt.Construct( "Entity_Entered_Tile", id, x, y );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_Entity_Entered_Tile;

    ----------------------------------------------------------------------------

    procedure Queue_Entity_Exited_Tile( id : Entity_Id; x, y : Natural ) is
        evt : A_Entity_Tile_Event := new Entity_Tile_Event;
    begin
        evt.Construct( "Entity_Exited_Tile", id, x, y );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_Entity_Exited_Tile;

    ----------------------------------------------------------------------------

    procedure Queue_Entity_Grounded( id : Entity_Id; grounded : Boolean ) is
        evt : A_Entity_Grounded_Event := new Entity_Grounded_Event;
    begin
        evt.Construct( id, grounded );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_Entity_Grounded;

    ----------------------------------------------------------------------------

    procedure Queue_Entity_Hit_Wall( id           : Entity_Id;
                                     dir          : Cardinal_Direction;
                                     firstContact : Boolean ) is
        evt : A_Entity_Hit_Wall_Event := new Entity_Hit_Wall_Event;
    begin
        evt.Construct( id, dir, firstContact );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_Entity_Hit_Wall;

    ----------------------------------------------------------------------------

    procedure Queue_Entity_Moved( id : Entity_Id; x, y, xv, yv : Float ) is
        evt : A_Entity_Moved_Event := new Entity_Moved_Event;
    begin
        evt.Construct( id, x, y, xv, yv );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_Entity_Moved;

    ----------------------------------------------------------------------------

    procedure Queue_Entity_Resized( id : Entity_Id; width, height : Natural ) is
        evt : A_Entity_Resized_Event := new Entity_Resized_Event;
    begin
        evt.Construct( id, width, height );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_Entity_Resized;

    ----------------------------------------------------------------------------

    procedure Queue_Follow_Entity( id : Entity_Id ) is
        evt : A_Entity_Event := new Entity_Event;
    begin
        evt.Construct( "Follow_Entity", id );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_Follow_Entity;

    ----------------------------------------------------------------------------

    procedure Queue_Frame_Changed( id : Entity_Id; frame : Natural ) is
        evt : A_Frame_Changed_Event := new Frame_Changed_Event;
    begin
        evt.Construct( id, frame );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_Frame_Changed;

    ----------------------------------------------------------------------------

    procedure Queue_Entity_Directive( id        : Entity_Id;
                                      directive : Hashed_String;
                                      mode      : Directive_Mode ) is
        evt : A_Entity_Directive_Event := new Entity_Directive_Event;
    begin
        evt.Construct( id, directive, mode );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_Entity_Directive;

    ----------------------------------------------------------------------------

    procedure Queue_Move_Entity( id : Entity_Id; x, y : Float ) is
        evt : A_Move_Entity_Event := new Move_Entity_Event;
    begin
        evt.Construct( id, x, y );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_Move_Entity;

    ----------------------------------------------------------------------------

    procedure Queue_Resize_Entity( id : Entity_Id; width, height : Natural ) is
        evt : A_Resize_Entity_Event := new Resize_Entity_Event;
    begin
        evt.Construct( id, width, height );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_Resize_Entity;

    ----------------------------------------------------------------------------

    procedure Queue_Set_Entity_Attribute( id        : Entity_Id;
                                          attribute : String;
                                          val       : Value_Ptr'Class ) is
        evt : A_Entity_Attribute_Event := new Entity_Attribute_Event;
    begin
        evt.Construct( "Set_Entity_Attribute", id, attribute, val );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_Set_Entity_Attribute;

    ----------------------------------------------------------------------------

    procedure Queue_Spawn_Entity( id     : String;
                                  x, y   : Float;
                                  width,
                                  height : Natural := 0;
                                  xv, yv : Float := 0.0 ) is
        evt : A_Spawn_Entity_Event := new Spawn_Entity_Event;
    begin
        evt.Construct( id, x, y, width, height, xv, yv );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_Spawn_Entity;

end Events.Entities;
