--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

with Events.Manager;

package body Events.Game is

    not overriding
    procedure Construct( this : access End_Game_Event; completed : Boolean ) is
    begin
        Event(this.all).Construct( "End_Game" );
        this.completed := completed;
    end Construct;

    ----------------------------------------------------------------------------

    function Is_Completed( this : not null access End_Game_Event'Class ) return Boolean is
    begin
        return this.completed;
    end Is_Completed;

    ----------------------------------------------------------------------------

    overriding
    function To_String( this : access End_Game_Event ) return String is
    begin
        return "<" & this.Get_Name & " - completed: " & Boolean'Image( this.completed ) & ">";
    end To_String;

    --==========================================================================

    not overriding
    procedure Construct( this : access Game_Paused_Event; paused : Boolean ) is
    begin
        Event(this.all).Construct( "Game_Paused" );
        this.paused := paused;
    end Construct;

    ----------------------------------------------------------------------------

    function Is_Paused( this : not null access Game_Paused_Event'Class ) return Boolean is
    begin
        return this.paused;
    end Is_Paused;

    ----------------------------------------------------------------------------

    overriding
    function To_String( this : access Game_Paused_Event ) return String is
    begin
        return "<" & this.Get_Name & " - paused: " & Boolean'Image( this.paused ) & ">";
    end To_String;

    --==========================================================================

    not overriding
    procedure Construct( this        : access Game_State_Event;
                         playing     : Boolean;
                         interrupted : Boolean ) is
    begin
        Event(this.all).Construct( "Game_State" );
        this.playing := playing;
        this.interrupted := not playing and then interrupted;
    end Construct;

    ----------------------------------------------------------------------------

    function Is_Interrupted( this : not null access Game_State_Event'Class ) return Boolean is
    begin
        return this.interrupted;
    end Is_Interrupted;

    ----------------------------------------------------------------------------

    function Is_Playing( this : not null access Game_State_Event'Class ) return Boolean is
    begin
        return this.playing;
    end Is_Playing;

    ----------------------------------------------------------------------------

    overriding
    function To_String( this : access Game_State_Event ) return String is
    begin
        return "<" & this.Get_Name & " - playing: " & Boolean'Image( this.playing ) & ">";
    end To_String;

    --==========================================================================

    overriding
    procedure Adjust( this : access Game_Var_Changed_Event ) is
    begin
        Event(this.all).Adjust;
        this.var := To_Unbounded_String( To_String( this.var ) );
        this.val := this.val.Clone;
    end Adjust;

    ----------------------------------------------------------------------------

    not overriding
    procedure Construct( this : access Game_Var_Changed_Event;
                         var  : String;
                         val  : Value_Ptr'Class ) is
    begin
        Event(this.all).Construct( "Game_Var_Changed" );
        this.var := To_Unbounded_String( var );
        this.val := val.Clone;
    end Construct;

    ----------------------------------------------------------------------------

    not overriding
    function Get_Value( this : access Game_Var_Changed_Event ) return Value_Ptr is
    begin
        return this.val.Clone;
    end Get_Value;

    ----------------------------------------------------------------------------

    function Get_Var( this : not null access Game_Var_Changed_Event'Class ) return String is
    begin
        return To_String( this.var );
    end Get_Var;

    ----------------------------------------------------------------------------

    overriding
    function To_String( this : access Game_Var_Changed_Event ) return String is
    begin
        return "<" & this.Get_Name & " - var: " & To_String( this.var ) &
               ", val: " & Image( this.val ) & ">";
    end To_String;

    --==========================================================================

    not overriding
    procedure Construct( this    : access Loading_Event;
                         name    : String;
                         loading : Boolean;
                         error   : String ) is
    begin
        Event(this.all).Construct( name );
        this.loading := loading;
        this.error := To_Unbounded_String( error );
    end Construct;

    ----------------------------------------------------------------------------

    function Get_Error( this : not null access Loading_Event'Class ) return String is
    begin
        return To_String( this.error );
    end Get_Error;

    ----------------------------------------------------------------------------

    function Is_Loading( this : not null access Loading_Event'Class ) return Boolean is
    begin
        return this.loading;
    end Is_Loading;

    ----------------------------------------------------------------------------

    overriding
    function To_String( this : access Loading_Event ) return String is
    begin
        return "<" & this.Get_Name &
               " - loading: " & Boolean'Image( this.loading ) &
               ", error: """ & To_String( this.error ) & """>";
    end To_String;

    --==========================================================================

    not overriding
    procedure Construct( this : access Pause_Game_Event; paused : Boolean ) is
    begin
        Event(this.all).Construct( "Pause_Game" );
        this.paused := paused;
    end Construct;

    ----------------------------------------------------------------------------

    function Is_Paused( this : not null access Pause_Game_Event'Class ) return Boolean is
    begin
        return this.paused;
    end Is_Paused;

    --==========================================================================

    not overriding
    procedure Construct( this : access Scroll_View_Event; x, y : Float ) is
    begin
        Event(this.all).Construct( "Scroll_View" );
        this.x := x;
        this.y := y;
    end Construct;

    ----------------------------------------------------------------------------

    function Get_X( this : not null access Scroll_View_Event'Class ) return Float is
    begin
        return this.x;
    end Get_X;

    ----------------------------------------------------------------------------

    function Get_Y( this : not null access Scroll_View_Event'Class ) return Float is
    begin
        return this.y;
    end Get_Y;

    --==========================================================================

    procedure Queue_End_Game( completed : Boolean ) is
        evt : A_End_Game_Event := new End_Game_Event;
    begin
        evt.Construct( completed );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_End_Game;

    ----------------------------------------------------------------------------

    procedure Queue_Game_Paused( paused : Boolean ) is
        evt : A_Game_Paused_Event := new Game_Paused_Event;
    begin
        evt.Construct( paused );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_Game_Paused;

    ----------------------------------------------------------------------------

    procedure Queue_Game_State( playing : Boolean; interrupted : Boolean := False ) is
        evt : A_Game_State_Event := new Game_State_Event;
    begin
        evt.Construct( playing, interrupted );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_Game_State;

    ----------------------------------------------------------------------------

    procedure Queue_Game_Var_Changed( var : String; val : Value_Ptr'Class ) is
        evt : A_Game_Var_Changed_Event := new Game_Var_Changed_Event;
    begin
        evt.Construct( var, val );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_Game_Var_Changed;

    ----------------------------------------------------------------------------

    procedure Queue_Loading_World( loading : Boolean; error : String := "" ) is
        evt : A_Loading_Event := new Loading_Event;
    begin
        evt.Construct( "Loading_World", loading, error );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_Loading_World;

    ----------------------------------------------------------------------------

    procedure Queue_New_Game is
        evt : A_Event := new Event;
    begin
        evt.Construct( "New_Game" );
        Events.Manager.Queue_Event( evt );
    end Queue_New_Game;

    ----------------------------------------------------------------------------

    procedure Queue_Pause_Game( paused : Boolean ) is
        evt : A_Pause_Game_Event := new Pause_Game_Event;
    begin
        evt.Construct( paused );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_Pause_Game;

    ----------------------------------------------------------------------------

    procedure Queue_Player_Died is
        evt : A_Event := new Event;
    begin
        evt.Construct( "Player_Died" );
        Events.Manager.Queue_Event( evt );
    end Queue_Player_Died;

    ----------------------------------------------------------------------------

    procedure Queue_Scroll_View( x, y : Float ) is
        evt : A_Scroll_View_Event := new Scroll_View_Event;
    begin
        evt.Construct( x, y );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_Scroll_View;

    ----------------------------------------------------------------------------

    procedure Queue_View_Ready is
        evt : A_Event := new Event;
    begin
        evt.Construct( "View_Ready" );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_View_Ready;

end Events.Game;
