--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

with Events.Manager;                    use Events.Manager;

package body Events.World is

    not overriding
    procedure Construct( this    : access Create_World_Event;
                         width,
                         height  : Positive;
                         libName,
                         domain  : String ) is
    begin
        Event(this.all).Construct( "Create_World" );
        this.width := width;
        this.height := height;
        this.libName := To_Unbounded_String( libName );
        this.domain := To_Unbounded_String( domain );
    end Construct;

    ----------------------------------------------------------------------------

    function Get_Domain( this : not null access Create_World_Event'Class ) return String is
    begin
        return To_String( this.domain );
    end Get_Domain;

    ----------------------------------------------------------------------------

    function Get_Height( this : not null access Create_World_Event'Class ) return Positive is
    begin
        return this.height;
    end Get_Height;

    ----------------------------------------------------------------------------

    function Get_Library_Name( this : not null access Create_World_Event'Class ) return String is
    begin
        return To_String( this.libName );
    end Get_Library_Name;

    ----------------------------------------------------------------------------

    function Get_Width( this : not null access Create_World_Event'Class ) return Positive is
    begin
        return this.width;
    end Get_Width;

    --==========================================================================

    overriding
    procedure Construct( this : access Load_World_Event; filename : String ) is
    begin
        Event(this.all).Construct( "Load_World" );
        this.filename := To_Unbounded_String( filename );
    end Construct;

    ----------------------------------------------------------------------------

    function Get_Filename( this : not null access Load_World_Event'Class ) return String is
    begin
        return To_String( this.filename );
    end Get_Filename;

    ----------------------------------------------------------------------------

    overriding
    function To_String( this : access Load_World_Event ) return String is
    begin
        return '<' & this.Get_Name & " - filename: " & To_String( this.filename ) & '>';
    end To_String;

    --==========================================================================

    overriding
    procedure Adjust( this : access World_Loaded_Event ) is
    begin
        Event(this.all).Adjust;
        this.layers := Copy( this.layers );
    end Adjust;

    ----------------------------------------------------------------------------

    not overriding
    procedure Construct( this      : access World_Loaded_Event;
                         width,
                         height,
                         tileWidth : Positive;
                         layers    : not null A_Layer_Array;
                         libName   : String ) is
    begin
        Event(this.all).Construct( "World_Loaded" );
        this.width := width;
        this.height := height;
        this.layers := Copy( layers );
        this.tileWidth := tileWidth;
        this.libName := To_Unbounded_String( libName );
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure Delete( this : in out World_Loaded_Event ) is
    begin
        Delete( this.layers );
        Event(this).Delete;
    end Delete;

    ----------------------------------------------------------------------------

    function Get_Height( this : not null access World_Loaded_Event'Class ) return Positive is
    begin
        return this.height;
    end Get_Height;

    ----------------------------------------------------------------------------

    function Get_Layers( this : not null access World_Loaded_Event'Class ) return A_Layer_Array is
    begin
        return this.layers;
    end Get_Layers;

    ----------------------------------------------------------------------------

    function Get_Library_Name( this : not null access World_Loaded_Event'Class ) return String is
    begin
        return To_String( this.libName );
    end Get_Library_Name;

    ----------------------------------------------------------------------------

    function Get_Tile_Width( this : not null access World_Loaded_Event'Class ) return Positive is
    begin
        return this.tileWidth;
    end Get_Tile_Width;

    ----------------------------------------------------------------------------

    function Get_Width( this : not null access World_Loaded_Event'Class ) return Positive is
    begin
        return this.width;
    end Get_Width;

    --==========================================================================

    procedure Construct( this    : access World_Property_Event;
                         evtName : String;
                         name    : String;
                         value   : Value_Ptr'Class ) is
    begin
        Event(this.all).Construct( evtName );
        this.name := To_Unbounded_String( name );
        this.value := value.Clone;
    end Construct;

    ----------------------------------------------------------------------------

    function Get_Property_Name( this : not null access World_Property_Event'Class ) return String is
    begin
        return To_String( this.name );
    end Get_Property_Name;

    ----------------------------------------------------------------------------

    function Get_Value( this : not null access World_Property_Event'Class ) return Value_Ptr is
    begin
        return this.value.Clone;
    end Get_Value;

    ----------------------------------------------------------------------------

    overriding
    function To_String( this : access World_Property_Event ) return String is
    begin
        return '<' & this.Get_Name & " - name: " & To_String( this.name ) &
               ", value: " & Image( this.value ) & '>';
    end To_String;

    --==========================================================================

    not overriding
    procedure Construct( this   : access Resize_World_Event;
                         width,
                         height : Positive ) is
    begin
        Event(this.all).Construct( "Resize_World" );
        this.width := width;
        this.height := height;
    end Construct;

    ----------------------------------------------------------------------------

    function Get_Height( this : not null access Resize_World_Event'Class ) return Positive is
    begin
        return this.height;
    end Get_Height;

    ----------------------------------------------------------------------------

    function Get_Width( this : not null access Resize_World_Event'Class ) return Positive is
    begin
        return this.width;
    end Get_Width;

    --==========================================================================

    not overriding
    procedure Construct( this    : access Set_Tile_Event;
                         layer   : Integer;
                         x, y    : Float;
                         tile_id : Natural ) is
    begin
        Event(this.all).Construct( "Set_Tile" );
        this.layer := layer;
        this.x := x;
        this.y := y;
        this.tile_id := tile_id;
    end Construct;

    ----------------------------------------------------------------------------

    function Get_Layer( this : not null access Set_Tile_Event'Class ) return Integer is
    begin
        return this.layer;
    end Get_Layer;

    ----------------------------------------------------------------------------

    function Get_Tile_ID( this : not null access Set_Tile_Event'Class ) return Natural is
    begin
        return this.tile_id;
    end Get_Tile_ID;

    ----------------------------------------------------------------------------

    function Get_X( this : not null access Set_Tile_Event'Class ) return Float is
    begin
        return this.x;
    end Get_X;

    ----------------------------------------------------------------------------

    function Get_Y( this : not null access Set_Tile_Event'Class ) return Float is
    begin
        return this.y;
    end Get_Y;

    --==========================================================================

    not overriding
    procedure Construct( this    : access Tile_Changed_Event;
                         layer   : Integer;
                         x, y    : Natural;
                         tile_id : Natural ) is
    begin
        Event(this.all).Construct( "Tile_Changed" );
        this.layer := layer;
        this.x := x;
        this.y := y;
        this.tile_id := tile_id;
    end Construct;

    ----------------------------------------------------------------------------

    function Get_Layer( this : not null access Tile_Changed_Event'Class ) return Integer is
    begin
        return this.layer;
    end Get_Layer;

    ----------------------------------------------------------------------------

    function Get_Tile_ID( this : not null access Tile_Changed_Event'Class ) return Natural is
    begin
        return this.tile_id;
    end Get_Tile_ID;

    ----------------------------------------------------------------------------

    function Get_X( this : not null access Tile_Changed_Event'Class ) return Natural is
    begin
        return this.x;
    end Get_X;

    ----------------------------------------------------------------------------

    function Get_Y( this : not null access Tile_Changed_Event'Class ) return Natural is
    begin
        return this.y;
    end Get_Y;

    --==========================================================================

    procedure Trigger_Create_World( width,
                                    height  : Positive;
                                    libName,
                                    domain  : String ) is
        evt      : A_Create_World_Event := new Create_World_Event;
        response : Response_Type;
    begin
        evt.Construct( width, height, libName, domain );
        Trigger_Event( A_Event(evt), response );
        if Get_Status( response ) /= ST_SUCCESS and then
           Get_Status( response ) /= ST_NONE
        then
            raise Program_Error with Get_Message( response );
        end if;
    end Trigger_Create_World;

    ----------------------------------------------------------------------------

    procedure Queue_Load_World( filename : String ) is
        evt : A_Load_World_Event := new Load_World_Event;
    begin
        evt.Construct( filename );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_Load_World;

    ----------------------------------------------------------------------------

    procedure Trigger_Load_World( filename : String ) is
        evt      : A_Load_World_Event := new Load_World_Event;
        response : Response_Type;
    begin
        evt.Construct( filename );
        Events.Manager.Trigger_Event( A_Event(evt), response );
        if Get_Status( response ) /= ST_SUCCESS and then
           Get_Status( response ) /= ST_NONE
        then
            raise Program_Error with Get_Message( response );
        end if;
    end Trigger_Load_World;

    ----------------------------------------------------------------------------

    procedure Queue_World_Loaded( width,
                                  height,
                                  tileWidth : Positive;
                                  layers    : not null A_Layer_Array;
                                  libName   : String ) is
        evt : A_World_Loaded_Event := new World_Loaded_Event;
    begin
        evt.Construct( width, height, tileWidth, layers, libName );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_World_Loaded;

    ----------------------------------------------------------------------------

    procedure Trigger_Resize_World( width, height : Positive ) is
        evt      : A_Resize_World_Event := new Resize_World_Event;
        response : Response_Type;
    begin
        evt.Construct( width, height );
        Events.Manager.Trigger_Event( A_Event(evt), response );
        if Get_Status( response ) /= ST_SUCCESS and then
           Get_Status( response ) /= ST_NONE
        then
            raise Program_Error with Get_Message( response );
        end if;
    end Trigger_Resize_World;

    ----------------------------------------------------------------------------

    procedure Queue_Set_Tile( layer   : Integer;
                              x, y    : Float;
                              tile_id : Natural ) is
        evt : A_Set_Tile_Event := new Set_Tile_Event;
    begin
        evt.Construct( layer, x, y, tile_id );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_Set_Tile;

    ----------------------------------------------------------------------------

    procedure Queue_Set_World_Property( name : String; value : Value_Ptr'Class ) is
        evt : A_World_Property_Event := new World_Property_Event;
    begin
        evt.Construct( "Set_World_Property", name, value );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_Set_World_Property;

    ----------------------------------------------------------------------------

    procedure Queue_Tile_Changed( layer   : Integer;
                                  x, y    : Natural;
                                  tile_id : Natural ) is
        evt : A_Tile_Changed_Event := new Tile_Changed_Event;
    begin
        evt.Construct( layer, x, y, tile_id );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_Tile_Changed;

    ----------------------------------------------------------------------------

    procedure Queue_World_Modified is
        evt : A_World_Modified_Event := new World_Modified_Event;
    begin
        evt.Construct( "World_Modified" );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_World_Modified;

    ----------------------------------------------------------------------------

    procedure Queue_World_Property_Changed( name : String; value : Value_Ptr'Class ) is
        evt : A_World_Property_Event := new World_Property_Event;
    begin
        evt.Construct( "World_Property_Changed", name, value );
        Events.Manager.Queue_Event( A_Event(evt) );
    end Queue_World_Property_Changed;

end Events.World;
