--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

with Ada.Containers;                    use Ada.Containers;
with Ada.Strings.Hash_Case_Insensitive;

package body Events is

    overriding
    function "="( l, r : Event_Type ) return Boolean is
    begin
        return l.name = r.name;
    end "=";

    ----------------------------------------------------------------------------

    overriding
    function "="( l, r : Event_Id ) return Boolean is
    begin
        return Hash_Type(l) = Hash_Type(r);
    end "=";

    ----------------------------------------------------------------------------

    function To_Event_Id( evtName : String ) return Event_Id is
    begin
        return Event_Id(Ada.Strings.Hash_Case_Insensitive( evtName ));
    end To_Event_Id;

    --==========================================================================

    function Get_Message( response : Response_Type ) return String is
    begin
        return To_String( response.msg );
    end Get_Message;

    ----------------------------------------------------------------------------

    function Get_Status( response : Response_Type ) return Status is
    begin
        return response.stat;
    end Get_Status;

    ----------------------------------------------------------------------------

    procedure Set_Response( response : in out Response_Type;
                            stat     : Status;
                            msg      : String := "" ) is
    begin
        response.stat := stat;
        response.msg := To_Unbounded_String( msg );
    end Set_Response;

    --==========================================================================

    not overriding
    procedure Construct( this : access Event; name : String ) is
    begin
        Object(this.all).Construct;
        this.eTime := Clock;
        this.eType.id   := To_Event_Id( name );
        this.eType.name := To_Unbounded_String( name );
    end Construct;

    ----------------------------------------------------------------------------

    function Get_Id( this : not null access Event'Class ) return Event_Id is
    begin
        return this.eType.id;
    end Get_Id;

    ----------------------------------------------------------------------------

    function Get_Name( this : not null access Event'Class ) return String is
    begin
        return To_String( this.eType.name );
    end Get_Name;

    ----------------------------------------------------------------------------

    overriding
    function To_String( this : access Event ) return String is
    begin
        return "<" & this.Get_Name & ">";
    end To_String;

    ----------------------------------------------------------------------------

    function Copy( src : A_Event ) return A_Event is
    begin
        return A_Event(Copy( A_Object(src) ));
    end Copy;

    ----------------------------------------------------------------------------

    procedure Delete( this : in out A_Event ) is
    begin
        Delete( A_Object(this) );
    end Delete;

end Events;
