--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

with Drawing;                           use Drawing;

package body Physics.Drawing is

    procedure Draw( cType : Clip_Type;
                    x, y  : Integer;
                    size  : Positive;
                    color : Allegro_Color ) is
    begin
        case cType is
            when Passive =>
                null;
            when OneWay =>
                Rectfill( x, y, x + size - 1, y + size / 2 - 1, color );
            when Wall =>
                Rectfill( x, y, x + size - 1, y + size - 1, color );
            when Slope_45_Up_Floor =>
                Triangle_Filled( x, y + size - 1,                 --     2
                                 x + size - 1, y,                 --   / |
                                 x + size - 1, y + size - 1,      -- 1---3
                                 color );
            when Slope_45_Up_Ceiling =>
                Triangle_Filled( x, y,                            -- 1---2
                                 x + size - 1, y,                 -- | /
                                 x, y + size - 1,                 -- 3
                                 color );
            when Slope_45_Down_Floor =>
                Triangle_Filled( x, y,                            -- 1
                                 x + size - 1, y + size - 1,      -- | \
                                 x, y + size - 1,                 -- 3---2
                                 color );
            when Slope_45_Down_Ceiling =>
                Triangle_Filled( x, y,                            -- 1---2
                                 x + size - 1, y,                 --   \ |
                                 x + size - 1, y + size - 1,      --     3
                                 color );
            when Slope_22_Up_Floor_Thin =>
                Triangle_Filled( x, y + size - 1,                 --
                                 x + size - 1, y + size / 2 - 1,  --   ..2
                                 x + size - 1, y + size - 1,      -- 1---3
                                 color );
            when Slope_22_Up_Floor_Wide =>
                Triangle_Filled( x, y + size / 2 - 1,             --   ..2
                                 x + size - 1, y,                 -- 1---3
                                 x + size - 1, y + size / 2 - 1,  -- #####
                                 color );
                Rectfill( x, y + size / 2 - 1,
                          x + size - 1, y + size - 1,
                          color );
            when Slope_22_Up_Ceiling_Wide =>
                Triangle_Filled( x, y + size / 2 - 1,             -- #####
                                 x + size - 1, y + size / 2 - 1,  -- 1---2
                                 x, y + size - 1,                 -- 3''
                                 color );
                Rectfill( x, y,
                          x + size - 1, y + size / 2 - 1,
                          color );
            when Slope_22_Up_Ceiling_Thin =>
                Triangle_Filled( x, y,                            -- 1---2
                                 x + size - 1, y,                 -- 3''
                                 x, y + size / 2 - 1,             --
                                 color );
            when Slope_22_Down_Floor_Wide =>
                Triangle_Filled( x, y,                            -- 1..
                                 x + size - 1, y + size / 2 - 1,  -- 3---2
                                 x, y + size / 2 - 1,             -- #####
                                 color );
                Rectfill( x, y + size / 2 - 1,
                          x + size - 1, y + size - 1,
                          color );
            when Slope_22_Down_Floor_Thin =>
                Triangle_Filled( x, y + size / 2 - 1,             --
                                 x + size - 1, y + size - 1,      -- 1..
                                 x, y + size - 1,                 -- 3---2
                                 color );
            when Slope_22_Down_Ceiling_Thin =>
                Triangle_Filled( x, y,                            -- 1---2
                                 x + size - 1, y,                 --   ''3
                                 x + size - 1, y + size / 2 - 1,  --
                                 color );
            when Slope_22_Down_Ceiling_Wide =>
                Triangle_Filled( x, y + size / 2 - 1,             -- #####
                                 x + size - 1, y + size / 2 - 1,  -- 1---2
                                 x + size - 1, y + size - 1,      --   ''3
                                 color );
                Rectfill( x, y,
                          x + size - 1, y + size / 2 - 1,
                          color );
        end case;
    end Draw;

end Physics.Drawing;
