--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

with Allegro.Bitmaps.Drawing;           use Allegro.Bitmaps.Drawing;
--with Debugging;                         use Debugging;
with Drawing;                           use Drawing;
with Widgets;                           use Widgets;

package body Renderers is

    function Create_Renderer( display : A_Allegro_Display;
                              win     : not null A_Window;
                              fps      : Positive ) return A_Renderer is
        this : constant A_Renderer := new Renderer;
    begin
        this.Construct( display, win, fps );
        return this;
    end Create_Renderer;

    ----------------------------------------------------------------------------

    not overriding
    procedure Construct( this    : access Renderer;
                         display : A_Allegro_Display;
                         win     : not null A_Window;
                         fps     : Positive ) is
    begin
        Limited_Object(this.all).Construct;
        this.frameDelta := Milliseconds( 1000 / fps );
        this.lastFrame := Clock - this.frameDelta;
        this.display := display;
        this.buffer := Al_Create_Bitmap( Al_Get_Display_Width( this.display ),
                                         Al_Get_Display_Height( this.display ) );
        this.win := win;
        this.win.Set_Offset( (Al_Get_Display_Width( this.display ) - win.Get_Width * win.Get_Scale) / 2,
                             (Al_Get_Display_Height( this.display ) - win.Get_Height * win.Get_Scale) / 2 );
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure Delete( this : in out Renderer ) is
    begin
        Al_Destroy_Bitmap( this.buffer );
        Limited_Object(this).Delete;
    end Delete;

    ----------------------------------------------------------------------------

    overriding
    function Get_Process_Name( this : access Renderer ) return String is
        pragma Unreferenced( this );
    begin
        return "Renderer";
    end Get_Process_Name;

    ----------------------------------------------------------------------------

    overriding
    procedure Tick( this : access Renderer; time : Tick_Time ) is
        pragma Unreferenced( time );
        now : constant Ada.Real_Time.Time := Clock;
    begin
        if now - this.frameDelta > this.lastFrame then
            -- in fullscreen modes, this is the color seen outside the
            -- application window if the chosen resolution * scale doesn't
            -- match the screen resolution.
            Set_Target_Bitmap( this.buffer );
            Clear_To_Color( this.win.Get_Outer_Border );

            -- exceptions raised here will be caught by the Process_Manager and
            -- terminate the application.
            Push_State;
            this.win.Draw;
            Pop_State;

            Al_Set_Target_Backbuffer( this.display );
            Al_Draw_Bitmap( this.buffer, 0.0, 0.0, 0 );
            Al_Flip_Display;

            this.lastFrame := now;
        end if;
    end Tick;

    ----------------------------------------------------------------------------

    procedure Delete( this : in out A_Renderer ) is
    begin
        Delete( A_Limited_Object(this) );
    end Delete;

end Renderers;
