--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

with Ada.Characters.Handling;           use Ada.Characters.Handling;
with Ada.Containers.Indefinite_Ordered_Maps;
with Debugging;                         use Debugging;
with Engine_Debugging;                  use Engine_Debugging;
with Support.Paths;                     use Support.Paths;

package body Resources.Archives is

    type Archive_Format is
        record
            load : A_Archive_Loader := null;
        end record;

    package Format_Map is new Ada.Containers.Indefinite_Ordered_Maps( String,
                                                                      Archive_Format,
                                                                      "<", "=" );
    use Format_Map;

    formats : Format_Map.Map;

    -- END PRIVATE DECLARATIONS --
    --==========================================================================

    procedure Register_Format( ext : String; loader : not null A_Archive_Loader ) is
        lext : constant String := To_Lower( ext );
    begin
        if not formats.Contains( lext ) then
            formats.Insert( lext, Archive_Format'(load => loader) );
        else
            null;
            pragma Debug( Dbg( "Archive format '" & lext & "' already registered.",
                               D_RES, WARNING ) );
        end if;
    end Register_Format;

    --==========================================================================

    function Load_Archive( resource : not null A_Resource_File ) return A_Archive is
        pos : constant Format_Map.Cursor := formats.Find( To_Lower( Get_Extension( resource.Get_Filename ) ) );
    begin
        if Has_Element( pos ) then
            return Element( pos ).load( resource );
        else
            raise ARCHIVE_FORMAT_ERROR with "Unrecognized archive extension";
        end if;
    end Load_Archive;

    ----------------------------------------------------------------------------

    not overriding
    procedure Construct( this : access Archive; path : String ) is
    begin
        Limited_Object(this.all).Construct;
        this.path := To_Unbounded_String( path );
    end Construct;

    ----------------------------------------------------------------------------

    procedure Delete( this : in out A_Archive ) is
    begin
        Delete( A_Limited_Object(this) );
    end Delete;

end Resources.Archives;
