--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

with Ada.Characters.Handling;           use Ada.Characters.Handling;
with Allegro.Bitmaps.Image_IO;          use Allegro.Bitmaps.Image_IO;
with Allegro.Color;                     use Allegro.Color;
with Allegro.Displays;                  use Allegro.Displays;
with Applications;                      use Applications;
with Applications.Gui;                  use Applications.Gui;
--with Debugging;                         use Debugging;
with Resources.Allegro_Files;           use Resources.Allegro_Files;
with Support.Paths;                     use Support.Paths;

package body Resources.Images is

    function Is_Format_Supported( format : String ) return Boolean is
        lower : constant String := To_Lower( format );
    begin
        return lower = "bmp" or else
               lower = "png";
    end Is_Format_Supported;

    ----------------------------------------------------------------------------

    function Load_Image( format : String;
                         data   : not null access Stream_Element_Array
                       ) return A_Allegro_Bitmap is
        target : constant A_Allegro_Bitmap := Al_Get_Target_Bitmap;
        bmp    : A_Allegro_Bitmap := null;
        file   : A_Allegro_File := null;
    begin
        file := Open_Memfile( data );
        Al_Set_Target_Backbuffer( A_Gui_Application(Get_Application).Get_Display );
        bmp := Al_Load_Bitmap_f( file, '.' & To_Lower( format ) );
        Al_Fclose( file );
        if bmp = null then
            raise RESOURCE_FORMAT_ERROR with "Unrecognized image format '" & format & "'";
        end if;
        if To_Lower( format ) = "bmp" then
            Al_Convert_Mask_To_Alpha( bmp, Al_Map_RGB( 255, 0, 255 ) );
        end if;
        Al_Set_Target_Bitmap( target );
        return bmp;
    end Load_Image;

    ----------------------------------------------------------------------------

    function Load_Image( resource : A_Resource_File ) return A_Allegro_Bitmap is
        target : constant A_Allegro_Bitmap := Al_Get_Target_Bitmap;
        format : constant String := To_Lower( Get_Extension( resource.Get_Filename ) );
        bmp    : A_Allegro_Bitmap := null;
        file   : A_Allegro_File := null;
    begin
        if resource /= null and then resource.Size > 0 then
            file := resource.Get_File;
            Al_Set_Target_Backbuffer( A_Gui_Application(Get_Application).Get_Display );
            bmp := Al_Load_Bitmap_f( file, '.' & format );
            Al_Fclose( file );
            if bmp = null then
                raise RESOURCE_FORMAT_ERROR with "Unrecognized image format '" &
                                                 Get_Extension( resource.Get_Filename )&
                                                 "'";
            end if;
            if format = "bmp" then
                Al_Convert_Mask_To_Alpha( bmp, Al_Map_RGB( 255, 0, 255 ) );
            end if;
        end if;
        Al_Set_Target_Bitmap( target );
        return bmp;
    end Load_Image;

    ----------------------------------------------------------------------------

    function Load_Image( filepath : String; group : String ) return A_Allegro_Bitmap is
        resource : A_Resource_File := null;
        bmp      : A_Allegro_Bitmap := null;
    begin
        resource := Load_Resource( filepath, group );
        if resource /= null then
            bmp := Load_Image( resource );
            Release( resource );
        else
            raise RESOURCE_ERROR with "File not found: " & filepath;
        end if;
        return bmp;
    exception
        when others =>
            Release( resource );
            raise;
    end Load_Image;

end Resources.Images;
