--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

package body Scripting.Expressions.Functions is

    function Create_Function( name : not null A_Identifier_Token ) return A_Function is
        this : constant A_Function := new Function_Call;
    begin
        this.Construct( name.Get_Location, name.Get_Name );
        return this;
    end Create_Function;

    ----------------------------------------------------------------------------

    not overriding
    procedure Construct( this : access Function_Call;
                         loc  : Token_Location;
                         name : String ) is
    begin
        Expression(this.all).Construct( loc );
        this.name := To_Unbounded_String( name );
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure Delete( this : in out Function_Call ) is
    begin
        Delete_Contents( this.arguments );
        Expression(this).Delete;
    end Delete;

    ----------------------------------------------------------------------------

    overriding
    function Evaluate( this    : access Function_Call;
                       context : not null A_Eval_Context ) return Value_Ptr is
        arguments : Value_Array(1..Integer(this.arguments.Length));
        crsr      : Expression_Lists.Cursor := this.arguments.First;
        arg       : Integer := arguments'First;
    begin
        while Expression_Lists.Has_Element( crsr ) loop
            arguments(arg) := Expression_Lists.Element( crsr ).Evaluate( context );
            arg := arg + 1;
            crsr := Expression_Lists.Next( crsr );
        end loop;

        return context.Evaluate_Function( To_String( this.name ), arguments );
    end Evaluate;

    ----------------------------------------------------------------------------

    function Get_Name( this : not null access Function_Call'Class ) return String is
    begin
        return To_String( this.name );
    end Get_Name;

    ----------------------------------------------------------------------------

    procedure Set_Arguments( this      : not null access Function_Call'Class;
                             arguments : in out Expression_Lists.List ) is
    begin
        Delete_Contents( this.arguments );
        this.arguments := arguments;
        arguments.Clear;
    end Set_Arguments;

end Scripting.Expressions.Functions;
