--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

private with Ada.Strings.Unbounded;

package Scripting.Expressions.Functions is

    -- A Function is a call into the host application to perform an operation,
    -- given a set of arguments. Any type of value may be returned by the
    -- function when it is evaluated (ie: executed).
    type Function_Call is new Expression with private;
    type A_Function is access all Function_Call'Class;

    -- Creates a new Function from a Token, representing the function name.
    function Create_Function( name : not null A_Identifier_Token ) return A_Function;
    pragma Postcondition( Create_Function'Result /= null );

    -- todo: make this private.
    -- this is public as a workaround for a 6.2.0w OSX compiler bug
    function Evaluate( this    : access Function_Call;
                       context : not null A_Eval_Context ) return Value_Ptr;

    -- Returns the function's name. This should match with the definition of an
    -- identifier in the scripting language.
    function Get_Name( this : not null access Function_Call'Class ) return String;

    -- Sets the arguments to the function as a list of Expressions. 'arguments'
    -- will be consumed.
    procedure Set_Arguments( this      : not null access Function_Call'Class;
                             arguments : in out Expression_Lists.List );

private

    use Ada.Strings.Unbounded;

    type Function_Call is new Expression with
        record
            name      : Unbounded_String;
            arguments : Expression_Lists.List;
        end record;

    procedure Construct( this : access Function_Call;
                         loc  : Token_Location;
                         name : String );

    procedure Delete( this : in out Function_Call );

end Scripting.Expressions.Functions;
