--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

with Values.Numbers;                    use Values.Numbers;
with Values.Strings;                    use Values.Strings;

package body Scripting.Expressions.Literals is

    function Create_Literal( token : not null A_Token ) return A_Literal is
        this : A_Literal := null;
    begin
        case token.Get_Type is
            when TK_NUMBER =>
                this := new Literal;
                this.Construct( token.Get_Location, Create( A_Number_Token(token).Get_Value ) );

            when TK_STRING =>
                this := new Literal;
                this.Construct( token.Get_Location, Create( A_String_Token(token).Get_Value ) );

            when others => null;
        end case;
        return this;
    end Create_Literal;

    ----------------------------------------------------------------------------

    not overriding
    procedure Construct( this  : access Literal;
                         loc   : Token_Location;
                         value : Value_Ptr'Class ) is
    begin
        Expression(this.all).Construct( loc );
        this.value := value.Clone;
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    function Evaluate( this    : access Literal;
                       context : not null A_Eval_Context ) return Value_Ptr is
        pragma Unreferenced( context );
    begin
       return this.value.Clone;
    end Evaluate;

end Scripting.Expressions.Literals;
