--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

package Scripting.Expressions.Literals is

    -- A Literal is a literal value (boolean, number, or string) in an
    -- expression.
    type Literal is new Expression with private;
    type A_Literal is access all Literal'Class;

    -- Creates a new Literal from a Token representing a literal value. If the
    -- token is not a value type, null will be returned.
    function Create_Literal( token : not null A_Token ) return A_Literal;

private

    type Literal is new Expression with
        record
            value : Value_Ptr;
        end record;

    procedure Construct( this  : access Literal;
                         loc   : Token_Location;
                         value : Value_Ptr'Class );

    function Evaluate( this    : access Literal;
                       context : not null A_Eval_Context ) return Value_Ptr;

end Scripting.Expressions.Literals;
