--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

with Values.Booleans;                   use Values.Booleans;
with Values.Nulls;                      use Values.Nulls;
with Values.Numbers;                    use Values.Numbers;
with Values.Strings;                    use Values.Strings;

package body Scripting.Expressions.Operators is

    -- Add_Op performs addition and string concatenation.
    type Add_Op is new Binary_Operator with null record;

    function Evaluate( this    : access Add_Op;
                       context : not null A_Eval_Context ) return Value_Ptr;

    -- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

    -- And_Op performs a logical AND operation.
    type And_Op is new Binary_Operator with null record;

    function Evaluate( this    : access And_Op;
                       context : not null A_Eval_Context ) return Value_Ptr;

    -- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

    -- Divide_Op performs division.
    type Divide_Op is new Binary_Operator with null record;

    function Evaluate( this    : access Divide_Op;
                       context : not null A_Eval_Context ) return Value_Ptr;

    -- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

    -- Equals_Op tests value equality.
    type Equals_Op is new Binary_Operator with null record;

    function Evaluate( this    : access Equals_Op;
                       context : not null A_Eval_Context ) return Value_Ptr;

    -- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

    -- Greater_Op performs numeric and string comparison (case sensitive).
    type Greater_Op is new Binary_Operator with null record;

    function Evaluate( this    : access Greater_Op;
                       context : not null A_Eval_Context ) return Value_Ptr;

    -- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

    -- Greater_Equals_Op performs numeric and string comparison (case sensitive).
    type Greater_Equals_Op is new Binary_Operator with null record;

    function Evaluate( this    : access Greater_Equals_Op;
                       context : not null A_Eval_Context ) return Value_Ptr;

    -- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

    -- Less_Op performs numeric and string comparison (case sensitive).
    type Less_Op is new Binary_Operator with null record;

    function Evaluate( this    : access Less_Op;
                       context : not null A_Eval_Context ) return Value_Ptr;

    -- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

    -- Less_Equals_Op performs numeric and string comparison (case sensitive).
    type Less_Equals_Op is new Binary_Operator with null record;

    function Evaluate( this    : access Less_Equals_Op;
                       context : not null A_Eval_Context ) return Value_Ptr;

    -- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

    -- Mod_Op performs a numeric modulus/remainder operation.
    type Mod_Op is new Binary_Operator with null record;

    function Evaluate( this    : access Mod_Op;
                       context : not null A_Eval_Context ) return Value_Ptr;

    -- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

    -- Multiply_Op performs a multiplication operation.
    type Multiply_Op is new Binary_Operator with null record;

    function Evaluate( this    : access Multiply_Op;
                       context : not null A_Eval_Context ) return Value_Ptr;

    -- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

    -- Not_Equals_Op performs numeric and string comparison (case sensitive).
    type Not_Equals_Op is new Binary_Operator with null record;

    function Evaluate( this    : access Not_Equals_Op;
                       context : not null A_Eval_Context ) return Value_Ptr;

    -- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

    -- Or_Op performs a logical OR operation.
    type Or_Op is new Binary_Operator with null record;

    function Evaluate( this    : access Or_Op;
                       context : not null A_Eval_Context ) return Value_Ptr;

    -- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

    -- Subtract_Op performs substraction.
    type Subtract_Op is new Binary_Operator with null record;

    function Evaluate( this    : access Subtract_Op;
                       context : not null A_Eval_Context ) return Value_Ptr;

    ----------------------------------------------------------------------------

    -- Negate_Op performs numeric negation.
    type Negate_Op is new Unary_Operator with null record;

    function Evaluate( this    : access Negate_Op;
                       context : not null A_Eval_Context ) return Value_Ptr;

    -- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

    -- Not_Op performs a logical NOT operation.
    type Not_Op is new Unary_Operator with null record;

    function Evaluate( this    : access Not_Op;
                       context : not null A_Eval_Context ) return Value_Ptr;

    -- END PRIVATE DECLARATIONS --
    --==========================================================================

    function Create_Binary_Operator( token : not null A_Token ) return A_Binary_Operator is
        this : A_Binary_Operator;
    begin
        case token.Get_Type is
            when TK_AND =>
                this := new And_Op;
                this.Construct( token.Get_Location, 1 );

            when TK_EQUALS =>
                this := new Equals_Op;
                this.Construct( token.Get_Location, 2 );

            when TK_GREATER =>
                this := new Greater_Op;
                this.Construct( token.Get_Location, 2 );

            when TK_GREATER_EQUALS =>
                this := new Greater_Equals_Op;
                this.Construct( token.Get_Location, 2 );

            when TK_LESS =>
                this := new Less_Op;
                this.Construct( token.Get_Location, 2 );

            when TK_LESS_EQUALS =>
                this := new Less_Equals_Op;
                this.Construct( token.Get_Location, 2 );

            when TK_MINUS =>
                this := new Subtract_Op;
                this.Construct( token.Get_Location, 3 );

            when TK_NOT_EQUALS =>
                this := new Not_Equals_Op;
                this.Construct( token.Get_Location, 2 );

            when TK_OR =>
                this := new Or_Op;
                this.Construct( token.Get_Location, 1 );

            when TK_PERCENT =>
                this := new Mod_Op;
                this.Construct( token.Get_Location, 4 );

            when TK_PLUS =>
                this := new Add_Op;
                this.Construct( token.Get_Location, 3 );

            when TK_SLASH =>
                this := new Divide_Op;
                this.Construct( token.Get_Location, 4 );

            when TK_STAR =>
                this := new Multiply_Op;
                this.Construct( token.Get_Location, 4 );

            when others => null;
        end case;
        return this;
    end Create_Binary_Operator;

    ----------------------------------------------------------------------------

    not overriding
    procedure Construct( this       : access Binary_Operator;
                         loc        : Token_Location;
                         precedence : Positive ) is
    begin
        Expression(this.all).Construct( loc );
        this.precedence := precedence;
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure Delete( this : in out Binary_Operator ) is
    begin
        Delete( this.left );
        Delete( this.right );
        Expression(this).Delete;
    end Delete;

    ----------------------------------------------------------------------------

    function Get_Precedence( this : not null access Binary_Operator'Class ) return Positive is
    begin
        return this.precedence;
    end Get_Precedence;

    ----------------------------------------------------------------------------

    procedure Set_Operands( this        : not null access Binary_Operator'Class;
                            left, right : in out A_Expression ) is
    begin
        Delete( this.left );
        Delete( this.right );
        this.left := left;
        this.right := right;
        left := null;
        right := null;
    end Set_Operands;

    ----------------------------------------------------------------------------

    function Is_Binary( tokenType : Token_Type ) return Boolean is
    begin
        case tokenType is
            when TK_AND            |
                 TK_EQUALS         |
                 TK_GREATER        |
                 TK_GREATER_EQUALS |
                 TK_LESS           |
                 TK_LESS_EQUALS    |
                 TK_MINUS          |
                 TK_NOT_EQUALS     |
                 TK_OR             |
                 TK_PERCENT        |
                 TK_PLUS           |
                 TK_SLASH          |
                 TK_STAR           => return True;

            when others => return False;
        end case;
    end Is_Binary;

    --==========================================================================

    overriding
    function Evaluate( this    : access Add_Op;
                       context : not null A_Eval_Context ) return Value_Ptr is
        left, right : Value_Ptr;
    begin
        left := this.left.Evaluate( context );
        right := this.right.Evaluate( context );

        if left.Get.Get_Type = V_STRING and then right.Get.Get_Type = V_STRING then
            -- string concatenation
            return Value_Ptr(As_String( left ) & As_String( right ));
        elsif left.Get.Get_Type = V_NUMBER and then right.Get.Get_Type = V_NUMBER then
            -- mathematical addition (may raise exception)
            return Value_Ptr(As_Number( left ) + As_Number( right ));
        else
            -- unresolved value
            return Create_Null;
        end if;
    exception
        when others =>
            -- wrong types
            return Create_Null;
    end Evaluate;

    --==========================================================================

    overriding
    function Evaluate( this    : access And_Op;
                       context : not null A_Eval_Context ) return Value_Ptr is
        left, right : Value_Ptr;
    begin
        left := this.left.Evaluate( context );
        right := this.right.Evaluate( context );

        if left.Get.Get_Type /= V_BOOLEAN or else right.Get.Get_Type /= V_BOOLEAN then
            -- an unresolved value in either operand is false
            return Value_Ptr(Create( False ));
        else
            -- logical AND (may raise exception)
            return Value_Ptr(As_Boolean( left ) and As_Boolean( right ));
        end if;
    exception
        when others =>
            -- wrong types
            return Create_Null;
    end Evaluate;

    --==========================================================================

    overriding
    function Evaluate( this    : access Divide_Op;
                       context : not null A_Eval_Context ) return Value_Ptr is
        left, right : Value_Ptr;
    begin
        left := this.left.Evaluate( context );
        right := this.right.Evaluate( context );

        if left.Get.Get_Type /= V_NUMBER or else right.Get.Get_Type /= V_NUMBER then
            -- unresolved value
            return Create_Null;
        else
            -- division (may raise exception)
            return Value_Ptr(As_Number( left ) / As_Number( right ));
        end if;
    exception
        when others =>
            return Create_Null;
    end Evaluate;

    --==========================================================================

    overriding
    function Evaluate( this    : access Equals_Op;
                       context : not null A_Eval_Context ) return Value_Ptr is
    begin
        return Value_Ptr(Create( this.left.Evaluate( context ) = this.right.Evaluate( context ) ));
    exception
        when others =>
            return Create_Null;
    end Evaluate;

    --==========================================================================

    overriding
    function Evaluate( this    : access Greater_Op;
                       context : not null A_Eval_Context ) return Value_Ptr is
        left, right : Value_Ptr;
    begin
        left := this.left.Evaluate( context );
        right := this.right.Evaluate( context );

        return Value_Ptr(Create( left > right ));
    exception
        when others =>
            return Create_Null;
    end Evaluate;

    --==========================================================================

    overriding
    function Evaluate( this    : access Greater_Equals_Op;
                       context : not null A_Eval_Context ) return Value_Ptr is
        left, right : Value_Ptr;
    begin
        left := this.left.Evaluate( context );
        right := this.right.Evaluate( context );

        return Value_Ptr(Create( left >= right ));
    exception
        when others =>
            return Create_Null;
    end Evaluate;

    --==========================================================================

    overriding
    function Evaluate( this    : access Less_Op;
                       context : not null A_Eval_Context ) return Value_Ptr is
        left, right : Value_Ptr;
    begin
        left := this.left.Evaluate( context );
        right := this.right.Evaluate( context );

        return Value_Ptr(Create( left < right ));
    exception
        when others =>
            return Create_Null;
    end Evaluate;

    --==========================================================================

    overriding
    function Evaluate( this    : access Less_Equals_Op;
                       context : not null A_Eval_Context ) return Value_Ptr is
        left, right : Value_Ptr;
    begin
        left := this.left.Evaluate( context );
        right := this.right.Evaluate( context );

        return Value_Ptr(Create( left <= right ));
    exception
        when others =>
            return Create_Null;
    end Evaluate;

    --==========================================================================

    overriding
    function Evaluate( this    : access Mod_Op;
                       context : not null A_Eval_Context ) return Value_Ptr is
        left, right : Value_Ptr;
    begin
        left := this.left.Evaluate( context );
        right := this.right.Evaluate( context );

        if left.Get.Get_Type /= V_NUMBER or else right.Get.Get_Type = V_NUMBER then
            -- unresolved value
            return Create_Null;
        else
            -- modulo (may raise exception)
            return Value_Ptr(As_Number( left ) mod As_Number( right ));
        end if;
    exception
        when others =>
            return Create_Null;
    end Evaluate;

    --==========================================================================

    overriding
    function Evaluate( this    : access Multiply_Op;
                       context : not null A_Eval_Context ) return Value_Ptr is
        left, right : Value_Ptr;
    begin
        left := this.left.Evaluate( context );
        right := this.right.Evaluate( context );

        if left.Get.Get_Type /= V_NUMBER or else right.Get.Get_Type /= V_NUMBER then
            -- unresolved value
            return Create_Null;
        else
            -- multiplication (may raise exception)
            return Value_Ptr(As_Number( left ) * As_Number( right ));
        end if;
    exception
        when others =>
            return Create_Null;
    end Evaluate;

    --==========================================================================

    overriding
    function Evaluate( this    : access Not_Equals_Op;
                       context : not null A_Eval_Context ) return Value_Ptr is
        left, right : Value_Ptr;
    begin
        left := this.left.Evaluate( context );
        right := this.right.Evaluate( context );

        return Value_Ptr(Create( left /= right ));
    exception
        when others =>
            return Create_Null;
    end Evaluate;

    --==========================================================================

    overriding
    function Evaluate( this    : access Or_Op;
                       context : not null A_Eval_Context ) return Value_Ptr is
        left, right : Value_Ptr;
    begin
        left := this.left.Evaluate( context );
        right := this.right.Evaluate( context );

        if left.Get.Get_Type /= V_BOOLEAN then
            -- logical OR (may raise exception)
            -- left is not a boolean (false), so the result of OR depends on right
            return Value_Ptr(Create( right.Get.Get_Type = V_BOOLEAN and then As_Boolean( right ).Get.To_Boolean ));
        elsif right.Get.Get_Type /= V_BOOLEAN then
            -- logical OR (may raise exception)
            -- right is not a boolean (false), so the result of OR depends on left
            return Value_Ptr(Create( left.Get.Get_Type = V_BOOLEAN and then As_Boolean( left ).Get.To_Boolean ));
        else
            -- logical OR (may raise exception)
            return Value_Ptr(As_Boolean( left ) or As_Boolean( right ));
        end if;
    exception
        when others =>
            return Create_Null;
    end Evaluate;

    --==========================================================================

    overriding
    function Evaluate( this    : access Subtract_Op;
                       context : not null A_Eval_Context ) return Value_Ptr is
        left, right : Value_Ptr;
    begin
        left := this.left.Evaluate( context );
        right := this.right.Evaluate( context );

        if left.Get.Get_Type /= V_NUMBER or else right.Get.Get_Type /= V_NUMBER then
            -- unresolved value
            return Create_Null;
        else
            -- subtraction (may raise exception)
            return Value_Ptr(As_Number( left ) - As_Number( right ));
        end if;
    exception
        when others =>
            return Create_Null;
    end Evaluate;

    --==========================================================================

    function Create_Unary_Operator( token : not null A_Token ) return A_Unary_Operator is
        this : A_Unary_Operator;
    begin
        case token.Get_Type is

            when TK_BANG =>
                this := new Not_Op;
                this.Construct( token.Get_Location );

            when TK_MINUS =>
                this := new Negate_Op;
                this.Construct( token.Get_Location );

            when others => null;
        end case;
        return this;
    end Create_Unary_Operator;

    ----------------------------------------------------------------------------

    overriding
    procedure Delete( this : in out Unary_Operator ) is
    begin
        Delete( this.right );
        Expression(this).Delete;
    end Delete;

    ----------------------------------------------------------------------------

    procedure Set_Operand( this  : not null access Unary_Operator'Class;
                           right : in out A_Expression ) is
    begin
        Delete( this.right );
        this.right := right;
        right := null;
    end Set_Operand;

    ----------------------------------------------------------------------------

    function Is_Unary( tokenType : Token_Type ) return Boolean is
    begin
        case tokenType is
            when TK_BANG  |
                 TK_MINUS => return True;

            when others => return False;
        end case;
    end Is_Unary;

    --==========================================================================

    overriding
    function Evaluate( this    : access Negate_Op;
                       context : not null A_Eval_Context ) return Value_Ptr is
         right : Value_Ptr;
    begin
        right := this.right.Evaluate( context );

        if right.Get.Get_Type /= V_NUMBER then
            -- unresolved value
            return Create_Null;
        else
            -- negate the value (may raise exception)
            return Value_Ptr(-As_Number( right ));
        end if;
    exception
        when others =>
            return Create_Null;
    end Evaluate;

    --==========================================================================

    overriding
    function Evaluate( this    : access Not_Op;
                       context : not null A_Eval_Context ) return Value_Ptr is
         right : Value_Ptr;
    begin
        right := this.right.Evaluate( context );

        if right.Get.Get_Type /= V_BOOLEAN then
            -- unresolved value
            return right.Clone;
        else
            -- negate the value (may raise exception)
            return Value_Ptr(not As_Boolean( right ));
        end if;
    exception
        when others =>
            return Create_Null;
    end Evaluate;

end Scripting.Expressions.Operators;
