--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

private with Ada.Strings.Unbounded;

package Scripting.Expressions.Variables is

    -- A Variable is a value of any type (boolean, number, or string) that will
    -- be resolved by name only when the expression is evaluated.
    type Variable is new Expression with private;
    type A_Variable is access all Variable'Class;

    -- Creates a new Variable from a Token representing an indentifier name.
    function Create_Variable( token : not null A_Identifier_Token ) return A_Variable;
    pragma Postcondition( Create_Variable'Result /= null );

private

    use Ada.Strings.Unbounded;

    type Variable is new Expression with
        record
            name : Unbounded_String;
        end record;

    procedure Construct( this : access Variable; loc : Token_Location; name : String );

    function Evaluate( this    : access Variable;
                       context : not null A_Eval_Context ) return Value_Ptr;

end Scripting.Expressions.Variables;
