--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

with Ada.Containers;                    use Ada.Containers;
with Ada.Containers.Doubly_Linked_Lists;
with Objects;                           use Objects;
with Tokens;                            use Tokens;

package Scripting.Expressions is

    -- An Expression represents a mathematical or logical expression tree that
    -- can be evaluated to produce a value. It is composed of operators and
    -- operands. The three base data types used in expressions are numeric,
    -- string, and boolean, which are represented with the Values hierarchy.
    type Expression is abstract new Limited_Object with private;
    type A_Expression is access all Expression'Class;

    -- Evaluates the expression and it's subtree(s). Returns null if the
    -- expression can't be evaluated due to mismatched types or unresolved
    -- symbols.
    function Evaluate( this    : access Expression;
                       context : not null A_Eval_Context ) return Value_Ptr is abstract;

    -- Deletes the Expression.
    procedure Delete( this : in out A_Expression );
    pragma Postcondition( this = null );

    ----------------------------------------------------------------------------

    package Expression_Lists is new Ada.Containers.Doubly_Linked_Lists( A_Expression, "=" );

    -- Deep deletes the list.
    procedure Delete_Contents( expressions : in out Expression_Lists.List );
    pragma Postcondition( Expression_Lists.Length( expressions ) = 0 );

private

    type Expression is abstract new Limited_Object with
        record
            loc : Token_Location;
        end record;

    procedure Construct( this : access Expression; loc : Token_Location );

end Scripting.Expressions;
