--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

with Ada.Exceptions;                    use Ada.Exceptions;

package body Scripting.Scripts is

    function Create_Script return A_Script is
    begin
        return new Script;
    end Create_Script;

    ----------------------------------------------------------------------------

    overriding
    procedure Delete( this : in out Script ) is
    begin
        Delete_Contents( this.statements );
        Limited_Object(this).Delete;
    end Delete;

    ----------------------------------------------------------------------------

    not overriding
    procedure Evaluate( this    : access Script;
                        context : not null A_Eval_Context ) is
        pos : Statement_Lists.Cursor := this.statements.First;
    begin
        while Statement_Lists.Has_Element( pos ) loop
            Statement_Lists.Element( pos ).Evaluate( context );  -- may raise exception
            pos := Statement_Lists.Next( pos );
        end loop;
    exception
        when e : Evaluation_Exception =>
            raise Evaluation_Exception with "Script error: " & Exception_Message( e );
    end Evaluate;

    ----------------------------------------------------------------------------

    procedure Set_Statements( this       : not null access Script'Class;
                              statements : in out Statement_Lists.List ) is
    begin
        Delete_Contents( this.statements );
        this.statements := statements;
        statements.Clear;
    end Set_Statements;

    ----------------------------------------------------------------------------

    procedure Delete( this : in out A_Script ) is
    begin
        Delete( A_Limited_Object(this) );
    end Delete;

end Scripting.Scripts;
