--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

with Objects;                           use Objects;
with Scripting.Statements;              use Scripting.Statements;

package Scripting.Scripts is

    -- A Script is an executable series of Statements. It does not return a
    -- a value, so evaluation doesn't return a result.
    type Script is new Limited_Object with private;
    type A_Script is access all Script'Class;

    -- Creates a new empty Script.
    function Create_Script return A_Script;
    pragma Postcondition( Create_Script'Result /= null );

    -- Evaluates/executes the script. Evaluation_Exception will be raised if an
    -- error occurs.
    procedure Evaluate( this    : access Script;
                        context : not null A_Eval_Context );

    -- Sets the statements in the script. Each will be evaluated in order, when
    -- the script is evaluated. 'statements' will be consumed.
    procedure Set_Statements( this       : not null access Script'Class;
                              statements : in out Statement_Lists.List );

    -- Deletes the Script.
    procedure Delete( this : in out A_Script );
    pragma Postcondition( this = null );

private

    type Script is new Limited_Object with
        record
            statements : Statement_Lists.List;
        end record;

    procedure Delete( this : in out Script );

end Scripting.Scripts;
