--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

with Scripting.Expressions.Functions;   use Scripting.Expressions.Functions;

package Scripting.Statements.Methods is

    -- A Method statement evaluates a function, ignoring any return value.
    type Method is new Statement with private;
    type A_Method is access all Method'Class;

    -- Creates a new empty Method.
    function Create_Method return A_Method;
    pragma Postcondition( Create_Method'Result /= null );

    -- Sets the function that the method statement will evaluate.
    procedure Set_Function( this : not null access Method'Class;
                            func : not null A_Function );

private

    type Method is new Statement with
        record
            func : A_Function;
        end record;

    procedure Delete( this : in out Method );

    -- Evaluates/executes the Method. Evaluates the function but the return
    -- value is ignored. Evaluation_Exception will be raised if an error occurs.
    procedure Evaluate( this    : access Method;
                        context : not null A_Eval_Context );

end Scripting.Statements.Methods;
