--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

with Ada.Containers;                    use Ada.Containers;
with Ada.Containers.Doubly_Linked_Lists;
with Objects;                           use Objects;

package Scripting.Statements is

    -- A Statement represents a single command in a script. Scripts are composed
    -- of a series of statements. Statements can be evaluated/executed, but they
    -- don't return a value.
    type Statement is abstract new Limited_Object with private;
    type A_Statement is access all Statement'Class;

    -- Evaluates the statement, performing whatever action it represents. Each
    -- Statement class overrides this procedure.
    procedure Evaluate( this    : access Statement;
                        context : not null A_Eval_Context ) is abstract;

    -- Deletes the Statement.
    procedure Delete( this : in out A_Statement );
    pragma Postcondition( this = null );

    ----------------------------------------------------------------------------

    package Statement_Lists is new Ada.Containers.Doubly_Linked_Lists( A_Statement, "=" );

    -- Deep deletes the list.
    procedure Delete_Contents( statements : in out Statement_Lists.List );
    pragma Postcondition( Statement_Lists.Length( statements ) = 0 );

private

    type Statement is abstract new Limited_Object with null record;

end Scripting.Statements;
