--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

with Ada.Exceptions;                    use Ada.Exceptions;
with Values.Nulls;                      use Values.Nulls;

package body Scripting is

    function Create_Eval_Context( evalNode : A_Evaluation_Node ) return A_Eval_Context is
    begin
        return new Eval_Context'(evalNode => evalNode);
    end Create_Eval_Context;

    ----------------------------------------------------------------------------

    not overriding
    function Evaluate_Function( this      : access Eval_Context;
                                name      : String;
                                arguments : Value_Array ) return Value_Ptr is
    begin
        if this.evalNode /= null then
            return this.evalNode.Evaluate_Function( name, arguments );
        else
            return Create_Null;
        end if;
    exception
        when e : others =>
            raise Evaluation_Exception with
                  "Exception evaluating function " & name & "(): " &
                  Exception_Name( e ) & " - " &
                  Exception_Message( e );
    end Evaluate_Function;

    ----------------------------------------------------------------------------

    not overriding
    function Evaluate_Symbol( this : access Eval_Context; symbol : String ) return Value_Ptr is
    begin
        if this.evalNode /= null then
            return this.evalNode.Evaluate_Symbol( symbol );
        else
            return Create_Null;
        end if;
    exception
        when e : others =>
            raise Evaluation_Exception with
                  "Exception evaluating symbol " & symbol & ": " &
                  Exception_Name( e ) & " - " &
                  Exception_Message( e );
    end Evaluate_Symbol;

end Scripting;
