--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

private with Ada.Containers;
private with Ada.Strings.Hash_Case_Insensitive;

package Statuses is
    pragma Elaborate_Body;

    -- A Status is the hash value of a status string.
    type Status is private;

    function "="( l, r : Status ) return Boolean;

    -- Returns the Status value for a status id string; case insensitive.
    -- (Ex: To_Status("SUCCESS") = To_Status("success") Note that because a
    -- Status is a hash value, Status collisions are possible.
    function To_Status( id : String ) return Status;
    pragma Precondition( id'Length > 0 );

    ST_NONE      : constant Status;     -- "none"
    ST_FAILED    : constant Status;     -- "failed"
    ST_SUCCESS   : constant Status;     -- "success"
    ST_EXCEPTION : constant Status;     -- "exception"

private

    use Ada.Containers;

    type Status is new Ada.Containers.Hash_Type;

    ST_NONE      : constant Status := Status(Ada.Strings.Hash_Case_Insensitive( "none" ));
    ST_FAILED    : constant Status := Status(Ada.Strings.Hash_Case_Insensitive( "failed" ));
    ST_SUCCESS   : constant Status := Status(Ada.Strings.Hash_Case_Insensitive( "success" ));
    ST_EXCEPTION : constant Status := Status(Ada.Strings.Hash_Case_Insensitive( "exception" ));

end Statuses;
