--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

with Ada.Unchecked_Deallocation;

package body Streams.Packfiles is

    procedure Free is new Ada.Unchecked_Deallocation( Packfile_Stream'Class,
                                                      A_Packfile_Stream );

    -- END PRIVATE DECLARATIONS --
    --==========================================================================

    function Open_Stream( filename, mode : String ) return A_Packfile_Stream is
        strm : A_Packfile_Stream := new Packfile_Stream;
    begin
        strm.file := Al_Fopen( filename, mode );
        if strm.file /= null then
            strm.doClose := True;
        else
            Free( strm );
        end if;
        return strm;
    end Open_Stream;

    ----------------------------------------------------------------------------

    function Stream( f : not null A_Allegro_File ) return A_Packfile_Stream is
        strm : constant A_Packfile_Stream := new Packfile_Stream;
    begin
        strm.file := f;
        return strm;
    end Stream;

    ----------------------------------------------------------------------------

    function EOF( stream : not null access Packfile_Stream'Class ) return Boolean is
    begin
        return Al_Feof( stream.file );
    end EOF;

    ----------------------------------------------------------------------------

    function Error( stream : not null access Packfile_Stream'Class ) return Boolean is
    begin
        return Al_Ferror( stream.file );
    end Error;

    ----------------------------------------------------------------------------

    overriding
    procedure Read( stream : in out Packfile_Stream;
                    item   : out Stream_Element_Array;
                    last   : out Stream_Element_Offset ) is
    begin
        last := Stream_Element_Offset(
            Al_Fread( stream.file, item(item'First)'Address, item'Length ) );
    end Read;

    ----------------------------------------------------------------------------

    overriding
    procedure Write( stream : in out Packfile_Stream;
                     item   : Stream_Element_Array ) is
        ret : Integer;
        pragma Warnings( Off, ret );
    begin
        ret := Integer(Al_Fwrite( stream.file, item(item'First)'Address, item'Length ));
    end Write;

    ----------------------------------------------------------------------------

    procedure Close( stream : in out A_Packfile_Stream ) is
    begin
        if stream /= null then
            if stream.doClose then
                Al_Fclose( stream.file );
            end if;
            Free( stream );
        end if;
    end Close;

end Streams.Packfiles;
