--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

with Allegro.Displays;                  use Allegro.Displays;

private with Ada.Containers.Doubly_Linked_Lists;

limited private with Tiles.Atlases.Bins;

private package Tiles.Atlases is

    type Atlas is new Limited_Object with private;
    type A_Atlas is access all Atlas'Class;

    -- Creates a new Atlas with pages of the specified size but no larger than
    -- the largest texture size supported by the display.
    function Create_Atlas( display    : not null A_Allegro_Display;
                           pageWidth,
                           pageHeight : Positive ) return A_Atlas;
    pragma Postcondition( Create_Atlas'Result /= null );

    -- Adds a bitmap 'bmp' to the Atlas. It will be consumed and returned as a
    -- sub-bitmap of a page in the atlas. Be sure to destroy the bitmap before
    -- this atlas, or the bitmap will become invalid.
    procedure Add_Bitmap( this : not null access Atlas'Class;
                          bmp  : in out A_Allegro_Bitmap );

    -- Saves each page of the atlas as a bitmap in the current directory, using
    -- 'name' as the filename prefix and appending the page number. The bitmaps
    -- will be saved in .png format.
    procedure Save_Bitmaps( this : not null access Atlas'Class;
                            name : String );

    -- Deletes the Atlas. Make sure all sub-bitmap references previously
    -- returned from the atlas have been destroyed first or they will become
    -- invalid.
    procedure Delete( this : in out A_Atlas );

private

    type Page is
        record
            bmp  : A_Allegro_Bitmap;
            bin  : access Tiles.Atlases.Bins.Bin'Class;
            full : Boolean := False;
        end record;
    type A_Page is access all Page;

    package Page_Lists is new Ada.Containers.Doubly_Linked_Lists( A_Page, "=" );

    type Atlas is new Limited_Object with
        record
            display    : A_Allegro_Display := null;
            pageWidth,
            pageHeight : Natural := 0;
            pages      : Page_Lists.List;
        end record;

    -- Constructs the Atlas with pages of 'pageWidth' x 'pageHeight' but no
    -- larger than the largest texture size supported by the display.
    procedure Construct( this       : in out Atlas;
                         display    : not null A_Allegro_Display;
                         pageWidth,
                         pageHeight : Positive );

    procedure Delete( this : in out Atlas );

    -- Adds a new Page to the Atlas.
    procedure Add_Page( this : not null access Atlas'Class );

end Tiles.Atlases;
