--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

with Ada.Unchecked_Deallocation;

package body Tiles.Matrices is

    function A_Tile_Matrix_Input( stream : access Root_Stream_Type'Class ) return A_Tile_Matrix is
        matrix : A_Tile_Matrix;
    begin
        if Boolean'Input( stream ) then
            A_Tile_Matrix'Read( stream, matrix );
        end if;
        return matrix;
    end A_Tile_Matrix_Input;

    ----------------------------------------------------------------------------

    procedure A_Tile_Matrix_Output( stream : access Root_Stream_Type'Class; matrix : A_Tile_Matrix ) is
    begin
        Boolean'Output( stream, matrix /= null );
        if matrix /= null then
            A_Tile_Matrix'Write( stream, matrix );
        end if;
    end A_Tile_Matrix_Output;

    ----------------------------------------------------------------------------

    procedure A_Tile_Matrix_Read( stream : access Root_Stream_Type'Class; matrix : out A_Tile_Matrix ) is
        height, width : Natural;
    begin
        height := Natural'Input( stream );
        width := Natural'Input( stream );
        if height > MAX_MATRIX_SIZE or else width > MAX_MATRIX_SIZE then
            raise Constraint_Error with "Tile matrix in stream is too large";
        end if;
        matrix := new Tile_Matrix(1..height, 1..width);
        for r in matrix'Range(1) loop
            for c in matrix'Range(2) loop
                matrix(r, c) := Natural'Input( stream );
            end loop;
        end loop;
    end A_Tile_Matrix_Read;

    ----------------------------------------------------------------------------

    procedure A_Tile_Matrix_Write( stream : access Root_Stream_Type'Class; matrix : A_Tile_Matrix ) is
    begin
        Natural'Output( stream, matrix'Length(1) );    -- height
        Natural'Output( stream, matrix'Length(2) );    -- width
        for r in matrix'Range(1) loop
            for c in matrix'Range(2) loop
                Natural'Output( stream, matrix(r, c) );
            end loop;
        end loop;
    end A_Tile_Matrix_Write;

    ----------------------------------------------------------------------------

    function Copy( src : A_Tile_Matrix ) return A_Tile_Matrix is
        result : A_Tile_Matrix := null;
    begin
        if src /= null then
            result := new Tile_Matrix'(src.all);
        end if;
        return result;
    end Copy;

    ----------------------------------------------------------------------------

    procedure Delete( m : in out A_Tile_Matrix ) is
        procedure Free is new Ada.Unchecked_Deallocation( Tile_Matrix, A_Tile_Matrix );
    begin
        Free( m );
    end Delete;

end Tiles.Matrices;
