--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

package Tiles.Matrices is

    -- maximum size of a matrix in either dimension
    MAX_MATRIX_SIZE : constant := 32;

    ----------------------------------------------------------------------------

    -- A Tile_Matrix is a two-dimensional array of tile ids. It is stored in the
    -- index file of a tile library and used to draw repeating tile patterns or
    -- large tile-based objects when editing a world map.
    type Tile_Matrix is array (Integer range <>, Integer range <>) of Integer;
    type A_Tile_Matrix is access all Tile_Matrix;

    -- Returns a copy of the Tile_Matrix.
    function Copy( src : A_Tile_Matrix ) return A_Tile_Matrix;
    pragma Postcondition( Copy'Result /= src or else src = null );

    -- Deletes the Tile_Matrix.
    procedure Delete( m : in out A_Tile_Matrix );
    pragma Postcondition( m = null );

private

    function A_Tile_Matrix_Input( stream : access Root_Stream_Type'Class ) return A_Tile_Matrix;
    for A_Tile_Matrix'Input use A_Tile_Matrix_Input;

    procedure A_Tile_Matrix_Output( stream : access Root_Stream_Type'Class; matrix : A_Tile_Matrix );
    for A_Tile_Matrix'Output use A_Tile_Matrix_Output;

    procedure A_Tile_Matrix_Read( stream : access Root_Stream_Type'Class; matrix : out A_Tile_Matrix );
    for A_Tile_Matrix'Read use A_Tile_Matrix_Read;

    procedure A_Tile_Matrix_Write( stream : access Root_Stream_Type'Class; matrix : A_Tile_Matrix );
    for A_Tile_Matrix'Write use A_Tile_Matrix_Write;

end Tiles.Matrices;
