--
-- Copyright (c) 2012-2013 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

with Values;

pragma Elaborate_All( Values );

package body Values.Booleans is

    function Create_Instance( val : Boolean ) return Boolean_Ptr is
        this : constant A_Naked_Boolean := new Boolean_Value'(Value with val => val );
        ref  : Boolean_Ptr;
    begin
        ref.Set( this );
        return ref;
    end Create_Instance;

    BOOL_TRUE  : constant Boolean_Ptr := Create_Instance( True );
    BOOL_FALSE : constant Boolean_Ptr := Create_Instance( False );

    ----------------------------------------------------------------------------

    function Create( val : Boolean ) return Boolean_Ptr is
    begin
        if val then
            return BOOL_TRUE;
        else
            return BOOL_FALSE;
        end if;
    end Create;

    ----------------------------------------------------------------------------

    overriding
    function Clone( this : access Boolean_Value ) return Value_Ptr'Class is
    begin
        return Booleans.To_Ptr( this );
    end Clone;

    ----------------------------------------------------------------------------

    overriding
    function Compare( this : Boolean_Value; other : Value'Class ) return Integer is
    begin
        if this.Get_Type = other.Get_Type then
            if this.val = Boolean_Value(other).val then
                return 0;
            elsif not this.val then
                return -1;
            else
                return 1;
            end if;
        elsif this.Get_Type < other.Get_Type then
            return -1;
        else
            return 1;
        end if;
    end Compare;

    ----------------------------------------------------------------------------

    overriding
    function Get_Type( this : Boolean_Value ) return Value_Type is
        pragma Unreferenced( this );
    begin
        return V_BOOLEAN;
    end Get_Type;

    ----------------------------------------------------------------------------

    overriding
    function Image( this : Boolean_Value ) return String is
    begin
        if this.val then
            return "true";
        else
            return "false";
        end if;
    end Image;

    ----------------------------------------------------------------------------

    function To_Boolean( this : Boolean_Value ) return Boolean is
    begin
        return this.val;
    end To_Boolean;

    ----------------------------------------------------------------------------

    overriding
    function Value_Input( stream : access Root_Stream_Type'Class ) return Boolean_Value is
        this : aliased Boolean_Value;
    begin
        Boolean_Value'Read( stream, this );
        return this;
    end Value_Input;

    ----------------------------------------------------------------------------

    overriding
    procedure Value_Read( stream : access Root_Stream_Type'Class; this : out Boolean_Value ) is
    begin
        this.val := Boolean'Input( stream );
    end Value_Read;

    ----------------------------------------------------------------------------

    overriding
    procedure Value_Write( stream : access Root_Stream_Type'Class; this : Boolean_Value ) is
    begin
        Boolean'Output( stream, this.val );
    end Value_Write;

    ----------------------------------------------------------------------------

    function "not"( l : Boolean_Value ) return Boolean_Ptr'Class is begin return Create( not l.val ); end "not";

    function "and"( l, r : Boolean_Value ) return Boolean_Ptr'Class is begin return Create( l.val and r.val ); end "and";
    function  "or"( l, r : Boolean_Value ) return Boolean_Ptr'Class is begin return Create( l.val or  r.val ); end "or";
    function "xor"( l, r : Boolean_Value ) return Boolean_Ptr'Class is begin return Create( l.val xor r.val ); end "xor";

    --==========================================================================

    function As_Boolean( ptr : Value_Ptr'Class ) return Boolean_Ptr is
        result : Boolean_Ptr;
    begin
        if ptr.target /= null then
            if ptr.target.Get_Type = V_BOOLEAN then
                result.Set( ptr.target );
            end if;
        end if;
        return result;
    end As_Boolean;

    ----------------------------------------------------------------------------

    function As_Value( this : Boolean_Ptr ) return Value_Ptr is
        result : Value_Ptr;
    begin
        result.Set( this.target );
        return result;
    end As_Value;

    ----------------------------------------------------------------------------

    function Get( this : Boolean_Ptr ) return access Boolean_Value'Class is
    begin
        return A_Naked_Boolean(this.target);
    end Get;

    ----------------------------------------------------------------------------

    function "not"( l : Boolean_Ptr ) return Boolean_Ptr is begin return Boolean_Ptr(not l.Get.all); end "not";

    function "and"( l, r : Boolean_Ptr ) return Boolean_Ptr is begin return Boolean_Ptr(l.Get.all and r.Get.all); end "and";
    function  "or"( l, r : Boolean_Ptr ) return Boolean_Ptr is begin return Boolean_Ptr(l.Get.all or  r.Get.all); end "or";
    function "xor"( l, r : Boolean_Ptr ) return Boolean_Ptr is begin return Boolean_Ptr(l.Get.all xor r.Get.all); end "xor";

    ----------------------------------------------------------------------------

    function "="( l : Boolean_Ptr; r : Boolean ) return Boolean is begin return l = Create( r ); end "=";

    function "and"( l : Boolean_Ptr; r : Boolean ) return Boolean_Ptr is begin return l and Create( r ); end "and";
    function  "or"( l : Boolean_Ptr; r : Boolean ) return Boolean_Ptr is begin return l or  Create( r ); end "or";
    function "xor"( l : Boolean_Ptr; r : Boolean ) return Boolean_Ptr is begin return l xor Create( r ); end "xor";

    -- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

    function "="( l : Boolean; r : Boolean_Ptr ) return Boolean is begin return Create( l ) = r; end "=";

    function "and"( l : Boolean; r : Boolean_Ptr ) return Boolean_Ptr is begin return Create( l ) and r; end "and";
    function  "or"( l : Boolean; r : Boolean_Ptr ) return Boolean_Ptr is begin return Create( l ) or  r; end "or";
    function "xor"( l : Boolean; r : Boolean_Ptr ) return Boolean_Ptr is begin return Create( l ) xor r; end "xor";

end Values.Booleans;
