--
-- Copyright (c) 2012-2013 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

package Values.Booleans is

    type Boolean_Value is new Value with private;
    type Boolean_Ptr is new Value_Ptr with private;

    function Create( val : Boolean ) return Boolean_Ptr;

    overriding
    function Clone( this : access Boolean_Value ) return Value_Ptr'Class;

    overriding
    function Compare( this : Boolean_Value; other : Value'Class ) return Integer;

    overriding
    function Get_Type( this : Boolean_Value ) return Value_Type;

    overriding
    function Image( this : Boolean_Value ) return String;

    -- Access the internal value
    function To_Boolean( this : Boolean_Value ) return Boolean;

    -- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

    -- Unary logical operators
    function "not"( l : Boolean_Value ) return Boolean_Ptr'Class; pragma Inline( "not" );

    -- Binary logical operators
    function "and"( l, r : Boolean_Value ) return Boolean_Ptr'Class; pragma Inline( "and" );
    function  "or"( l, r : Boolean_Value ) return Boolean_Ptr'Class; pragma Inline( "or" );
    function "xor"( l, r : Boolean_Value ) return Boolean_Ptr'Class; pragma Inline( "xor" );

    ----------------------------------------------------------------------------

    -- Casts a Value_Ptr down to a Boolean_Ptr. Returns Nul on failure.
    function As_Boolean( ptr : Value_Ptr'Class ) return Boolean_Ptr;

    -- Casts a Boolean_Ptr up to a Value_Ptr.
    function As_Value( this : Boolean_Ptr ) return Value_Ptr;

    -- Returns an access to the Boolean_Ptr, or null if no target.
    function Get( this : Boolean_Ptr ) return access Boolean_Value'Class; pragma Inline( Get );

    -- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

    -- The following logical operators are equivalent to "[op] l.Get"
    -- 'l' must have a target or the behavior is undefined.
    function "not"( l : Boolean_Ptr ) return Boolean_Ptr; pragma Inline( "not" );

    -- The following logical operators are equivalent to "l.Get [op] r.Get"
    -- Both 'l' and 'r' must have targets or the behavior is undefined.
    function "and"( l, r : Boolean_Ptr ) return Boolean_Ptr; pragma Inline( "and" );
    function  "or"( l, r : Boolean_Ptr ) return Boolean_Ptr; pragma Inline( "or" );
    function "xor"( l, r : Boolean_Ptr ) return Boolean_Ptr; pragma Inline( "xor" );

    -- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

    -- The following operators are useful for mixing Boolean_Ptrs with the
    -- primitive boolean type. Equivalent to: "l [op] Create( r )"
    function "="( l : Boolean_Ptr; r : Boolean ) return Boolean; pragma Inline( "=" );

    function "and"( l : Boolean_Ptr; r : Boolean ) return Boolean_Ptr; pragma Inline( "and" );
    function  "or"( l : Boolean_Ptr; r : Boolean ) return Boolean_Ptr; pragma Inline( "or" );
    function "xor"( l : Boolean_Ptr; r : Boolean ) return Boolean_Ptr; pragma Inline( "xor" );

    function "="( l : Boolean; r : Boolean_Ptr ) return Boolean; pragma Inline( "=" );

    function "and"( l : Boolean; r : Boolean_Ptr ) return Boolean_Ptr; pragma Inline( "and" );
    function  "or"( l : Boolean; r : Boolean_Ptr ) return Boolean_Ptr; pragma Inline( "or" );
    function "xor"( l : Boolean; r : Boolean_Ptr ) return Boolean_Ptr; pragma Inline( "xor" );

    Nul : constant Boolean_Ptr;

private

    type Boolean_Value is new Value with
        record
            val : Boolean := False;
        end record;
    type A_Naked_Boolean is access all Boolean_Value'Class;

    overriding
    function Value_Input( stream : access Root_Stream_Type'Class ) return Boolean_Value;
    for Boolean_Value'Input use Value_Input;

    overriding
    procedure Value_Read( stream : access Root_Stream_Type'Class; this : out Boolean_Value );
    for Boolean_Value'Read use Value_Read;

    procedure Value_Write( stream : access Root_Stream_Type'Class; this : Boolean_Value );
    for Boolean_Value'Write use Value_Write;

    -- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

    type Boolean_Ptr is new Value_Ptr with null record;

    Nul : constant Boolean_Ptr := (Value_Ptr with others => <>);

end Values.Booleans;
