--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

with Values.Ids;                        use Values.Ids;
with Values.Numbers;                    use Values.Numbers;
with Values.Strings;                    use Values.Strings;

package body Values.Casting is

    function Cast_Enum( ptr : Value_Ptr'Class; default : Enumeration ) return Enumeration is
        str    : constant String_Ptr := As_String( ptr );
        result : Enumeration;
    begin
        if str /= Strings.Nul then
            result := Enumeration'Value( str.Get.To_String );
            if result'Valid then
                return result;
            end if;
        end if;
        return default;
    exception
        when others =>
            -- Enumeration'Value throw exception on invalid enumeration value
            return default;
    end Cast_Enum;

    ----------------------------------------------------------------------------

    function Cast_Id( ptr : Value_Ptr'Class; default : Id_Type := 0 ) return Id_Type is
        id : constant Id_Ptr := As_Id( ptr );
    begin
        if id /= Ids.Nul then
            return Id_Type(id.Get.To_Unsigned_64);
        end if;
        return default;
    end Cast_Id;

    ----------------------------------------------------------------------------

    function Cast_Int( ptr : Value_Ptr'Class; default : Integer := 0 ) return Integer is
        num : constant Number_Ptr := As_Number( ptr );
    begin
        if num /= Numbers.Nul then
            return num.Get.To_Int;
        end if;
        return default;
    end Cast_Int;

    ----------------------------------------------------------------------------

    function Cast_String( ptr : Value_Ptr'Class; default : String := "" ) return String is
        str : constant String_Ptr := As_String( ptr );
    begin
        if str /= Strings.Nul then
            return str.Get.To_String;
        end if;
        return default;
    end Cast_String;

end Values.Casting;
