--
-- Copyright (c) 2012-2013 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

package body Values.Errors is

    function Create( code : Error_Code; msg : String ) return Value_Ptr is
        this : constant A_Naked_Value := new Error_Value'(Value with
                                                          code => code,
                                                          msg  => To_Unbounded_String( msg ) );
        ref : Value_Ptr;
    begin
        ref.Set( this );
        return ref;
    end Create;

    ----------------------------------------------------------------------------

    overriding
    function Clone( this : access Error_Value ) return Value_Ptr'Class is
    begin
        return To_Ptr( this );
    end Clone;

    ----------------------------------------------------------------------------

    function Compare( this : Error_Value; other : Value'Class ) return Integer is
    begin
        if this.Get_Type = other.Get_Type then
            if this.code = Error_Value(other).code then
                if this.msg = Error_Value(other).msg then
                    return 0;
                elsif this.msg < Error_Value(other).msg then
                    return -1;
                else
                    return 1;
                end if;
            elsif this.code < Error_Value(other).code then
                return -1;
            else
                return 1;
            end if;
        elsif this.Get_Type < other.Get_Type then
            return -1;
        else
            return 1;
        end if;
    end Compare;

    ----------------------------------------------------------------------------

    function Get_Code( this : Error_Value ) return Error_Code is
    begin
        return this.code;
    end Get_Code;

    ----------------------------------------------------------------------------

    function Get_Type( this : Error_Value ) return Value_Type is
        pragma Unreferenced( this );
    begin
        return V_ERROR;
    end Get_Type;

    ----------------------------------------------------------------------------

    function Get_Message( this : Error_Value ) return String is
    begin
        return To_String( this.msg );
    end Get_Message;

    ----------------------------------------------------------------------------

    overriding
    function Image( this : Error_Value ) return String is
    begin
        return "[Error " & Error_Code'Image( this.code ) & ": " &
               To_String( this.msg ) & "]";
    end Image;

    ----------------------------------------------------------------------------

    overriding
    function Value_Input( stream : access Root_Stream_Type'Class ) return Error_Value is
        this : aliased Error_Value;
    begin
        Error_Value'Read( stream, this );
        return this;
    end Value_Input;

    ----------------------------------------------------------------------------

    overriding
    procedure Value_Read( stream : access Root_Stream_Type'Class; this : out Error_Value ) is
    begin
        this.code := Error_Code'Value( String'Input( stream ) );
        this.msg := To_Unbounded_String( String'Input( stream ) );
    end Value_Read;

    ----------------------------------------------------------------------------

    overriding
    procedure Value_Write( stream : access Root_Stream_Type'Class; this : Error_Value ) is
    begin
        String'Output( stream, Error_Code'Image( this.code ) );
        String'Output( stream, To_String( this.msg ) );
    end Value_Write;

end Values.Errors;
