--
-- Copyright (c) 2012-2013 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

private with Ada.Strings.Unbounded;

package Values.Errors is

    type Error_Code is
    (
        Generic_Exception,
        Illegal_Operation,
        Type_Mismatch
    );

    ----------------------------------------------------------------------------

    type Error_Value is new Value with private;

    function Create( code : Error_Code; msg : String ) return Value_Ptr;

    overriding
    function Clone( this : access Error_Value ) return Value_Ptr'Class;

    overriding
    function Compare( this : Error_Value; other : Value'Class ) return Integer;

    function Get_Code( this : Error_Value ) return Error_Code;

    overriding
    function Image( this : Error_Value ) return String;

    overriding
    function Get_Type( this : Error_Value ) return Value_Type;

    function Get_Message( this : Error_Value ) return String;

private

    use Ada.Strings.Unbounded;

    type Error_Value is new Value with
        record
            code : Error_Code;
            msg  : Unbounded_String;
        end record;

    overriding
    function Value_Input( stream : access Root_Stream_Type'Class ) return Error_Value;
    for Error_Value'Input use Value_Input;

    overriding
    procedure Value_Read( stream : access Root_Stream_Type'Class; this : out Error_Value );
    for Error_Value'Read use Value_Read;

    procedure Value_Write( stream : access Root_Stream_Type'Class; this : Error_Value );
    for Error_Value'Write use Value_Write;

end Values.Errors;
