--
-- Copyright (c) 2012-2013 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

with Support;                           use Support;
with Values;

pragma Elaborate_All( Values );

package body Values.Ids is

    function Create( val : Unsigned_64 ) return Id_Ptr is
        this : constant A_Naked_Id := new Id_Value'(Value with val => val );
        ref  : Id_Ptr;
    begin
        ref.Set( this );
        return ref;
    end Create;

    ----------------------------------------------------------------------------

    overriding
    function Clone( this : access Id_Value ) return Value_Ptr'Class is
    begin
        return Ids.To_Ptr( this );
    end Clone;

    ----------------------------------------------------------------------------

    overriding
    function Compare( this : Id_Value; other : Value'Class ) return Integer is
    begin
        if this.Get_Type = other.Get_Type then
            if this.val = Id_Value(other).val then
                return 0;
            elsif this.val < Id_Value(other).val then
                return -1;
            else
                return 1;
            end if;
        elsif this.Get_Type < other.Get_Type then
            return -1;
        else
            return 1;
        end if;
    end Compare;

    ----------------------------------------------------------------------------

    overriding
    function Get_Type( this : Id_Value ) return Value_Type is
        pragma Unreferenced( this );
    begin
        return V_ID;
    end Get_Type;

    ----------------------------------------------------------------------------

    overriding
    function Image( this : Id_Value ) return String is
    begin
        return "@" & Image( this.val );
    end Image;

    ----------------------------------------------------------------------------

    function To_Unsigned_64( this : Id_Value ) return Unsigned_64 is
    begin
        return this.val;
    end To_Unsigned_64;

    ----------------------------------------------------------------------------

    overriding
    function Value_Input( stream : access Root_Stream_Type'Class ) return Id_Value is
        this : aliased Id_Value;
    begin
        Id_Value'Read( stream, this );
        return this;
    end Value_Input;

    ----------------------------------------------------------------------------

    overriding
    procedure Value_Read( stream : access Root_Stream_Type'Class; this : out Id_Value ) is
    begin
        this.val := Unsigned_64'Input( stream );
    end Value_Read;

    ----------------------------------------------------------------------------

    overriding
    procedure Value_Write( stream : access Root_Stream_Type'Class; this : Id_Value ) is
    begin
        Unsigned_64'Output( stream, this.val );
    end Value_Write;

    --==========================================================================

    function As_Id( ptr : Value_Ptr'Class ) return Id_Ptr is
        result : Id_Ptr;
    begin
        if ptr.target /= null then
            if ptr.target.Get_Type = V_ID then
                result.Set( ptr.target );
            end if;
        end if;
        return result;
    end As_Id;

    ----------------------------------------------------------------------------

    function As_Value( this : Id_Ptr ) return Value_Ptr is
        result : Value_Ptr;
    begin
        result.Set( this.target );
        return result;
    end As_Value;

    ----------------------------------------------------------------------------

    function Get( this : Id_Ptr ) return access Id_Value'Class is
    begin
        return A_Naked_Id(this.target);
    end Get;

end Values.Ids;
