--
-- Copyright (c) 2012-2013 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

with Values;

pragma Elaborate_All( Values );

package body Values.Nulls is

    function Create_Null_Instance return Value_Ptr is
        this : constant A_Naked_Value := new Null_Value;
        ref  : Value_Ptr;
    begin
        ref.Set( this );
        return ref;
    end Create_Null_Instance;

    instance : constant Value_Ptr := Create_Null_Instance;

    ----------------------------------------------------------------------------

    function Create_Null return Value_Ptr is
    begin
        return instance;
    end Create_Null;

    ----------------------------------------------------------------------------

    overriding
    function Clone( this : access Null_Value ) return Value_Ptr'Class is
        pragma Unreferenced( this );
    begin
        return instance;
    end Clone;

    ----------------------------------------------------------------------------

    overriding
    function Compare( this : Null_Value; other : Value'Class ) return Integer is
    begin
        if this.Get_Type = other.Get_Type then
            return 0;
        elsif this.Get_Type < other.Get_Type then
            return -1;
        else
            return 1;
        end if;
    end Compare;

    ----------------------------------------------------------------------------

    overriding
    function Get_Type( this : Null_Value ) return Value_Type is
        pragma Unreferenced( this );
    begin
        return V_NULL;
    end Get_Type;

    ----------------------------------------------------------------------------

    overriding
    function Image( this : Null_Value ) return String is
        pragma Unreferenced( this );
    begin
        return "null";
    end Image;

    ----------------------------------------------------------------------------

    overriding
    function Value_Input( stream : access Root_Stream_Type'Class ) return Null_Value is
        this : aliased Null_Value;
    begin
        Null_Value'Read( stream, this );
        return this;
    end Value_Input;

    ----------------------------------------------------------------------------

    overriding
    procedure Value_Read( stream : access Root_Stream_Type'Class; this : out Null_Value ) is
        pragma Unreferenced( stream, this );
    begin
        null;
    end Value_Read;

    ----------------------------------------------------------------------------

    overriding
    procedure Value_Write( stream : access Root_Stream_Type'Class; this : Null_Value ) is
        pragma Unreferenced( stream, this );
    begin
        null;
    end Value_Write;

end Values.Nulls;
