--
-- Copyright (c) 2012-2013 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

package body Values.Strings is

    function Create( val : String ) return String_Ptr is
        this : constant A_Naked_String := new String_Value'(Value with val => To_Unbounded_String(val) );
        ref  : String_Ptr;
    begin
        ref.Set( this );
        return ref;
    end Create;

    ----------------------------------------------------------------------------

    function Create( val : Unbounded_String ) return String_Ptr is
        this : constant A_Naked_String := new String_Value'(Value with val => val );
        ref  : String_Ptr;
    begin
        ref.Set( this );
        return ref;
    end Create;

    ----------------------------------------------------------------------------

    overriding
    function Clone( this : access String_Value ) return Value_Ptr'Class is
    begin
        return Strings.To_Ptr( this );
    end Clone;

    ----------------------------------------------------------------------------

    overriding
    function Compare( this : String_Value; other : Value'Class ) return Integer is
    begin
        if this.Get_Type = other.Get_Type then
            if this.val = String_Value(other).val then
                return 0;
            elsif this.val < String_Value(other).val then
                return -1;
            else
                return 1;
            end if;
        elsif this.Get_Type < other.Get_Type then
            return -1;
        else
            return 1;
        end if;
    end Compare;

    ----------------------------------------------------------------------------

    overriding
    function Get_Type( this : String_Value ) return Value_Type is
        pragma Unreferenced( this );
    begin
        return V_STRING;
    end Get_Type;

    ----------------------------------------------------------------------------

    overriding
    function Image( this : String_Value ) return String is
        result : Unbounded_String;
    begin
        Append( result, """" );
        Append( result, Escape( To_String( this.val ) ) );
        Append( result, """" );
        return To_String( result );
    end Image;

    ----------------------------------------------------------------------------

    function Length( this : String_Value ) return Natural is
    begin
        return Length( this.val );
    end Length;

    ----------------------------------------------------------------------------

    function To_String( this : String_Value ) return String is
    begin
        return To_String( this.val );
    end To_String;

    ----------------------------------------------------------------------------

    overriding
    function Value_Input( stream : access Root_Stream_Type'Class ) return String_Value is
        this : aliased String_Value;
    begin
        String_Value'Read( stream, this );
        return this;
    end Value_Input;

    ----------------------------------------------------------------------------

    overriding
    procedure Value_Read( stream : access Root_Stream_Type'Class; this : out String_Value ) is
    begin
        this.val := To_Unbounded_String( String'Input( stream ) );
    end Value_Read;

    ----------------------------------------------------------------------------

    overriding
    procedure Value_Write( stream : access Root_Stream_Type'Class; this : String_Value ) is
    begin
        String'Output( stream, To_String( this.val ) );
    end Value_Write;

    ----------------------------------------------------------------------------

    function "&"( l, r : String_Value ) return String_Ptr'Class is
    begin
        return Create( To_String( l.val ) & To_String( r.val ) );
    end "&";

    --==========================================================================

    function As_String( ptr : Value_Ptr'Class ) return String_Ptr is
        result : String_Ptr;
    begin
        if ptr.target /= null then
            if ptr.target.Get_Type = V_STRING then
                result.Set( ptr.target );
            end if;
        end if;
        return result;
    end As_String;

    ----------------------------------------------------------------------------

    function As_Value( this : String_Ptr ) return Value_Ptr is
        result : Value_Ptr;
    begin
        result.Set( this.target );
        return result;
    end As_Value;

    ----------------------------------------------------------------------------

    function Get( this : String_Ptr ) return access String_Value'Class is
    begin
        return A_Naked_String(this.target);
    end Get;

    ----------------------------------------------------------------------------

    function "&"( l, r : String_Ptr ) return String_Ptr is begin return String_Ptr(l.Get.all & r.Get.all); end "&";

    ----------------------------------------------------------------------------

    function "="( l : String_Ptr; r : String           ) return Boolean is begin return l = Create( r ); end "=";
    function "="( l : String_Ptr; r : Unbounded_String ) return Boolean is begin return l = Create( r ); end "=";

    function  "<"( l : String_Ptr; r : String           ) return Boolean is begin return l < Create( r ); end "<";
    function  "<"( l : String_Ptr; r : Unbounded_String ) return Boolean is begin return l < Create( r ); end "<";
    function  ">"( l : String_Ptr; r : String           ) return Boolean is begin return l > Create( r ); end ">";
    function  ">"( l : String_Ptr; r : Unbounded_String ) return Boolean is begin return l > Create( r ); end ">";
    function "<="( l : String_Ptr; r : String           ) return Boolean is begin return l <= Create( r ); end "<=";
    function "<="( l : String_Ptr; r : Unbounded_String ) return Boolean is begin return l <= Create( r ); end "<=";
    function ">="( l : String_Ptr; r : String           ) return Boolean is begin return l >= Create( r ); end ">=";
    function ">="( l : String_Ptr; r : Unbounded_String ) return Boolean is begin return l >= Create( r ); end ">=";

    function "&"( l : String_Ptr; r : String           ) return String_Ptr is begin return l & Create( r ); end "&";
    function "&"( l : String_Ptr; r : Unbounded_String ) return String_Ptr is begin return l & Create( r ); end "&";

    -- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

    function  "="( l : String;           r : String_Ptr ) return Boolean is begin return Create( l ) = r; end "=";
    function  "="( l : Unbounded_String; r : String_Ptr ) return Boolean is begin return Create( l ) = r; end "=";
    function  "<"( l : String;           r : String_Ptr ) return Boolean is begin return Create( l ) < r; end "<";
    function  "<"( l : Unbounded_String; r : String_Ptr ) return Boolean is begin return Create( l ) < r; end "<";
    function  ">"( l : String;           r : String_Ptr ) return Boolean is begin return Create( l ) > r; end ">";
    function  ">"( l : Unbounded_String; r : String_Ptr ) return Boolean is begin return Create( l ) > r; end ">";
    function "<="( l : String;           r : String_Ptr ) return Boolean is begin return Create( l ) <= r; end "<=";
    function "<="( l : Unbounded_String; r : String_Ptr ) return Boolean is begin return Create( l ) <= r; end "<=";
    function ">="( l : String;           r : String_Ptr ) return Boolean is begin return Create( l ) >= r; end ">=";
    function ">="( l : Unbounded_String; r : String_Ptr ) return Boolean is begin return Create( l ) >= r; end ">=";

    function "&"( l : String;           r : String_Ptr ) return String_Ptr is begin return Create( l ) & r; end "&";
    function "&"( l : Unbounded_String; r : String_Ptr ) return String_Ptr is begin return Create( l ) & r; end "&";

    --==========================================================================

    function Escape( str : String ) return String is
        result : Unbounded_String;
    begin
        for i in str'Range loop
            case str(i) is
                when ASCII.HT => Append( result, "\t" );
                when ASCII.LF => Append( result, "\n" );
                when ASCII.CR => Append( result, "\f" );
                when '\'      => Append( result, "\\" );
                when '"'      => Append( result, "\""" );
                when others   => Append( result, str(i) );
            end case;
        end loop;
        return To_String( result );
    end Escape;

end Values.Strings;
