--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

-- The Simple_Button_Listener class provides a mechanism for registering any
-- method of a class, matching a certain prototype, as a button action handler.
-- The two handler prototypes are: a simple procedure in a target class, or a
-- procedure that also accepts a Button_Action as an argument.
--
-- Instantiate this package with the class and class-wide access type of an
-- object that will handle button actions.
generic
    type Target (<>) is tagged limited private;
package Widgets.Buttons.Simple_Listeners is

    type A_Handler1 is access
        procedure( object : not null access Target'Class );

    type A_Handler2 is access
        procedure( object : not null access Target'Class;
                   action : A_Button_Action );

    -- Invokes the 'handler' method of 'obj' for button actions matching 'id'.
    function Listener( id      : Action_Id;
                       obj     : access Target'Class;
                       handler : not null A_Handler1 ) return A_Button_Listener;
    pragma Postcondition( Listener'Result /= null );

    -- Invokes the 'handler' method of 'obj' for button actions matching 'id'.
    function Listener( id      : Action_Id;
                       obj     : access Target'Class;
                       handler : not null A_Handler2 ) return A_Button_Listener;
    pragma Postcondition( Listener'Result /= null );

    -- Invokes the 'handler' method of 'obj' for Click button actions.
    function Listener( obj     : access Target'Class;
                       handler : not null A_Handler1 ) return A_Button_Listener;
    pragma Postcondition( Listener'Result /= null );

    -- Invokes the 'handler' method of 'obj' for Click button actions.
    function Listener( obj     : access Target'Class;
                       handler : not null A_Handler2 ) return A_Button_Listener;
    pragma Postcondition( Listener'Result /= null );

private

    type Simple_Button_Listener is new Simple_Action_Listener and
                                       Button_Listener with
        record
            id       : Action_Id;
            object   : access Target'Class := null;
            handler1 : A_Handler1 := null;
            handler2 : A_Handler2 := null;
        end record;

    -- Invokes the handler procedure of the simple button listener's target.
    procedure Handle_Action( this   : access Simple_Button_Listener;
                             action : A_Button_Action );

end Widgets.Buttons.Simple_Listeners;
