--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

--with Debugging;                         use Debugging;
with Game_Views;                        use Game_Views;
with Mouse;                             use Mouse;
with Widgets.Containers.Windows;        use Widgets.Containers.Windows;

package body Widgets.Containers.Panels.Dialogs is

    overriding
    procedure Construct( this  : access Dialog;
                         view  : not null access Game_Views.Game_View'Class;
                         id    : String;
                         title : String;
                         icon  : String ) is
    begin
        Panel(this.all).Construct( view, id, title, icon );
        this.Add_Listener( A_Key_Listener(this) );
        this.Add_Listener( A_Visibility_Listener(this) );
        this.visible := False;
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure Delete( this : in out Dialog ) is
    begin
        this.Remove_Listener( A_Key_Listener(this) );
        Panel(this).Delete;
    end Delete;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Action( this    : access Dialog;
                             action  : A_Key_Action;
                             handled : out Boolean ) is
        pragma Unreferenced( this, action );
    begin
        -- Don't let any key events escape to the parent widget. This is to
        -- prevent the user from calling shortcuts on the window that are
        -- unrelated to this dialog.
        handled := True;
    end Handle_Action;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Action( this   : access Dialog;
                             action : A_Visibility_Action ) is
    begin
        if action.Get_Id = Change_Visibility then
            if action.Is_Visible then
                A_Dialog(this).On_Dialog_Show;
            else
                A_Dialog(this).On_Dialog_Hide;
            end if;
        end if;
    end Handle_Action;

    ----------------------------------------------------------------------------

    not overriding
    procedure Hide( this : access Dialog ) is
    begin
        if this.Is_Visible then
            this.Get_View.Pause_Game( pause => False );
            this.Get_Window.Set_Modal( null );
        end if;
    end Hide;

    ----------------------------------------------------------------------------

    overriding
    procedure On_Mouse_Move( this : access Dialog;
                             evt  : not null A_Mouse_Event ) is
        ex : constant Integer := evt.Get_X;
        ey : constant Integer := evt.Get_Y;
    begin
        if this.pressed and then
           (this.dragStartX >= 0 and then this.dragStartY >= 0) and then
           (ex /= this.dragStartX or else ey /= this.dragStartY)
        then
            this.Move( ex - this.dragStartX, ey - this.dragStartY );
        end if;
    end On_Mouse_Move;

    ----------------------------------------------------------------------------

    overriding
    procedure On_Mouse_Press( this : access Dialog;
                              evt  : not null A_Mouse_Button_Event ) is
    begin
        if evt.Get_Button = Mouse_Left then
            if evt.Get_Y < this.contentTop and then Length( this.title ) > 0 then
                this.dragStartX := evt.Get_X;
                this.dragStartY := evt.Get_Y;
            else
                this.dragStartX := -1;
                this.dragStartY := -1;
            end if;
        end if;
    end On_Mouse_Press;

    ----------------------------------------------------------------------------

    not overriding
    procedure Show( this : access Dialog ) is
    begin
        this.Get_View.Pause_Game( pause => True );
        this.Get_Window.Set_Modal( A_Widget(this) );
    end Show;

end Widgets.Containers.Panels.Dialogs;
