--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

--with Debugging;                         use Debugging;

with Widgets.Containers;                use Widgets.Containers;
with Widgets.Containers.Windows;        use Widgets.Containers.Windows;

package body Widgets.Containers.Pulldown_Menus is

    use Widget_Lists;

    ----------------------------------------------------------------------------

    function Create_Pulldown_Menu( view : not null access Game_Views.Game_View'Class;
                                   id   : String;
                                   text : String ) return A_Pulldown_Menu is
        this : A_Pulldown_Menu := new Pulldown_Menu;
    begin
        this.Construct( view, id, text );
        return this;
    exception
        when others =>
            Delete( A_Widget(this) );
            raise;
    end Create_Pulldown_Menu;

    ----------------------------------------------------------------------------

    not overriding
    procedure Construct( this : access Pulldown_Menu;
                         view : not null access Game_Views.Game_View'Class;
                         id   : String;
                         text : String ) is
    begin
        Container(this.all).Construct( view, id );
        this.border := Border_Raised;
        this.visible := False;
        this.text := To_Unbounded_String( text );
    end Construct;

    ----------------------------------------------------------------------------

    not overriding
    procedure Add_Menu_Item( this : access Pulldown_Menu;
                             item : in out A_Menu_Item ) is
    begin
        item.Prepend_Listener( A_Menu_Listener(this) );
        this.Add_Child( A_Widget(item) );
    end Add_Menu_Item;

    ----------------------------------------------------------------------------

    overriding
    procedure Draw_Content( this : access Pulldown_Menu ) is
    begin
        Themes.Get.Draw_Border( this.border,
                                0, 0, this.cwidth - 1, this.cheight - 1,
                                this.colors(Foreground) );
    end Draw_Content;

    ----------------------------------------------------------------------------

    overriding
    function Get_Min_Height( this : access Pulldown_Menu ) return Natural is
        height : Integer := 0;

        procedure Examine( pos : Widget_Lists.Cursor ) is
        begin
            height := height + Element( pos ).Get_Min_Height;
        end Examine;

    begin
        if this.minHeight > 0 then
            return this.minHeight;
        else
            this.children.Iterate( Examine'Access );
            return height + 2;
        end if;
    end Get_Min_Height;

    ----------------------------------------------------------------------------

    overriding
    function Get_Min_Width( this : access Pulldown_Menu ) return Natural is
        width : Integer := 0;

        procedure Examine( pos : Widget_Lists.Cursor ) is
            w : constant Integer := Element( pos ).Get_Min_Width;
        begin
            if w > width then
                width := w;
            end if;
        end Examine;

    begin
        if this.minWidth > 0 then
            return this.minWidth;
        else
            this.children.Iterate( Examine'Access );
            return width + 2;
        end if;
    end Get_Min_Width;

    ----------------------------------------------------------------------------

    not overriding
    function Get_Text( this : access Pulldown_Menu ) return String is
    begin
        return To_String( this.text );
    end Get_Text;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Action( this   : access Pulldown_Menu;
                             action : A_Menu_Action ) is
    begin
        if action.Get_Id = Menu_Selected or else action.Get_Id = Menu_Unselected then
            this.Get_Window.Pop_Popup;
        end if;
    end Handle_Action;

    ----------------------------------------------------------------------------

    overriding
    procedure Pack( this : access Pulldown_Menu ) is
        top : Positive := 1;

        procedure Examine( pos : Widget_Lists.Cursor ) is
            child   : constant A_Widget := Element( pos );
            cHeight : constant Natural := child.Get_Min_Height;
        begin
            child.Set_Layout_LTRB( 1, top, 1, -(top + cHeight - 1) );
            top := top + cHeight;
        end Examine;

    begin
        if this.Is_Rooted then
            this.children.Iterate( Examine'Access );
            Container(this.all).Pack;
        end if;
    end Pack;

    ----------------------------------------------------------------------------

    not overriding
    procedure Remove_Menu_Item( this : access Pulldown_Menu;
                                item : not null A_Menu_Item ) is
    begin
        item.Remove_Listener( A_Menu_Listener(this) );
        Container(this.all).Remove_Child( A_Widget(item) );
    end Remove_Menu_Item;

    ----------------------------------------------------------------------------

    overriding
    function To_String( this : access Pulldown_Menu ) return String is
    begin
        return "<" & A_Limited_Object(this).Get_Class_Name &
                     " '" & To_String( this.text ) & "'>";
    end To_String;

end Widgets.Containers.Pulldown_Menus;
