--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

--with Debugging;                         use Debugging;

package body Widgets.Layouts is

    function Create_Layout_Center( width, height : Natural := 0 ) return A_Layout is
        this : constant A_Layout_Center := new Layout_Center;
    begin
        this.Construct( width, height );
        return A_Layout(this);
    end Create_Layout_Center;

    ----------------------------------------------------------------------------

    overriding
    procedure Apply( this : access Layout_Center; widget : not null A_Widget ) is
        orig_x1 : constant Integer := widget.x1;
        orig_y1 : constant Integer := widget.y1;
        orig_x2 : constant Integer := widget.x2;
        orig_y2 : constant Integer := widget.y2;
        w       : Integer := this.width;
        h       : Integer := this.height;
    begin
        if w = 0 then
            w := widget.Get_Min_Width;
        end if;
        if h = 0 then
            h := widget.Get_Min_Height;
        end if;

        widget.x1 := orig_x1 + ((orig_x2 - orig_x1 + 1) / 2) - (w / 2);
        widget.x2 := widget.x1 + w - 1;
        widget.y1 := orig_y1 + ((orig_y2 - orig_y1 + 1) / 2) - (h / 2);
        widget.y2 := widget.y1 + h - 1;
    end Apply;

    ----------------------------------------------------------------------------

    not overriding
    procedure Construct( this : access Layout_Center; width, height : Natural ) is
    begin
        Layout(this.all).Construct;
        this.width := width;
        this.height := height;
    end Construct;

    --==========================================================================

    function Create_Layout_CenterH( width       : Natural;
                                    top, bottom : Integer ) return A_Layout is
        this : constant A_Layout_CenterH := new Layout_CenterH;
    begin
        this.Construct( width, top, bottom );
        return A_Layout(this);
    end Create_Layout_CenterH;

    ----------------------------------------------------------------------------

    overriding
    procedure Apply( this : access Layout_CenterH; widget : not null A_Widget ) is
        orig_x1 : constant Integer := widget.x1;
        orig_y1 : constant Integer := widget.y1;
        orig_x2 : constant Integer := widget.x2;
        orig_y2 : constant Integer := widget.y2;
    begin
        widget.x1 := orig_x1 + (orig_x2 - orig_x1) / 2 - this.width / 2;
        widget.x2 := orig_x2 - (orig_x2 - orig_x1) / 2 + this.width / 2;
        if this.top >= 0 then
            widget.y1 := orig_y1 + this.top;
        else
            widget.y1 := orig_y2 + this.top;
        end if;
        if this.bottom >= 0 then
            widget.y2 := orig_y2 - this.bottom;
        else
            widget.y2 := orig_y1 - this.bottom;
        end if;
    end Apply;

    ----------------------------------------------------------------------------

    not overriding
    procedure Construct( this        : access Layout_CenterH;
                         width       : Natural;
                         top, bottom : Integer ) is
    begin
        Layout(this.all).Construct;
        this.width := width;
        this.top := top;
        this.bottom := bottom;
    end Construct;

    --==========================================================================

    function Create_Layout_CenterHY( width  : Natural;
                                     y1, y2 : Integer ) return A_Layout is
        this : constant A_Layout_CenterHY := new Layout_CenterHY;
    begin
        this.Construct( width, y1, y2 );
        return A_Layout(this);
    end Create_Layout_CenterHY;

    ----------------------------------------------------------------------------

    overriding
    procedure Apply( this : access Layout_CenterHY; widget : not null A_Widget ) is
        orig_x1 : constant Integer := widget.x1;
        orig_y1 : constant Integer := widget.y1;
        orig_x2 : constant Integer := widget.x2;
        w       : Natural := this.width;
    begin
        if w = 0 then
            w := widget.Get_Min_Width;
        end if;
        widget.x1 := orig_x1 + (orig_x2 - orig_x1) / 2 - w / 2;
        widget.x2 := orig_x2 - (orig_x2 - orig_x1) / 2 + w / 2;
        widget.y1 := orig_y1 + this.y1;
        widget.y2 := Integer'Max( widget.y1, orig_y1 + this.y2 );
    end Apply;

    ----------------------------------------------------------------------------

    not overriding
    procedure Construct( this   : access Layout_CenterHY;
                         width  : Natural;
                         y1, y2 : Integer ) is
    begin
        Layout(this.all).Construct;
        this.width := width;
        this.y1 := y1;
        this.y2 := y2;
    end Construct;

    --==========================================================================

    function Create_Layout_LTRB( left, top, right, bottom : Integer ) return A_Layout is
        this : constant A_Layout_LTRB := new Layout_LTRB;
    begin
        this.Construct( left, top, right, bottom );
        return A_Layout(this);
    end Create_Layout_LTRB;

    ----------------------------------------------------------------------------

    overriding
    procedure Apply( this : access Layout_LTRB; widget : not null A_Widget ) is
        orig_x1 : constant Integer := widget.x1;
        orig_y1 : constant Integer := widget.y1;
        orig_x2 : constant Integer := widget.x2;
        orig_y2 : constant Integer := widget.y2;
    begin
        if this.left >= 0 then
            widget.x1 := orig_x1 + this.left;
        else
            widget.x1 := orig_x2 + this.left;
        end if;
        if this.top >= 0 then
            widget.y1 := orig_y1 + this.top;
        else
            widget.y1 := orig_y2 + this.top;
        end if;
        if this.right >= 0 then
            widget.x2 := orig_x2 - this.right;
        else
            widget.x2 := orig_x1 - this.right;
        end if;
        if this.bottom >= 0 then
            widget.y2 := orig_y2 - this.bottom;
        else
            widget.y2 := orig_y1 - this.bottom;
        end if;
    end Apply;

    ----------------------------------------------------------------------------

    not overriding
    procedure Construct( this : access Layout_LTRB; left, top, right, bottom : Integer ) is
    begin
        Layout(this.all).Construct;
        this.left := left;
        this.top := top;
        this.right := right;
        this.bottom := bottom;
    end Construct;

    --==========================================================================

    function Create_Layout_LTWH( left, top     : Integer;
                                 width, height : Natural ) return A_Layout is
        this : constant A_Layout_LTWH := new Layout_LTWH;
    begin
        this.Construct( left, top, width, height );
        return A_Layout(this);
    end Create_Layout_LTWH;

    ----------------------------------------------------------------------------

    overriding
    procedure Apply( this : access Layout_LTWH; widget : not null A_Widget ) is
        orig_x1 : constant Integer := widget.x1;
        orig_y1 : constant Integer := widget.y1;
        orig_x2 : constant Integer := widget.x2;
        orig_y2 : constant Integer := widget.y2;
        w       : Integer := this.width;
        h       : Integer := this.height;
    begin
        if w = 0 then
            w := widget.Get_Min_Width;
        end if;
        if h = 0 then
            h := widget.Get_Min_Height;
        end if;

        if this.left >= 0 then
            widget.x1 := orig_x1 + this.left;
            widget.x2 := widget.x1 + w - 1;
        else
            widget.x2 := orig_x2 + this.left;
            widget.x1 := widget.x2 - w + 1;
        end if;
        if this.top >= 0 then
            widget.y1 := orig_y1 + this.top;
            widget.y2 := widget.y1 + h - 1;
        else
            widget.y2 := orig_y2 + this.top;
            widget.y1 := widget.y2 - h + 1;
        end if;
    end Apply;

    ----------------------------------------------------------------------------

    not overriding
    procedure Construct( this   : access Layout_LTWH;
                         left,
                         top    : Integer;
                         width,
                         height : Natural ) is
    begin
        Layout(this.all).Construct;
        this.left := left;
        this.top := top;
        this.width := width;
        this.height := height;
    end Construct;

    --==========================================================================

    function Create_Layout_XYWH( x, y          : Integer;
                                 width, height : Natural ) return A_Layout is
        this : constant A_Layout_XYWH := new Layout_XYWH;
    begin
        this.Construct( x, y, width, height );
        return A_Layout(this);
    end Create_Layout_XYWH;

    ----------------------------------------------------------------------------

    overriding
    procedure Apply( this : access Layout_XYWH; widget : not null A_Widget ) is
        w : Integer := this.width;
        h : Integer := this.height;
    begin
        if w = 0 then
            w := widget.Get_Min_Width;
        end if;
        if h = 0 then
            h := widget.Get_Min_Height;
        end if;

        widget.x1 := widget.x1 + this.x;
        widget.x2 := widget.x1 + w - 1;

        widget.y1 := widget.y1 + this.y;
        widget.y2 := widget.y1 + h - 1;
    end Apply;

    ----------------------------------------------------------------------------

    not overriding
    procedure Construct( this   : access Layout_XYWH;
                         x, y   : Integer;
                         width,
                         height : Natural ) is
    begin
        Layout(this.all).Construct;
        this.x := x;
        this.y := y;
        this.width := width;
        this.height := height;
    end Construct;

    --==========================================================================

    procedure Delete( this : in out A_Layout ) is
    begin
        Delete( A_Object(this) );
    end Delete;

end Widgets.Layouts;
