--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

with Drawing;                           use Drawing;
with Mouse;                             use Mouse;
with Themes.Palette;                    use Themes.Palette;

package body Widgets.Menu_Items.Menu_Checkboxes is

    function Create_Menu_Checkbox( view    : not null access Game_Views.Game_View'Class;
                                   id      : String;
                                   text    : String := "";
                                   checked : Boolean := False
                                 ) return A_Menu_Checkbox is
        this : A_Menu_Checkbox := new Menu_Checkbox;
    begin
        this.Construct( view, id, text, checked );
        return this;
    exception
        when others =>
            Delete( A_Widget(this) );
            raise;
    end Create_Menu_Checkbox;

    ----------------------------------------------------------------------------

    not overriding
    procedure Construct( this    : access Menu_Checkbox;
                         view    : not null access Game_Views.Game_View'Class;
                         id      : String;
                         text    : String;
                         checked : Boolean ) is
    begin
        Menu_Item(this.all).Construct( view, id, text );
        this.checked := checked;
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure Draw_Content( this : access Menu_Checkbox ) is
        xpad    : constant Integer := Themes.Get.Get_Text_Width( PADCHAR, To_String( this.fontName ), this.fontSize );
        boxsize : constant Integer := this.Box_Size;
    begin
        Menu_Item(this.all).Draw_Content;

        -- draw the check mark (icon or square)
        if this.checked then
            Rectfill( xpad + 2, this.cheight / 2 - boxsize / 2 + 2,
                      xpad + boxsize - 3, this.cheight / 2 + boxsize / 2 - 3,
                      Hard_Contrast( this.Get_Background ) );
        else
            Rect( xpad + 2, this.cheight / 2 - boxsize / 2 + 2,
                  xpad + boxsize - 3, this.cheight / 2 + boxsize / 2 - 3,
                  Hard_Contrast( this.Get_Background ) );
        end if;
    end Draw_Content;

    ----------------------------------------------------------------------------

    overriding
    procedure On_Mouse_Release( this : access Menu_Checkbox;
                                evt  : not null A_Mouse_Button_Event ) is
        x : constant Integer := evt.Get_X;
        y : constant Integer := evt.Get_Y;
    begin
        if evt.Get_Button = Mouse_Left then
            if x >= 0 and then y >= 0 and then
               x < this.Get_Width and then y < this.Get_Height
            then
                if not this.checked then
                    this.checked := True;
                    this.Dispatch_Action( Menu_Selected );
                else
                    this.checked := False;
                    this.Dispatch_Action( Menu_Unselected );
                end if;
            end if;
            this.pressed := False;
        end if;
    end On_Mouse_Release;

end Widgets.Menu_Items.Menu_Checkboxes;
