--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

-- The Simple_Menu_Listener class provides a mechanism for registering
-- any method of a class, matching a certain prototype, as a menu action handler.
--
-- Instantiate this package with the class and class-wide access type of an
-- object that will handle menu actions.
generic
    type Target (<>) is tagged limited private;
package Widgets.Menu_Items.Simple_Listeners is

    type A_Handler is access
        procedure( object : not null access Target'Class;
                   action : A_Menu_Action );

    -- Invokes the 'handler' method of 'obj' for menu actions matching 'id'.
    function Listener( id      : Action_Id;
                       obj     : access Target'Class;
                       handler : not null A_Handler ) return A_Menu_Listener;
    pragma Postcondition( Listener'Result /= null );

    -- Invokes the 'handler' method of 'obj' for Menu_Selected actions.
    function Listener( obj     : access Target'Class;
                       handler : not null A_Handler ) return A_Menu_Listener;

private

    type Simple_Menu_Listener is new Simple_Action_Listener and
                                     Menu_Listener with
        record
            id      : Action_Id;
            object  : access Target'Class := null;
            handler : A_Handler := null;
        end record;

    -- Invokes the handler procedure of the simple menu listener's target.
    procedure Handle_Action( this   : access Simple_Menu_Listener;
                             action : A_Menu_Action );

end Widgets.Menu_Items.Simple_Listeners;
