--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

with Applications;
with Allegro.Bitmaps;                   use Allegro.Bitmaps;
with Allegro.Color;                     use Allegro.Color;
--with Debugging;                         use Debugging;
with Drawing;                           use Drawing;
with GNAT.Directory_Operations;
with Preferences;                       use Preferences;
with Resources.Images;                  use Resources.Images;
with Support.Paths;                     use Support.Paths;

pragma Elaborate_All( Applications );
pragma Elaborate_All( Preferences );
pragma Elaborate_All( Support.Paths );

package body Applications.Gui.Games.Ked is

    KEEN_DATA_DIR : constant String := App_Data_Directory &
                                       "Wellwood" & Slash &
                                       "Keen Galaxy" & Slash;

    KEEN_USER_DIR : constant String := Home_Directory &
                                       "Keen Galaxy" & Slash;

    -- END PRIVATE DECLARATIONS --
    --==========================================================================

    function Create_Application return A_Application is
        this : constant A_Game_Application := new Ked_Application;
    begin
        this.Construct( company         => "Kevin Wellwood",
                        name            => "ked",
                        userDir         => KEEN_USER_DIR,
                        defaultWidth    => 1024,
                        defaultHeight   => 640,
                        defaultScale    => 0,
                        defaultWindowed => True );
        this.interactive := True;
        this.useMouse := True;
        GNAT.Directory_Operations.Change_Dir( Execution_Directory );

        return A_Application(this);
    end Create_Application;

    ----------------------------------------------------------------------------

    overriding
    procedure Draw_Loading_Screen( this : access Ked_Application ) is
        icon16,
        icon32,
        icon64  : A_Allegro_Bitmap;
        loading : A_Allegro_Bitmap;
    begin
        icon16 := Load_Image( "ked-16.png", "resources" );
        icon32 := Load_Image( "ked-32.png", "resources" );
        icon64 := Load_Image( "ked-64.png", "resources" );
        Al_Set_Display_Icons( this.display, icons => (icon16, icon32, icon64) );
        Al_Destroy_Bitmap( icon16 );
        Al_Destroy_Bitmap( icon32 );
        Al_Destroy_Bitmap( icon64 );

        loading := Load_Image( "loading.png", "resources" );
        if loading /= null then
            Clear_To_Color( Al_Map_RGB( 92, 92, 92 ) );
            Draw_Bitmap( loading,
                         (Get_Target_Width - Al_Get_Bitmap_Width( loading )) / 2,
                         (Get_Target_Height - Al_Get_Bitmap_Height( loading )) / 2 );
            Al_Destroy_Bitmap( loading );
        end if;
    end Draw_Loading_Screen;

begin

    Register_Allocator( Create_Application'Access );

    Preferences.Set_Default( "application", "max_logic_fps", 80 );
    Preferences.Set_Default( "application", "max_render_fps", 60 );
    Preferences.Set_Default( "application", "editing_mode", True );
    Preferences.Set_Default( "application", "license", "credits" & Slash );
    Preferences.Set_Default( "application", "media", KEEN_DATA_DIR & "media" & Slash );

end Applications.Gui.Games.Ked;
