--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

with Ada.Exceptions;                    use Ada.Exceptions;
with Debugging;                         use Debugging;
with Engine_Debugging;                  use Engine_Debugging;

package body Game_Views.Ked.Actions is

    procedure Action_Import( view : not null A_Ked_View ) is
    begin
        view.Do_Dialog_Action( DA_Import, saveChanges => True );
    end Action_Import;

    ----------------------------------------------------------------------------

    procedure Action_New( view : not null A_Ked_View ) is
    begin
        view.Do_Dialog_Action( DA_New, saveChanges => True );
    end Action_New;

    ----------------------------------------------------------------------------

    procedure Action_Open( view : not null A_Ked_View ) is
    begin
        view.Do_Dialog_Action( DA_Open, saveChanges => True );
    end Action_Open;

    ----------------------------------------------------------------------------

    procedure Action_Quit( view : not null A_Ked_View ) is
    begin
        view.Do_Dialog_Action( DA_Quit, saveChanges => True );
    end Action_Quit;

    ----------------------------------------------------------------------------

    procedure Action_Saveas( view : not null A_Ked_View ) is
    begin
        if not view.Is_World_Loaded then
            return;
        end if;

        view.Do_Dialog_Action( DA_Save, saveChanges => False );
    end Action_Saveas;

    ----------------------------------------------------------------------------

    procedure Action_Save( view : not null A_Ked_View ) is
        filename : constant String := view.Get_Filename;
    begin
        if not view.Is_World_Loaded then
            return;
        end if;

        if filename'Length > 0 then
            begin
                view.Save_World( filename );
            exception
                when e : others =>
                    Dbg( "Error saving world: " & Exception_Message( e ), D_VIEW, ERROR );
            end;
        else
            Action_Saveas( view );
        end if;
    end Action_Save;

end Game_Views.Ked.Actions;
