--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

with Actions;                           use Actions;
with Allegro.Keyboard;                  use Allegro.Keyboard;
with Game_Views.Ked.Actions;
with Keyboard;                          use Keyboard;
with Preferences;                       use Preferences;
with Widgets.Containers;                use Widgets.Containers;
with Widgets.Containers.Panels.Dialogs; use Widgets.Containers.Panels.Dialogs;
with Widgets.Containers.Pulldown_Menus; use Widgets.Containers.Pulldown_Menus;
with Widgets.Menubars;                  use Widgets.Menubars;
with Widgets.Menu_Items;                use Widgets.Menu_Items;
with Widgets.Menu_Items.Menu_Checkboxes;use Widgets.Menu_Items.Menu_Checkboxes;

package body Game_Views.Ked.Menubar is

    -- height of the menubar in pixels
    MENUBAR_HEIGHT : constant := 22;

    ----------------------------------------------------------------------------

    -- file menu

    procedure File_New( view   : not null access Ked_View'Class;
                        action : A_Menu_Action );

    procedure File_Open( view   : not null access Ked_View'Class;
                         action : A_Menu_Action );

    procedure File_Save( view   : not null access Ked_View'Class;
                         action : A_Menu_Action );

    procedure File_Saveas( view   : not null access Ked_View'Class;
                           action : A_Menu_Action );

    procedure File_Import( view   : not null access Ked_View'Class;
                           action : A_Menu_Action );

    procedure File_Quit( view   : not null access Ked_View'Class;
                         action : A_Menu_Action );

    -- world menu

    procedure World_Properties( view   : not null access Ked_View'Class;
                                action : A_Menu_Action );

    procedure World_Resize( view   : not null access Ked_View'Class;
                            action : A_Menu_Action );

    -- options menu

    procedure Options_Gridsnap( view   : not null access Ked_View'Class;
                                action : A_Menu_Action );

    procedure Options_PinkScene( view   : not null access Ked_View'Class;
                                 action : A_Menu_Action );

    procedure Options_Outlines( view   : not null access Ked_View'Class;
                                action : A_Menu_Action );

    -- help menu

    procedure Help_About( view   : not null access Ked_View'Class;
                          action : A_Menu_Action );

    -- shortcuts

    procedure Shortcut_File_Import( view   : not null access Ked_View'Class;
                                    action : A_Key_Action );

    procedure Shortcut_File_New( view   : not null access Ked_View'Class;
                                 action : A_Key_Action );

    procedure Shortcut_File_Open( view   : not null access Ked_View'Class;
                                  action : A_Key_Action );

    procedure Shortcut_File_Save( view   : not null access Ked_View'Class;
                                  action : A_Key_Action );

    -- END PRIVATE DECLARATIONS --
    --==========================================================================

    procedure File_New( view   : not null access Ked_View'Class;
                        action : A_Menu_Action ) is
        pragma Unreferenced( action );
    begin
        Game_Views.Ked.Actions.Action_New( A_Ked_View(view) );
    end File_New;

    ----------------------------------------------------------------------------

    procedure File_Open( view   : not null access Ked_View'Class;
                         action : A_Menu_Action ) is
        pragma Unreferenced( action );
    begin
        Game_Views.Ked.Actions.Action_Open( A_Ked_View(view) );
    end File_Open;

    ----------------------------------------------------------------------------

    procedure File_Save( view   : not null access Ked_View'Class;
                         action : A_Menu_Action ) is
        pragma Unreferenced( action );
    begin
        Game_Views.Ked.Actions.Action_Save( A_Ked_View(view) );
    end File_Save;

    ----------------------------------------------------------------------------

    procedure File_Saveas( view   : not null access Ked_View'Class;
                           action : A_Menu_Action ) is
        pragma Unreferenced( action );
    begin
        Game_Views.Ked.Actions.Action_Saveas( A_Ked_View(view) );
    end File_Saveas;

    ----------------------------------------------------------------------------

    procedure File_Import( view   : not null access Ked_View'Class;
                           action : A_Menu_Action ) is
        pragma Unreferenced( action );
    begin
        view.Do_Dialog_Action( DA_Import, saveChanges => True );
    end File_Import;

    ----------------------------------------------------------------------------

    procedure File_Quit( view   : not null access Ked_View'Class;
                         action : A_Menu_Action ) is
        pragma Unreferenced( action );
    begin
        view.Do_Dialog_Action( DA_Quit, saveChanges => True );
    end File_Quit;

    ----------------------------------------------------------------------------

    procedure World_Properties( view   : not null access Ked_View'Class;
                                action : A_Menu_Action ) is
        pragma Unreferenced( action );
    begin
        if view.Is_World_Loaded then
            A_Dialog(view.Get_Widget( "pnlProp" )).Show;
        end if;
    end World_Properties;

    ----------------------------------------------------------------------------

    procedure World_Resize( view   : not null access Ked_View'Class;
                            action : A_Menu_Action ) is
        pragma Unreferenced( action );
    begin
        if view.Is_World_Loaded then
            A_Dialog(view.Get_Widget( "pnlResize" )).Show;
        end if;
    end World_Resize;

    ----------------------------------------------------------------------------

    procedure Options_Gridsnap( view   : not null access Ked_View'Class;
                                action : A_Menu_Action ) is
        pragma Unreferenced( view );
    begin
        if action.Get_Id = Menu_Selected then
            Set_Pref( "gridsnap", True );
        elsif action.Get_Id = Menu_Unselected then
            Set_Pref( "gridsnap", False );
        end if;
    end Options_Gridsnap;

    ----------------------------------------------------------------------------

    procedure Options_PinkScene( view   : not null access Ked_View'Class;
                                 action : A_Menu_Action ) is
         pragma Unreferenced( view );
    begin
        if action.Get_Id = Menu_Selected then
            Set_Pref( "pinkscene", True );
        elsif action.Get_Id = Menu_Unselected then
            Set_Pref( "pinkscene", False );
        end if;
    end Options_PinkScene;

    ----------------------------------------------------------------------------

    procedure Options_Outlines( view   : not null access Ked_View'Class;
                                action : A_Menu_Action ) is
         pragma Unreferenced( view );
    begin
        if action.Get_Id = Menu_Selected then
            Set_Pref( "outlines", True );
        elsif action.Get_Id = Menu_Unselected then
            Set_Pref( "outlines", False );
        end if;
    end Options_Outlines;

    ----------------------------------------------------------------------------

    procedure Help_About( view   : not null access Ked_View'Class;
                          action : A_Menu_Action ) is
         pragma Unreferenced( action );
    begin
        A_Dialog(view.Get_Widget( "pnlAbout" )).Show;
    end Help_About;

    ----------------------------------------------------------------------------

    procedure Shortcut_File_Import( view   : not null access Ked_View'Class;
                                    action : A_Key_Action ) is
        pragma Unreferenced( action );
    begin
        Game_Views.Ked.Actions.Action_Import( A_Ked_View(view) );
    end Shortcut_File_Import;

    ----------------------------------------------------------------------------

    procedure Shortcut_File_New( view   : not null access Ked_View'Class;
                                 action : A_Key_Action ) is
        pragma Unreferenced( action );
    begin
        Game_Views.Ked.Actions.Action_New( A_Ked_View(view) );
    end Shortcut_File_New;

    ----------------------------------------------------------------------------

    procedure Shortcut_File_Open( view   : not null access Ked_View'Class;
                                  action : A_Key_Action ) is
        pragma Unreferenced( action );
    begin
        Game_Views.Ked.Actions.Action_Open( A_Ked_View(view) );
    end Shortcut_File_Open;

    ----------------------------------------------------------------------------

    procedure Shortcut_File_Save( view   : not null access Ked_View'Class;
                                  action : A_Key_Action ) is
        pragma Unreferenced( action );
    begin
        Game_Views.Ked.Actions.Action_Save( A_Ked_View(view) );
    end Shortcut_File_Save;

    --==========================================================================
    -- Public procedures

    procedure Make_Menubar( view : not null A_Ked_View;
                            win  : not null A_Window ) is
        loaded           : constant Boolean := view.Is_World_Loaded;
        mb               : A_Menubar;
        fileMenu,
        worldMenu,
        optionsMenu,
        helpMenu         : A_Pulldown_Menu;
        fileNew,
        fileOpen,
        fileImport,
        fileSave,
        fileSaveas,
        fileQuit,
        worldProperties,
        worldResize      : A_Menu_Item;
        optionsGridsnap,
        optionsPinkScene,
        optionsOutlines  : A_Menu_Checkbox;
        helpAbout        : A_Menu_Item;
    begin
        -- File menu --
        fileNew := Create_Menu_Item( view, "miFileNew", "New..." );
        fileOpen := Create_Menu_Item( view, "miFileOpen", "Open..." );
        fileImport := Create_Menu_Item( view, "miFileImport", "Import..." );
        fileSave := Create_Menu_Item( view, "miFileSave", "Save" );
        fileSave.Set_Enabled( loaded );
        fileSaveas := Create_Menu_Item( view, "miFileSaveas", "Save As..." );
        fileSaveas.Set_Enabled( loaded );
        fileQuit := Create_Menu_Item( view, "miFileQuit", "Quit" );

        fileNew.Add_Listener( Listener( view, File_New'Access ) );
        fileOpen.Add_Listener( Listener( view, File_Open'Access ) );
        fileImport.Add_Listener( Listener( view, File_Import'Access ) );
        fileSave.Add_Listener( Listener( view, File_Save'Access ) );
        fileSaveas.Add_Listener( Listener( view, File_Saveas'Access ) );
        fileQuit.Add_Listener( Listener( view, File_Quit'Access ) );

        fileMenu := Create_Pulldown_Menu( view, "pdmFile", "File" );
        fileMenu.Add_Menu_Item( fileNew );
        fileMenu.Add_Menu_Item( fileOpen );
        fileMenu.Add_Menu_Item( fileImport );
        fileMenu.Add_Menu_Item( fileSave );
        fileMenu.Add_Menu_Item( fileSaveas );
        fileMenu.Add_Menu_Item( fileQuit );

        -- World menu --
        worldProperties := Create_Menu_Item( view, "miWorldProp", "Properties..." );
        worldProperties.Set_Enabled( loaded );
        worldResize := Create_Menu_Item( view, "miWorldResize", "Resize..." );
        worldResize.Set_Enabled( loaded );

        worldProperties.Add_Listener( Listener( view, World_Properties'Access ) );
        worldResize.Add_Listener( Listener( view, World_Resize'Access ) );

        worldMenu := Create_Pulldown_Menu( view, "pdmWorld", "World" );
        worldMenu.Add_Menu_Item( worldProperties );
        worldMenu.Add_Menu_Item( worldResize );

        -- Options menu --
        optionsGridsnap := Create_Menu_Checkbox( view, "miOptionsGrid", "Snap Items to Grid", checked => Get_Pref( "gridsnap" ) );
        optionsPinkScene := Create_Menu_Checkbox( view, "miOptionsPink", "Pink Scene Background", checked => Get_Pref( "pinkscene" ) );
        optionsOutlines := Create_Menu_Checkbox( view, "miOptionsOutlines", "Layer Outlines", checked => Get_Pref( "outlines" ) );

        optionsGridsnap.Add_Listener( Listener( Menu_Selected,    view, Options_Gridsnap'Access ) );
        optionsGridsnap.Add_Listener( Listener( Menu_Unselected,  view, Options_Gridsnap'Access ) );
        optionsPinkScene.Add_Listener( Listener( Menu_Selected,   view, Options_PinkScene'Access ) );
        optionsPinkScene.Add_Listener( Listener( Menu_Unselected, view, Options_PinkScene'Access ) );
        optionsOutlines.Add_Listener( Listener( Menu_Selected,    view, Options_Outlines'Access ) );
        optionsOutlines.Add_Listener( Listener( Menu_Unselected,  view, Options_Outlines'Access ) );

        optionsMenu := Create_Pulldown_Menu( view, "pdmOptions", "Options" );
        optionsMenu.Add_Menu_Item( A_Menu_Item(optionsGridsnap) );
        optionsMenu.Add_Menu_Item( A_Menu_Item(optionsPinkScene) );
        optionsMenu.Add_Menu_Item( A_Menu_Item(optionsOutlines) );

        -- Help menu --
        helpAbout := Create_Menu_Item( view, "miHelpAbout", "About..." );
        helpAbout.Add_Listener( Listener( view, Help_About'Access ) );

        helpMenu := Create_Pulldown_Menu( view, "pdmHelp", "Help" );
        helpMenu.Add_Menu_Item( helpAbout );

        -- add all pulldown menus to menubar
        mb := Create_Menubar( view, "menubar" );
        mb.Add( A_Widget(fileMenu) );
        mb.Add( A_Widget(worldMenu) );
        mb.Add( A_Widget(optionsMenu) );
        mb.Add( A_Widget(helpMenu) );
        win.Set_Menubar( mb );

        -- shortcuts for menu items
#if WINDOWS then
        win.Add_Listener( Listener( ALLEGRO_KEY_N, (CTRL=>Yes, others=>No), Key_Press, view, Shortcut_File_New'Access ) );
        win.Add_Listener( Listener( ALLEGRO_KEY_O, (CTRL=>Yes, others=>No), Key_Press, view, Shortcut_File_Open'Access ) );
        win.Add_Listener( Listener( ALLEGRO_KEY_I, (CTRL=>Yes, others=>No), Key_Press, view, Shortcut_File_Import'Access ) );
        win.Add_Listener( Listener( ALLEGRO_KEY_S, (CTRL=>Yes, others=>No), Key_Press, view, Shortcut_File_Save'Access ) );
#elsif OSX then
        win.Add_Listener( Listener( ALLEGRO_KEY_N, (CMD=>Yes, others=>No), Key_Press, view, Shortcut_File_New'Access ) );
        win.Add_Listener( Listener( ALLEGRO_KEY_O, (CMD=>Yes, others=>No), Key_Press, view, Shortcut_File_Open'Access ) );
        win.Add_Listener( Listener( ALLEGRO_KEY_I, (CMD=>Yes, others=>No), Key_Press, view, Shortcut_File_Import'Access ) );
        win.Add_Listener( Listener( ALLEGRO_KEY_S, (CMD=>Yes, others=>No), Key_Press, view, Shortcut_File_Save'Access ) );
#end if;
    exception
        when others =>
            Delete( A_Widget(mb) );
            Delete( A_Widget(fileMenu) );
            Delete( A_Widget(worldMenu) );
            Delete( A_Widget(optionsMenu) );
            Delete( A_Widget(helpMenu) );
            Delete( A_Widget(fileNew) );
            Delete( A_Widget(fileOpen) );
            Delete( A_Widget(fileImport) );
            Delete( A_Widget(fileSave) );
            Delete( A_Widget(fileSaveas) );
            Delete( A_Widget(fileQuit) );
            Delete( A_Widget(worldProperties) );
            Delete( A_Widget(worldResize) );
            Delete( A_Widget(optionsGridsnap) );
            Delete( A_Widget(optionsPinkScene) );
            Delete( A_Widget(optionsOutlines) );
            Delete( A_Widget(helpAbout) );
            raise;
    end Make_Menubar;

    ----------------------------------------------------------------------------

    procedure Handle_New_Map( view : not null A_Ked_View ) is
        loaded : constant Boolean := view.Is_World_Loaded;
    begin
        view.Get_Widget( "miFileSave" ).Set_Enabled( loaded );
        view.Get_Widget( "miFileSaveas" ).Set_Enabled( loaded );
        view.Get_Widget( "miWorldProp" ).Set_Enabled( loaded );
        view.Get_Widget( "miWorldResize" ).Set_Enabled( loaded );
    end Handle_New_Map;

    ----------------------------------------------------------------------------

    function Get_Menubar_Height return Natural is
    begin
        return MENUBAR_HEIGHT;
    end Get_Menubar_Height;

end Game_Views.Ked.Menubar;
