--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

with Actions;                           use Actions;
with Allegro.Keyboard;                  use Allegro.Keyboard;
with Keyboard;                          use Keyboard;
with Preferences;                       use Preferences;
with Support;                           use Support;
with Themes;                            use Themes;
with Themes.Palette;                    use Themes.Palette;
with Values.Numbers;                    use Values.Numbers;
with Widgets.Buttons;                   use Widgets.Buttons;
with Widgets.Buttons.Checkboxes;        use Widgets.Buttons.Checkboxes;
with Widgets.Buttons.Toggles;           use Widgets.Buttons.Toggles;
with Widgets.Containers.Panels;         use Widgets.Containers.Panels;
with Widgets.Containers.Scenes.Ked;     use Widgets.Containers.Scenes.Ked;

package body Game_Views.Ked.Toolbar.Layers_Menu is

    btnSize   : constant := 22;
    rowHeight : constant := 20;

    ----------------------------------------------------------------------------

    -- Sets the currently active layer for tool operations
    procedure Set_Active_Layer( view   : not null access Ked_View'Class;
                                action : A_Button_Action );

    procedure Shortcut_Set_Active_Layer( view   : not null access Ked_View'Class;
                                         action : A_Key_Action );

    procedure Shortcut_Show_Layer( view   : not null access Ked_View'Class;
                                   action : A_Key_Action );

    procedure Show_Clipping( view   : not null access Ked_View'Class;
                             action : A_Button_Action );

    procedure Show_Layer( view   : not null access Ked_View'Class;
                          action : A_Button_Action );

    procedure Show_Layers( view   : not null access Ked_View'Class;
                           action : A_Button_Action );

    -- END PRIVATE DECLARATIONS --
    --==========================================================================

    procedure Make_Menu( view : not null A_Ked_View;
                         win  : not null A_Window ) is
        button : A_Button;
        panel  : A_Panel;
    begin
        button := Create_Toggle_Button( view, "btnLayer", icon => "layer_icon" );
        button.Set_Layout_LTRB( -(btnSize-1), -(btnSize-1), 0, 0 );
        button.Set_Focusable( False );
        button.Set_Enabled( False );
        button.Add_Listener( Listener( Press, view, Show_Layers'Access ) );
        button.Add_Listener( Listener( Release, view, Show_Layers'Access ) );
        win.Add_Widget( A_Widget(button) );

        panel := Create_Panel( view, "pnlLayers" );
        panel.Set_Layout_LTRB( 0, 0, -1, -1 );
        panel.Set_Visible( False );
        win.Add_Widget( A_Widget(panel) );

        view.layerGrp := Create_Button_Group;

        win.Add_Listener( Listener( ALLEGRO_KEY_1, (others=>No), Key_Press, view, Shortcut_Set_Active_Layer'Access ) );
        win.Add_Listener( Listener( ALLEGRO_KEY_2, (others=>No), Key_Press, view, Shortcut_Set_Active_Layer'Access ) );
        win.Add_Listener( Listener( ALLEGRO_KEY_3, (others=>No), Key_Press, view, Shortcut_Set_Active_Layer'Access ) );
        win.Add_Listener( Listener( ALLEGRO_KEY_4, (others=>No), Key_Press, view, Shortcut_Set_Active_Layer'Access ) );
        win.Add_Listener( Listener( ALLEGRO_KEY_5, (others=>No), Key_Press, view, Shortcut_Set_Active_Layer'Access ) );
        win.Add_Listener( Listener( ALLEGRO_KEY_6, (others=>No), Key_Press, view, Shortcut_Set_Active_Layer'Access ) );
        win.Add_Listener( Listener( ALLEGRO_KEY_7, (others=>No), Key_Press, view, Shortcut_Set_Active_Layer'Access ) );
        win.Add_Listener( Listener( ALLEGRO_KEY_8, (others=>No), Key_Press, view, Shortcut_Set_Active_Layer'Access ) );
        win.Add_Listener( Listener( ALLEGRO_KEY_9, (others=>No), Key_Press, view, Shortcut_Set_Active_Layer'Access ) );
        win.Add_Listener( Listener( ALLEGRO_KEY_1, (CTRL=>Yes, others=>No), Key_Press, view, Shortcut_Show_Layer'Access ) );
        win.Add_Listener( Listener( ALLEGRO_KEY_2, (CTRL=>Yes, others=>No), Key_Press, view, Shortcut_Show_Layer'Access ) );
        win.Add_Listener( Listener( ALLEGRO_KEY_3, (CTRL=>Yes, others=>No), Key_Press, view, Shortcut_Show_Layer'Access ) );
        win.Add_Listener( Listener( ALLEGRO_KEY_4, (CTRL=>Yes, others=>No), Key_Press, view, Shortcut_Show_Layer'Access ) );
        win.Add_Listener( Listener( ALLEGRO_KEY_5, (CTRL=>Yes, others=>No), Key_Press, view, Shortcut_Show_Layer'Access ) );
        win.Add_Listener( Listener( ALLEGRO_KEY_6, (CTRL=>Yes, others=>No), Key_Press, view, Shortcut_Show_Layer'Access ) );
        win.Add_Listener( Listener( ALLEGRO_KEY_7, (CTRL=>Yes, others=>No), Key_Press, view, Shortcut_Show_Layer'Access ) );
        win.Add_Listener( Listener( ALLEGRO_KEY_8, (CTRL=>Yes, others=>No), Key_Press, view, Shortcut_Show_Layer'Access ) );
        win.Add_Listener( Listener( ALLEGRO_KEY_9, (CTRL=>Yes, others=>No), Key_Press, view, Shortcut_Show_Layer'Access ) );
    end Make_Menu;

    ----------------------------------------------------------------------------

    procedure Set_Active_Layer( view   : not null access Ked_View'Class;
                                action : A_Button_Action ) is
        oldLayer : constant Integer := A_Ked_Scene(view.Get_Widget( "scene" )).Get_Active_Layer;
        newLayer : constant Integer := As_Number( action.Get_Source.Get_Attribute( "layer" ) ).Get.To_Int;
    begin
        if action.Get_Id = Press then
            A_Button(view.Get_Widget( "chkLayer" & Image( oldLayer ) )).Set_Enabled( True );
            A_Ked_Scene(view.Get_Widget( "scene" )).Set_Active_Layer( newLayer );
            action.Get_Source.Set_Color( Foreground, Themes.Get.Color( Selected ) );
            A_Button(view.Get_Widget( "chkLayer" & Image( newLayer ) )).Set_State( True );
            A_Button(view.Get_Widget( "chkLayer" & Image( newLayer ) )).Set_Enabled( False );
        elsif action.Get_Id = Release then
            action.Get_Source.Set_Color( Foreground, Themes.Get.Color( Foreground ) );
        end if;
    end Set_Active_Layer;

    ----------------------------------------------------------------------------

    procedure Shortcut_Set_Active_Layer( view   : not null access Ked_View'Class;
                                         action : A_Key_Action ) is
        layer : Integer := 0;
    begin
        layer := (action.Get_Key - ALLEGRO_KEY_1) + 1;   -- for KEY_1 .. KEY_9
        A_Button(view.Get_Widget( "btnLayer" & Image( layer ) )).Set_State( True );
    exception
        when others =>
            -- a button for the layer doesn't exist
            null;
    end Shortcut_Set_Active_Layer;

    ----------------------------------------------------------------------------

    procedure Shortcut_Show_Layer( view   : not null access Ked_View'Class;
                                   action : A_Key_Action ) is
        layer : Integer := 0;
        chk   : A_Button;
    begin
        layer := (action.Get_Key - ALLEGRO_KEY_1) + 1;
        chk := A_Button(view.Get_Widget( "chkLayer" & Image( layer ) ));
        if chk.Is_Enabled then
            chk.Toggle_State;
        end if;
    exception
        when ID_NOT_FOUND =>
            -- a checkbox for the layer doesn't exist
            null;
    end Shortcut_Show_Layer;

    ----------------------------------------------------------------------------

    procedure Show_Clipping( view   : not null access Ked_View'Class;
                             action : A_Button_Action ) is
        pragma Unreferenced( view );
    begin
        if A_Button(action.Get_Source).Get_State then
            Set_Pref( "draw_clipping", True );
            action.Get_Source.Set_Color( Foreground, Themes.Get.Color( Selected ) );
        else
            Set_Pref( "draw_clipping", False );
            action.Get_Source.Set_Color( Foreground, Themes.Get.Color( Foreground ) );
        end if;
    end Show_Clipping;

    ----------------------------------------------------------------------------

    procedure Show_Layer( view   : not null access Ked_View'Class;
                          action : A_Button_Action ) is
        layer : constant Integer := As_Number( action.Get_Source.Get_Attribute( "layer" ) ).Get.To_Int;
    begin
        A_Ked_Scene(view.Get_Widget( "scene" )).Set_Layer_Visible( layer, action.Get_Id = Press );
    end Show_Layer;

    ----------------------------------------------------------------------------

    procedure Show_Layers( view   : not null access Ked_View'Class;
                           action : A_Button_Action ) is
    begin
        view.Get_Widget( "pnlLayers" ).Set_Visible( action.Get_Id = Press );
    end Show_Layers;

    ----------------------------------------------------------------------------

    procedure Update_Menu( view   : not null A_Ked_View;
                           layers : not null A_Layer_Array ) is
        theme      : constant A_Theme := Themes.Get;
        layerPanel : constant A_Panel := A_Panel(view.Get_Widget( "pnlLayers" ));
        btn        : A_Button;
    begin
        view.layerGrp.Clear;
        layerPanel.Clear_Widgets;
        layerPanel.Set_Layout_LTRB( -150, -(btnSize + (layers'Length + 1) * rowHeight + 1), 0, btnSize );
        view.Get_Widget( "btnLayer" ).Set_Enabled( True );

        for i in layers'Range loop
            declare
                btn : A_Button := Create_Toggle_Button( view, "btnLayer" & Image( i ),
                                                        "Layer " & Image( i ) );
                chk : A_Checkbox := Create_Checkbox( view, "chkLayer" & Image( i ) );
                top : constant Integer := (layers'Last - i + 1) * rowHeight;
            begin
                -- create the active button
                btn.Set_Focusable( False );
                btn.Set_Attribute( "layer", Create( i ) );
                btn.Set_Border( Border_None );
                btn.Contrast_Text( True );
                btn.Set_Layout_LTRB( 0, top, 0, -(top + rowHeight - 1) );
                if i = layers'First then
                    btn.Set_State( True );
                    btn.Set_Color( Foreground, theme.Color(Selected) );
                end if;
                btn.Add_Listener( Listener( Press, view, Set_Active_Layer'Access ) );
                btn.Add_Listener( Listener( Release, view, Set_Active_Layer'Access ) );
                view.layerGrp.Add( btn );
                layerPanel.Add_Widget( A_Widget(btn) );

                -- create the visibility checkbox
                chk.Set_Focusable( False );
                chk.Set_Attribute( "layer", Create( i ) );
                chk.Set_Align( Align_Center );
                chk.Set_Check_Icon( "eye_icon" );
                chk.Set_Layout_LTRB( 2, top, -15, -(top + rowHeight - 1) );
                chk.Set_State( True );
                chk.Set_Color( Foreground, base_colors(Transparent) );
                chk.Set_Border( Border_Line );
                chk.Add_Listener( Listener( Press, view, Show_Layer'Access ) );
                chk.Add_Listener( Listener( Release, view, Show_Layer'Access ) );
                layerPanel.Add_Widget( A_Widget(chk) );
            end;
        end loop;

        -- create the visibility button for clipping layer
        btn := Create_Toggle_Button( view, "btnLayerClipping", "Show Clipping" );
        btn.Set_Focusable( False );
        btn.Set_Border( Border_None );
        btn.Contrast_Text( True );
        btn.Set_Layout_LTRB( 0, 0, 0, -(rowHeight-1) );
        if Get_Pref( "draw_clipping" ) then
            btn.Set_State( True );
            btn.Set_Color( Foreground, theme.Color(Selected) );
        end if;
        btn.Add_Listener( Listener( Press, view, Show_Clipping'Access ) );
        btn.Add_Listener( Listener( Release, view, Show_Clipping'Access ) );
        layerPanel.Add_Widget( A_Widget(btn) );
    end Update_Menu;

end Game_Views.Ked.Toolbar.Layers_Menu;
