--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

with Actions;                           use Actions;
with Ada.Tags;                          use Ada.Tags;
with Allegro.Color;                     use Allegro.Color;
with Allegro.Keyboard;                  use Allegro.Keyboard;
with Debugging;                         use Debugging;
with Entities.Enemies;
with Entities.Items;
with Entities.Poles;
with Entities.Triggers;
with Entities.Triggers.Keen;
with Events.Ked;                        use Events.Ked;
with Game_Views.Ked.Menubar;
with Game_Views.Ked.Toolbar.Layers_Menu;
with Keyboard;                          use Keyboard;
with Support;                           use Support;
with Themes;                            use Themes;
with Themes.Palette;                    use Themes.Palette;
with Tools.Entity_Spawners;             use Tools.Entity_Spawners;
with Tools.Pointers;                    use Tools.Pointers;
with Widgets.Buttons;                   use Widgets.Buttons;
with Widgets.Buttons.Pushes;            use Widgets.Buttons.Pushes;
with Widgets.Buttons.Toggles;           use Widgets.Buttons.Toggles;
with Widgets.Containers.Panels;         use Widgets.Containers.Panels;
with Widgets.Containers.Scenes.Ked;     use Widgets.Containers.Scenes.Ked;
with Widgets.Containers.Scroll_Panes;   use Widgets.Containers.Scroll_Panes;
with Widgets.Labels;                    use Widgets.Labels;
with Widgets.Palettes;                  use Widgets.Palettes;
with Widgets.Palettes.Enemies;          use Widgets.Palettes.Enemies;
with Widgets.Palettes.Items;            use Widgets.Palettes.Items;
with Widgets.Palettes.Matrices;         use Widgets.Palettes.Matrices;
with Widgets.Palettes.Tiles;            use Widgets.Palettes.Tiles;

package body Game_Views.Ked.Toolbar is

    PALETTE_WIDTH : constant := 325;
    TOOLBAR_WIDTH : constant := 22;

    ----------------------------------------------------------------------------

    procedure Action_Pin_Palette( view   : not null access Ked_View'Class;
                                  action : A_Button_Action );

    procedure Action_Show_Items( view   : not null access Ked_View'Class;
                                 action : A_Button_Action );

    procedure Action_Show_Enemies( view   : not null access Ked_View'Class;
                                   action : A_Button_Action );

    procedure Action_Show_Matrices( view   : not null access Ked_View'Class;
                                    action : A_Button_Action );

    procedure Action_Show_Tiles( view   : not null access Ked_View'Class;
                                 action : A_Button_Action );

    procedure Action_Show_Palette( view   : not null access Ked_View'Class;
                                   action : A_Button_Group_Action );

    procedure Action_Hide_Palette( view   : not null access Ked_View'Class;
                                   action : A_Button_Group_Action );

    procedure Action_Use_Pointer( view   : not null access Ked_View'Class;
                                  action : A_Button_Action );

    procedure Action_Use_Pole_Tool( view   : not null access Ked_View'Class;
                                    action : A_Button_Action );

    procedure Action_Use_Switch_Tool( view   : not null access Ked_View'Class;
                                      action : A_Button_Action );

    procedure Action_Use_Trigger_Tool( view   : not null access Ked_View'Class;
                                       action : A_Button_Action );

    procedure Select_Tool( view   : not null access Ked_View'Class;
                           action : A_Key_Action );

    -- END PRIVATE DECLARATIONS --
    --==========================================================================

    procedure Action_Pin_Palette( view   : not null access Ked_View'Class;
                                  action : A_Button_Action ) is
    begin
        view.paletteGrp.Set_Keep_Selected( action.Get_Id = Press );
    end Action_Pin_Palette;

    ----------------------------------------------------------------------------

    procedure Action_Show_Enemies( view   : not null access Ked_View'Class;
                                   action : A_Button_Action ) is
    begin
        if action.Get_Id = Press then
            view.Get_Widget( "spnEnemy" ).Set_Visible( True );
            view.Get_Widget( "btnEnemy" ).Set_Border( Border_Raised );
            A_Label(view.Get_Widget( "lblPalTitle" )).Set_Text( "Enemies" );
        elsif action.Get_Id = Release then
            view.Get_Widget( "spnEnemy" ).Set_Visible( False );
            view.Get_Widget( "btnEnemy" ).Set_Border( Border_None );
        end if;
    end Action_Show_Enemies;

    ----------------------------------------------------------------------------

    procedure Action_Show_Items( view   : not null access Ked_View'Class;
                                 action : A_Button_Action ) is
    begin
        if action.Get_Id = Press then
            view.Get_Widget( "spnItem" ).Set_Visible( True );
            view.Get_Widget( "btnItem" ).Set_Border( Border_Raised );
            A_Label(view.Get_Widget( "lblPalTitle" )).Set_Text( "Items" );
        elsif action.Get_Id = Release then
            view.Get_Widget( "spnItem" ).Set_Visible( False );
            view.Get_Widget( "btnItem" ).Set_Border( Border_None );
        end if;
    end Action_Show_Items;

    ----------------------------------------------------------------------------

    procedure Action_Show_Matrices( view   : not null access Ked_View'Class;
                                    action : A_Button_Action ) is
    begin
        if action.Get_Id = Press then
            view.Get_Widget( "spnMatrix" ).Set_Visible( True );
            view.Get_Widget( "btnMatrix" ).Set_Border( Border_Raised );
            A_Label(view.Get_Widget( "lblPalTitle" )).Set_Text( "Tile Matrices" );
        elsif action.Get_Id = Release then
            view.Get_Widget( "spnMatrix" ).Set_Visible( False );
            view.Get_Widget( "btnMatrix" ).Set_Border( Border_None );
        end if;
    end Action_Show_Matrices;

    ----------------------------------------------------------------------------

    procedure Action_Show_Tiles( view   : not null access Ked_View'Class;
                                 action : A_Button_Action ) is
    begin
        if action.Get_Id = Press then
            view.Get_Widget( "spnTile" ).Set_Visible( True );
            view.Get_Widget( "btnPaint" ).Set_Border( Border_Raised );
            A_Label(view.Get_Widget( "lblPalTitle" )).Set_Text( "Tiles" );
        elsif action.Get_Id = Release then
            view.Get_Widget( "spnTile" ).Set_Visible( False );
            view.Get_Widget( "btnPaint" ).Set_Border( Border_None );
        end if;
    end Action_Show_Tiles;

    ----------------------------------------------------------------------------

    procedure Action_Show_Palette( view   : not null access Ked_View'Class;
                                   action : A_Button_Group_Action ) is
        pragma Unreferenced( action );
        palette     : constant A_Panel := A_Panel(view.Get_Widget( "pnlPalette" ));
        scene       : constant A_Ked_Scene := A_Ked_Scene(view.Get_Widget( "scene" ));
        oldWidth    : constant Natural := scene.Get_Width;
        oldFocusX   : constant Float := scene.Get_Focus_X;
        shiftAmount : Float;
    begin
        palette.Set_Visible( True );

        -- set the new width for the scene. this may change the focus in the
        -- scene's On_Resize, in order to keep the focus in bounds.
        scene.Set_Layout_LTRB( 0, Game_Views.Ked.Menubar.Get_Menubar_Height,
                               Get_Toolbar_Width + PALETTE_WIDTH, 0 );

        -- calculate the total amount the focus needs to shift to keep the
        -- original focused point in the center of the scene.
        shiftAmount := Float(scene.Get_Width - oldWidth) / 2.0 / scene.Get_Zoom;

        -- adjust the shift amount to account for any focus shift that may have
        -- occured during the widget resize.
        shiftAmount := shiftAmount - (scene.Get_Focus_X - oldFocusX);

        scene.Shift_Focus( shiftAmount, 0.0 );
    end Action_Show_Palette;

    ----------------------------------------------------------------------------

    procedure Action_Hide_Palette( view   : not null access Ked_View'Class;
                                   action : A_Button_Group_Action ) is
        pragma Unreferenced( action );
        palette     : constant A_Panel := A_Panel(view.Get_Widget( "pnlPalette" ));
        scene       : constant A_Ked_Scene := A_Ked_Scene(view.Get_Widget( "scene" ));
        oldWidth    : constant Natural := scene.Get_Width;
        oldFocusX   : constant Float := scene.Get_Focus_X;
        shiftAmount : Float;
    begin
        palette.Set_Visible( False );

        -- set the new width for the scene. this may change the focus in the
        -- scene's On_Resize, in order to keep the focus in bounds.
        scene.Set_Layout_LTRB( 0, Game_Views.Ked.Menubar.Get_Menubar_Height,
                               Get_Toolbar_Width, 0 );

        -- calculate the total amount the focus needs to shift to keep the
        -- original focused point in the center of the scene.
        shiftAmount := Float(scene.Get_Width - oldWidth) / 2.0 / scene.Get_Zoom;

        -- adjust the shift amount to account for any focus shift that may have
        -- occured during the widget resize.
        shiftAmount := shiftAmount - (scene.Get_Focus_X - oldFocusX);

        scene.Shift_Focus( shiftAmount, 0.0 );
    end Action_Hide_Palette;

    ----------------------------------------------------------------------------

    procedure Action_Use_Pointer( view   : not null access Ked_View'Class;
                                  action : A_Button_Action ) is
        tool : A_Tool;
    begin
        if action.Get_Id = Press then
            view.Get_Widget( "btnPointer" ).Set_Border( Border_Raised );
        elsif action.Get_Id = Release then
            view.Get_Widget( "btnPointer" ).Set_Border( Border_None );
            tool := Create_Pointer;
            Queue_Set_Tool( tool );
            Delete( tool );
        end if;
    end Action_Use_Pointer;

    ----------------------------------------------------------------------------

    procedure Action_Use_Pole_Tool( view   : not null access Ked_View'Class;
                                    action : A_Button_Action ) is
        tool : A_Tool;
    begin
        if action.Get_Id = Press then
            view.Get_Widget( "btnPole" ).Set_Border( Border_Raised );
        elsif action.Get_Id = Release then
            view.Get_Widget( "btnPole" ).Set_Border( Border_None );
            tool := Create_Entity_Spawner( External_Tag( Entities.Poles.Pole'Tag ), snap => True );
            Queue_Set_Tool( tool );
            Delete( tool );
        end if;
    end Action_Use_Pole_Tool;

    ----------------------------------------------------------------------------

    procedure Action_Use_Switch_Tool( view   : not null access Ked_View'Class;
                                      action : A_Button_Action ) is
        pragma Unreferenced( view );
--      tool : A_Tool;
    begin
        if action.Get_Id = Click then
            null;
            pragma Debug( Dbg( "STUB: Action_Use_Switch_Tool" ) );
--          tool := Create_Switch_Tool;
--          Queue_Set_Tool( tool );
--          Delete( tool );
        end if;
    end Action_Use_Switch_Tool;

    ----------------------------------------------------------------------------

    procedure Action_Use_Trigger_Tool( view   : not null access Ked_View'Class;
                                       action : A_Button_Action ) is
        tool : A_Tool;
    begin
        if action.Get_Id = Press then
            view.Get_Widget( "btnTrigger" ).Set_Border( Border_Raised );
        elsif action.Get_Id = Release then
            view.Get_Widget( "btnTrigger" ).Set_Border( Border_None );
            tool := Create_Entity_Spawner( External_Tag( Entities.Triggers.Keen.Keen_Trigger'Tag ), snap => True );
            Queue_Set_Tool( tool );
            Delete( tool );
        end if;
    end Action_Use_Trigger_Tool;

    ----------------------------------------------------------------------------

    function Get_Toolbar_Width return Natural is
    begin
        return TOOLBAR_WIDTH;
    end Get_Toolbar_Width;

    ----------------------------------------------------------------------------

    procedure Handle_New_Map( view : not null A_Ked_View ) is
    begin
      --view.Get_Widget( "btnTack" ).Set_Enabled( True );
        view.Get_Widget( "btnPaint" ).Set_Enabled( True );
        view.Get_Widget( "btnMatrix" ).Set_Enabled( True );
        view.Get_Widget( "btnItem" ).Set_Enabled( True );
        view.Get_Widget( "btnEnemy" ).Set_Enabled( True );
        view.Get_Widget( "btnTrigger" ).Set_Enabled( True );
        view.Get_Widget( "btnPole" ).Set_Enabled( True );
        view.Get_Widget( "btnSwitch" ).Set_Enabled( True );
        view.Get_Widget( "btnLift" ).Set_Enabled( True );
        view.Get_Widget( "btnPortal" ).Set_Enabled( True );
        view.Get_Widget( "btnPointer" ).Set_Enabled( True );
    end Handle_New_Map;

    ----------------------------------------------------------------------------

    procedure Handle_Tool_Changed( view : not null A_Ked_View;
                                   tool : not null A_Tool ) is
        theme : constant A_Theme := Themes.Get;

        function Button_Color( on : Boolean ) return Allegro_Color is
        begin
            if on then
                return theme.Color( Selected );
            end if;
            return theme.Color( Foreground );
        end Button_Color;

    begin
        if not A_Button(view.Get_Widget( "btnTack" )).Get_State then
            view.paletteGrp.Unset;
        end if;
        view.Get_Widget( "btnPaint"   ).Set_Color( Foreground, Button_Color( tool.Get_Type = Paint_Tool ) );
        view.Get_Widget( "btnMatrix"  ).Set_Color( Foreground, Button_Color( tool.Get_Type = Matrix_Tool ) );
        view.Get_Widget( "btnItem"    ).Set_Color( Foreground, Button_Color( tool.Get_Type = Entity_Tool and then A_Entity_Spawner(tool).In_Class( Entities.Items.CLASS_PATTERN ) ) );
        view.Get_Widget( "btnEnemy"   ).Set_Color( Foreground, Button_Color( tool.Get_Type = Entity_Tool and then A_Entity_Spawner(tool).In_Class( Entities.Enemies.CLASS_PATTERN ) ) );
        view.Get_Widget( "btnTrigger" ).Set_Color( Foreground, Button_Color( tool.Get_Type = Entity_Tool and then A_Entity_Spawner(tool).In_Class( Entities.Triggers.CLASS_PATTERN ) ) );
        view.Get_Widget( "btnPole"    ).Set_Color( Foreground, Button_Color( tool.Get_Type = Entity_Tool and then A_Entity_Spawner(tool).In_Class( External_Tag( Entities.Poles.Pole'Tag ) ) ) );
      --view.Get_Widget( "btnSwitch"  ).Set_Color( Foreground, Button_Color( tool.all in Switch_Tool'Class ) );
        view.Get_Widget( "btnPointer" ).Set_Color( Foreground, Button_Color( tool.Get_Type = Pointer_Tool ) );
    end Handle_Tool_Changed;

    ----------------------------------------------------------------------------

    procedure Make_Toolbar( view : not null A_Ked_View; win : not null A_Window ) is
        top            : constant Integer := Game_Views.Ked.Menubar.Get_Menubar_Height;
        palTitleHeight : constant Integer := 20;
        btnHeight      : constant Integer := Get_Toolbar_Width - 2;  -- square buttons
        scenePosWidth  : constant Integer := 80;

        label         : A_Label;
        button        : A_Button;
        tilePalette   : A_Palette;
        matrixPalette : A_Palette;
        itemPalette   : A_Palette;
        enemyPalette  : A_Palette;
        panel         : A_Panel;
        panel2        : A_Panel;
        scrollPane    : A_Scroll_Pane;
    begin
        view.paletteGrp := Create_Button_Group;
        view.paletteGrp.Set_Keep_Selected( False );
        view.paletteGrp.Add_Listener( Listener( Group_Set, view, Action_Show_Palette'Access ) );
        view.paletteGrp.Add_Listener( Listener( Group_Cleared, view, Action_Hide_Palette'Access ) );

        panel := Create_Panel( view, "pnlToolbar" );
        panel.Set_Layout_LTRB( -(Get_Toolbar_Width-1), top, 0, Get_Toolbar_Width );

            button := Create_Toggle_Button( view, "btnPaint", icon => "tile_icon" );
            button.Set_Layout_LTRB( 0, btnHeight*0, 0, -(btnHeight*1-1) );
            button.Set_Border( Border_None );
            button.Set_Focusable( False );
            button.Set_Enabled( False );
            button.Add_Listener( Listener( Press, view, Action_Show_Tiles'Access ) );
            button.Add_Listener( Listener( Release, view, Action_Show_Tiles'Access ) );
            view.paletteGrp.Add( button );
            panel.Add_Widget( A_Widget(button) );

            button := Create_Toggle_Button( view, "btnMatrix", icon => "matrix_icon" );
            button.Set_Layout_LTRB( 0, btnHeight*1, 0, -(btnHeight*2-1) );
            button.Set_Border( Border_None );
            button.Set_Focusable( False );
            button.Set_Enabled( False );
            button.Add_Listener( Listener( Press, view, Action_Show_Matrices'Access ) );
            button.Add_Listener( Listener( Release, view, Action_Show_Matrices'Access ) );
            view.paletteGrp.Add( button );
            panel.Add_Widget( A_Widget(button) );

            button := Create_Toggle_Button( view, "btnItem", icon => "item_icon" );
            button.Set_Layout_LTRB( 0, btnHeight*2, 0, -(btnHeight*3-1) );
            button.Set_Border( Border_None );
            button.Set_Focusable( False );
            button.Set_Enabled( False );
            button.Add_Listener( Listener( Press, view, Action_Show_Items'Access ) );
            button.Add_Listener( Listener( Release, view, Action_Show_Items'Access ) );
            view.paletteGrp.Add( button );
            panel.Add_Widget( A_Widget(button) );

            button := Create_Toggle_Button( view, "btnEnemy", icon => "enemy_icon" );
            button.Set_Layout_LTRB( 0, btnHeight*3, 0, -(btnHeight*4-1) );
            button.Set_Border( Border_None );
            button.Set_Focusable( False );
            button.Set_Enabled( False );
            button.Add_Listener( Listener( Press, view, Action_Show_Enemies'Access ) );
            button.Add_Listener( Listener( Release, view, Action_Show_Enemies'Access ) );
            view.paletteGrp.Add( button );
            panel.Add_Widget( A_Widget(button) );

            panel2 := Create_Panel( view, "pnlToolSep" );
            panel2.Set_Layout_CenterHY( TOOLBAR_WIDTH-8,
                                        btnHeight*4+btnHeight/2-1,
                                        btnHeight*4+btnHeight/2 );
            panel.Add_Widget( A_Widget(panel2) );

            button := Create_Push_Button( view, "btnTrigger", icon => "trigger_icon" );
            button.Set_Layout_LTRB( 0, btnHeight*5, 0, -(btnHeight*6-1) );
            button.Set_Border( Border_None );
            button.Set_Focusable( False );
            button.Set_Enabled( False );
            button.Add_Listener( Listener( Press, view, Action_Use_Trigger_Tool'Access ) );
            button.Add_Listener( Listener( Release, view, Action_Use_Trigger_Tool'Access ) );
            panel.Add_Widget( A_Widget(button) );

            button := Create_Push_Button( view, "btnPole", icon => "pole_icon" );
            button.Set_Layout_LTRB( 0, btnHeight*6, 0, -(btnHeight*7-1) );
            button.Set_Border( Border_None );
            button.Set_Focusable( False );
            button.Set_Enabled( False );
            button.Add_Listener( Listener( Press, view, Action_Use_Pole_Tool'Access ) );
            button.Add_Listener( Listener( Release, view, Action_Use_Pole_Tool'Access ) );
            panel.Add_Widget( A_Widget(button) );

            button := Create_Push_Button( view, "btnSwitch", icon => "switch_icon" );
            button.Set_Layout_LTRB( 0, btnHeight*7, 0, -(btnHeight*8-1) );
            button.Set_Border( Border_None );
            button.Set_Focusable( False );
            button.Set_Enabled( False );
            button.Add_Listener( Listener( Click, view, Action_Use_Switch_Tool'Access ) );
            panel.Add_Widget( A_Widget(button) );

            button := Create_Push_Button( view, "btnLift", icon=>"lift_icon" );
            button.Set_Layout_LTRB( 0, btnHeight*8, 0, -(btnHeight*9-1) );
            button.Set_Border( Border_None );
            button.Set_Focusable( False );
            button.Set_Enabled( False );
            panel.Add_Widget( A_Widget(button) );

            button := Create_Push_Button( view, "btnPortal", icon => "portal_icon" );
            button.Set_Layout_LTRB( 0, btnHeight*9, 0, -(btnHeight*10-1) );
            button.Set_Border( Border_None );
            button.Set_Focusable( False );
            button.Set_Enabled( False );
            panel.Add_Widget( A_Widget(button) );

            button := Create_Push_Button( view, "btnPointer", icon => "pointer_icon" );
            button.Set_Layout_LTRB( 0, btnHeight*10, 0, -(btnHeight*11-1) );
            button.Set_Border( Border_None );
            button.Set_Color( Foreground, Themes.Get.Color( Selected ) );
            button.Set_Focusable( False );
            button.Set_Enabled( False );
            button.Add_Listener( Listener( Press, view, Action_Use_Pointer'Access ) );
            button.Add_Listener( Listener( Release, view, Action_Use_Pointer'Access ) );
            panel.Add_Widget( A_Widget(button) );

        win.Add_Widget( A_Widget(panel) );

        panel := Create_Panel( view, "pnlScenePos" );
        panel.Set_Layout_LTRB( -(TOOLBAR_WIDTH+scenePosWidth), -(btnHeight+1), TOOLBAR_WIDTH, 0 );

            label := Create_Label( view, "lblScenePos", "" );
            label.Set_Align( Align_Right );
            label.Set_Layout_LTRB( -75, 0, 5, 0 );
            panel.Add_Widget( A_Widget(label) );

        win.Add_Widget( A_Widget(panel) );

        panel := Create_Panel( view, "pnlPalette" );
        panel.Set_Layout_LTRB( -(PALETTE_WIDTH+TOOLBAR_WIDTH-1), top, TOOLBAR_WIDTH, 0 );
        panel.Set_Visible( False );

            panel2 := Create_Panel( view, "pnlPaletteTop" );
            panel2.Set_Layout_LTRB( 0, 0, 0, -(palTitleHeight-1) );
            panel2.Set_Color( Foreground, panel2.Get_Color( Selected ) );
            panel2.Set_Border( Border_None );

                label := Create_Label( view, "lblPalTitle", "" );
                label.Set_Layout_LTRB( 4, 0, palTitleHeight + 4, 0 );
                label.Set_Color( Text, base_colors(White) );
                label.Set_Color( Foreground, label.Get_Color( Selected ) );
                panel2.Add_Widget( A_Widget(label) );

                button := Create_Toggle_Button( view, "btnTack", icon => "thumbtack_icon" );
                button.Set_Layout_LTRB( -(palTitleHeight-1) + 2, 2, 2, 2 );
                button.Set_Focusable( False );
                button.Set_Color( Foreground, button.Get_Color( Selected ) );
                button.Add_Listener( Listener( Press, view, Action_Pin_Palette'Access ) );
                button.Add_Listener( Listener( Release, view, Action_Pin_Palette'Access ) );
                panel2.Add_Widget( A_Widget(button) );

            panel.Add_Widget( A_Widget(panel2) );

            panel2 := Create_Panel( view, "pnlPaletteBottom" );
            panel2.Set_Layout_LTRB( 0, palTitleHeight + 1, 0, 0 );
            panel2.Set_Border( Border_None );

                scrollPane := Create_Scroll_Pane( view, "spnTile" );
                scrollPane.Set_Layout_LTRB( 0, 0, 0, 0 );
                scrollPane.Draw_Hbar( False );
                scrollPane.Set_Border( Border_None );
                scrollPane.Set_Visible( False );
                    tilePalette := Create_Tile_Palette( view, "tilePalette" );
                    tilePalette.Set_Icon_Size( 16, 16 );
                    tilePalette.Set_Columns( 18 );
                    scrollPane.Set_Client( A_Widget(tilePalette) );
                panel2.Add_Widget( A_Widget(scrollPane) );

                scrollPane := Create_Scroll_Pane( view, "spnMatrix" );
                scrollPane.Set_Layout_LTRB( 0, 0, 0, 0 );
                scrollPane.Draw_Hbar( False );
                scrollPane.Set_Border( Border_None );
                scrollPane.Set_Visible( False );
                    matrixPalette := Create_Matrix_Palette( view, "matrixPalette" );
                    matrixPalette.Set_Icon_Size( 16, 16 );
                    matrixPalette.Set_Columns( 18 );
                    scrollPane.Set_Client( A_Widget(matrixPalette) );
                panel2.Add_Widget( A_Widget(scrollPane) );

                scrollPane := Create_Scroll_Pane( view, "spnItem" );
                scrollPane.Set_Layout_LTRB( 0, 0, 0, 0 );
                scrollPane.Draw_Hbar( False );
                scrollPane.Draw_Vbar( False );
                scrollPane.Set_Border( Border_None );
                scrollPane.Set_Visible( False );
                    itemPalette := Create_Item_Palette( view, "itemPalette" );
                    itemPalette.Set_Icon_Size( 32, 32 );
                    itemPalette.Set_Columns( Floor( Float(PALETTE_WIDTH) / 33.0 ) );
                    scrollPane.Set_Client( A_Widget(itemPalette) );
                panel2.Add_Widget( A_Widget(scrollPane) );

                scrollPane := Create_Scroll_Pane( view, "spnEnemy" );
                scrollPane.Set_Layout_LTRB( 0, 0, 0, 0 );
                scrollPane.Draw_Hbar( False );
                scrollPane.Draw_Vbar( False );
                scrollPane.Set_Border( Border_None );
                scrollPane.Set_Visible( False );
                    enemyPalette := Create_Enemy_Palette( view, "enemyPalette" );
                    enemyPalette.Set_Icon_Size( 48, 48 );
                    enemyPalette.Set_Columns( Floor( Float(PALETTE_WIDTH) / 49.0 ) );
                    scrollPane.Set_Client( A_Widget(enemyPalette) );
                panel2.Add_Widget( A_Widget(scrollPane) );

            panel.Add_Widget( A_Widget(panel2) );

        win.Add_Widget( A_Widget(panel) );

        Layers_Menu.Make_Menu( view, win );

        win.Add_Listener( Listener( ALLEGRO_KEY_S, (others=>No), Key_Press, view, Select_Tool'Access ) );       -- select/pointer tool
        win.Add_Listener( Listener( ALLEGRO_KEY_ESCAPE, (others=>No), Key_Press, view, Select_Tool'Access ) );  -- select/pointer tool
        win.Add_Listener( Listener( ALLEGRO_KEY_T, (others=>No), Key_Press, view, Select_Tool'Access ) );       -- tile palette
        win.Add_Listener( Listener( ALLEGRO_KEY_M, (others=>No), Key_Press, view, Select_Tool'Access ) );       -- matrix palette
        win.Add_Listener( Listener( ALLEGRO_KEY_I, (others=>No), Key_Press, view, Select_Tool'Access ) );       -- item palette
        win.Add_Listener( Listener( ALLEGRO_KEY_E, (others=>No), Key_Press, view, Select_Tool'Access ) );       -- enemy palette
        win.Add_Listener( Listener( ALLEGRO_KEY_R, (others=>No), Key_Press, view, Select_Tool'Access ) );       -- trigger tool
        win.Add_Listener( Listener( ALLEGRO_KEY_P, (others=>No), Key_Press, view, Select_Tool'Access ) );       -- pole tool
    exception
        when others =>
            Delete( A_Widget(label) );
            Delete( A_Widget(button) );
            Delete( A_Widget(tilePalette) );
            Delete( A_Widget(matrixPalette) );
            Delete( A_Widget(itemPalette) );
            Delete( A_Widget(enemyPalette) );
            Delete( A_Widget(panel) );
            Delete( A_Widget(panel2) );
            Delete( A_Widget(scrollPane) );
            raise;
    end Make_Toolbar;

    ----------------------------------------------------------------------------

    procedure Select_Tool( view   : not null access Ked_View'Class;
                           action : A_Key_Action ) is
    begin
        case action.Get_Key is
            when ALLEGRO_KEY_T => A_Button(view.Get_Widget( "btnPaint" )).Toggle_State;
            when ALLEGRO_KEY_M => A_Button(view.Get_Widget( "btnMatrix" )).Toggle_State;
            when ALLEGRO_KEY_I => A_Button(view.Get_Widget( "btnItem" )).Toggle_State;
            when ALLEGRO_KEY_E => A_Button(view.Get_Widget( "btnEnemy" )).Toggle_State;
            when ALLEGRO_KEY_R => A_Button(view.Get_Widget( "btnTrigger" )).Toggle_State;
            when ALLEGRO_KEY_P => A_Button(view.Get_Widget( "btnPole" )).Toggle_State;
            when ALLEGRO_KEY_ESCAPE |
                 ALLEGRO_KEY_S => A_Button(view.Get_Widget( "btnPointer" )).Toggle_State;
            when others => null;  -- should never happen
        end case;
    end Select_Tool;

end Game_Views.Ked.Toolbar;
