--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

with Resources;                         use Resources;
with Tiles.Libraries.Manager;           use Tiles.Libraries.Manager;
with Themes;                            use Themes;

pragma Elaborate_All( Themes );

package body Themes.Ked is

    LIB_NAME : constant String := "icons";

    -- END PRIVATE DECLARATIONS --
    --==========================================================================

    function Create_Theme return A_Theme is
        this : A_Theme := new Ked_Theme;
    begin
        this.Construct;
        return this;
    exception
        when others =>
            Delete( this );
            raise;
    end Create_Theme;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Ked_Theme ) is
    begin
        Theme(this.all).Construct;
        this.colors(Background)    := Al_Map_RGB( 148, 145, 140 );
        this.colors(Foreground)    := Al_Map_RGB( 212, 208, 200 );
        this.colors(Selected)      := Al_Map_RGB(  58,  95, 205 );
        this.colors(Selected_Text) := Al_Map_RGB( 255, 255, 255 );
        this.colors(Text)          := Al_Map_RGB(   0,   0,   0 );
        this.Register_Font( "dejavusans.ttf", DEFAULT_FONT_NAME );  -- may raise exception
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure Delete( this : in out Ked_Theme ) is
    begin
        Unload_Library( this.lib );
        Theme(this).Delete;
    end Delete;

    ----------------------------------------------------------------------------

    overriding
    function Get_Bitmap( this : access Ked_Theme;
                         id   : Natural ) return A_Allegro_Bitmap is
    begin
        if this.lib = null then
            this.lib := Load_Library( LIB_NAME, bitmaps => True );
        end if;
        if this.lib /= null then
            return this.lib.Get_Tile( id ).Get_Bitmap;
        else
            raise RESOURCE_ERROR with "Error loading tile library <" & LIB_NAME & ">";
        end if;
    end Get_Bitmap;

    ----------------------------------------------------------------------------

    overriding
    function Get_ID( this : access Ked_Theme; name : String ) return Natural is
    begin
        if this.lib = null then
            this.lib := Load_Library( LIB_NAME, bitmaps => True );
        end if;
        if this.lib /= null then
            return this.lib.Get_Id( name );
        else
            raise RESOURCE_ERROR with "Error loading tile library <" & LIB_NAME & ">";
        end if;
    end Get_ID;

    ----------------------------------------------------------------------------

    overriding
    function Get_Tile( this : access Ked_Theme; id : Natural ) return A_Tile is
    begin
        if this.lib = null then
            this.lib := Load_Library( LIB_NAME, bitmaps => True );
        end if;
        if this.lib /= null then
            return this.lib.Get_Tile( id );
        else
            raise RESOURCE_ERROR with "Error loading tile library <" & LIB_NAME & ">";
        end if;
    end Get_Tile;

begin

    Register_Allocator( Create_Theme'Access );

end Themes.Ked;
