--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

--with Debugging;                         use Debugging;
with Entities;                          use Entities;
with Events.Ked;                        use Events.Ked;
with Preferences;                       use Preferences;
with Support;
with Tools.Pointers;                    use Tools.Pointers;

package body Tools.Entity_Spawners is

    function Create_Entity_Spawner( id : String; snap : Boolean := True ) return A_Tool is
        this : constant A_Entity_Spawner := new Entity_Spawner;
    begin
        this.Construct( id, snap );
        return A_Tool(this);
    end Create_Entity_Spawner;

    ----------------------------------------------------------------------------

    not overriding
    procedure Construct( this : access Entity_Spawner;
                         id   : String;
                         snap : Boolean ) is
    begin
        Tool(this.all).Construct;
        this.id := To_Unbounded_String( id );
        this.snap := snap;
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure Apply( this : access Entity_Spawner; context : Tool_Context ) is

        ------------------------------------------------------------------------

        function Snap( n : Integer; size : Natural ) return Float is
            gridSize : constant Integer := Get_Pref( "gridsize" );
        begin
            if this.snap and then (Get_Pref( "gridsnap" ) xor context.modifiers(CTRL)) then
                -- Snap the entity if it's snappable (this.snap) and gridsnap is
                -- on. Allow the Ctrl key to invert the snapping behavior.
                -- Snap the entity to the center of the grid unless it's an even
                -- number of tiles wide. (For example, a 2x2 tile entity should
                -- be placed exactly over 2x2 tiles, which requires snapping to
                -- the edges of the grid and not the center.)
                return Float(Support.Grid_Snap( n,
                                                gridSize,
                                                centered => size mod (gridSize * 2) /= 0 ));
            end if;
            return Float(n);
        end Snap;

        ------------------------------------------------------------------------

        tool : A_Tool;
        temp : A_Entity;
        id   : Entity_Id;
        pragma Warnings( Off, id );
    begin
        if context.first then
            if context.func = Primary then
                if context.worldX >= 0 and then context.worldX < context.world.Get_Width and then
                   context.worldY >= 0 and then context.worldY < context.world.Get_Height
                then
                    temp := Entities.Template( To_String( this.id ) );
                    id := context.world.Spawn_Entity( To_String( this.id ),
                                                      Snap( context.worldX, temp.Get_Width ),
                                                      Snap( context.worldY, temp.Get_Height ) );
                end if;
            elsif context.func = Secondary or else context.func = Tertiary then
                tool := Create_Pointer;
                Queue_Set_Tool( tool );
                Delete( tool );
            end if;
        end if;
    end Apply;

    ----------------------------------------------------------------------------

    function Get_ID( this : not null access Entity_Spawner'Class ) return String is
    begin
        return To_String( this.id );
    end Get_ID;

    ----------------------------------------------------------------------------

    overriding
    function Get_Type( this : access Entity_Spawner ) return Tool_Type is
        pragma Unreferenced( this );
    begin
        return Entity_Tool;
    end Get_Type;

    ----------------------------------------------------------------------------

    function In_Class( this    : not null access Entity_Spawner'Class;
                       pattern : String ) return Boolean is
    begin
        return Entities.In_Class( To_String( this.id ), pattern );
    end In_Class;

end Tools.Entity_Spawners;
