--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

--with Debugging;                         use Debugging;
with Events.Ked;                        use Events.Ked;
with Tools.Paintbrushes;                use Tools.Paintbrushes;
with Tools.Pointers;                    use Tools.Pointers;

package body Tools.Matrixbrushes is

    function Create_Matrixbrush( matrix  : not null A_Tile_Matrix;
                                 offsetX,
                                 offsetY : Integer ) return A_Tool is
        this : constant A_Matrixbrush := new Matrixbrush;
    begin
        this.Construct( matrix, offsetX, offsetY );
        return A_Tool(this);
    end Create_Matrixbrush;

    ----------------------------------------------------------------------------

    overriding
    procedure Adjust( this : access Matrixbrush ) is
    begin
        Tool(this.all).Adjust;
        this.matrix := Copy( this.matrix );
    end Adjust;

    ----------------------------------------------------------------------------

    not overriding
    procedure Construct( this     : access Matrixbrush;
                         matrix   : A_Tile_Matrix;
                         offsetX,
                         offsetY  : Integer ) is
    begin
        Tool(this.all).Construct;
        this.matrix := Copy( matrix );
        this.offsetX := offsetX;
        this.offsetY := offsetY;
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure Delete( this : in out Matrixbrush ) is
    begin
        Delete( this.matrix );
        Tool(this).Delete;
    end Delete;

    ----------------------------------------------------------------------------

    overriding
    procedure Apply( this : access Matrixbrush; context : Tool_Context ) is
        tileX  : constant Integer := context.worldX / context.world.Tile_Width;
        tileY  : constant Integer := context.worldY / context.world.Tile_Width;
        offX,
        offY   : Integer;
        width,
        height : Integer;
        tool   : A_Tool;
    begin
        if context.func = Primary then
            if context.first then
                this.startTileX := tileX;
                this.startTileY := tileY;
                this.overTileX := -1;
                this.overTileY := -1;
            end if;
            if tileX /= this.overTileX or else tileY /= this.overTileY then
                this.overTileX := tileX;
                this.overTileY := tileY;
                height := this.matrix'Length(1);
                width := this.matrix'Length(2);
                if context.modifiers(CTRL) then
                    -- Auto-Offset constraint: draw the matrix as though the
                    -- first draw location is at offset of the matrix.
                    offX := width - (((this.startTileX - 1 - this.offsetX) mod width) + 1);
                    offY := height - (((this.startTileY - 1 - this.offsetY) mod height) + 1);
                else
                    offX := this.offsetX;
                    offY := this.offsetY;
                end if;
                context.world.Set_Tile( context.layer, tileX, tileY,
                                        this.matrix(
                                            ((tileY + offY) mod height) + 1,
                                            ((tileX + offX) mod width) + 1
                                        )
                                      );
            end if;
        elsif context.first then
            if context.func = Secondary then
                if tileX < context.world.Get_Width_Tiles and then
                   tileY < context.world.Get_Height_Tiles
                then
                    tool := Create_Paintbrush( context.world.Get_Tile_Id( context.layer, tileX, tileY ) );
                    Queue_Set_Tool( tool );
                    Delete( tool );
                end if;
            elsif context.func = Tertiary then
                tool := Create_Pointer;
                Queue_Set_Tool( tool );
                Delete( tool );
            end if;
        end if;
    end Apply;

    ----------------------------------------------------------------------------

    function Get_Matrix( this : not null access Matrixbrush'Class ) return A_Tile_Matrix is
    begin
        return this.matrix;
    end Get_Matrix;

    ----------------------------------------------------------------------------

    procedure Get_Offset( this : not null access Matrixbrush'Class;
                          x, y : out Integer ) is
    begin
        x := this.offsetX;
        y := this.offsetY;
    end Get_Offset;

    ----------------------------------------------------------------------------

    overriding
    function Get_Type( this : access Matrixbrush ) return Tool_Type is
        pragma Unreferenced( this );
    begin
        return Matrix_Tool;
    end Get_Type;

end Tools.Matrixbrushes;
