--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

--with Debugging;                         use Debugging;
with Events.Ked;                        use Events.Ked;
with Tools.Pointers;                    use Tools.Pointers;

package body Tools.Paintbrushes is

    function Create_Paintbrush( tile : Natural ) return A_Tool is
        this : constant A_Tool := new Paintbrush;
    begin
        Paintbrush(this.all).Construct( tile );
        return this;
    end Create_Paintbrush;

    ----------------------------------------------------------------------------

    not overriding
    procedure Construct( this : access Paintbrush; tile : Natural ) is
    begin
        Object(this.all).Construct;
        this.tile := tile;
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure Apply( this : access Paintbrush; context : Tool_Context ) is
        tileX : constant Integer := context.worldX / context.world.Tile_Width;
        tileY : constant Integer := context.worldY / context.world.Tile_Width;
        tool  : A_Tool := null;
    begin
        if context.func = Primary then
            if this.lastFunc /= Primary or else
               (tileX /= this.overTileX or else
                tileY /= this.overTileY or else
                context.layer /= this.overLayer)
            then
                this.overTileX := tileX;
                this.overTileY := tileY;
                this.overLayer := context.layer;
                context.world.Set_Tile( context.layer, tileX, tileY, this.tile );
            end if;
        elsif context.func = Secondary then
            if this.lastFunc /= Secondary or else
               (tileX /= this.overTileX or else
                tileY /= this.overTileY or else
                context.layer /= this.overLayer)
            then
                this.overTileX := tileX;
                this.overTileY := tileY;
                this.overLayer := context.layer;
                tool := Create_Paintbrush( context.world.Get_Tile_Id( context.layer, tileX, tileY ) );
                Queue_Set_Tool( tool );
                Delete( tool );
            end if;
        elsif context.first and then context.func = Tertiary then
            tool := Create_Pointer;
            Queue_Set_Tool( tool );
            Delete( tool );
        end if;
        this.lastFunc := context.func;
    end Apply;

    ----------------------------------------------------------------------------

    function Get_Tile( this : not null access Paintbrush'Class ) return Natural is
    begin
        return this.tile;
    end Get_Tile;

    ----------------------------------------------------------------------------

    overriding
    function Get_Type( this : access Paintbrush ) return Tool_Type is
        pragma Unreferenced( this );
    begin
        return Paint_Tool;
    end Get_Type;

end Tools.Paintbrushes;
