--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

with Ada.Tags;                          use Ada.Tags;
--with Debugging;                         use Debugging;
with Entities.Triggers.Keen;
with Events.Ked;                        use Events.Ked;
with Preferences;                       use Preferences;
with Support;                           use Support;
with Tools.Pointers;                    use Tools.Pointers;

package body Tools.Trigger_Spawners is

    function Create_Trigger_Spawner return A_Tool is
        this : constant A_Tool := new Trigger_Spawner;
    begin
        this.Construct;
        return this;
    end Create_Trigger_Spawner;

    ----------------------------------------------------------------------------

    overriding
    procedure Apply( this  : access Trigger_Spawner; context : Tool_Context ) is

        ------------------------------------------------------------------------

        function Snap( n : Integer ) return Float is
        begin
            if Get_Pref( "gridsnap" ) then
                return Float(Support.Grid_Snap( n, Get_Pref( "gridsize" ) ));
            end if;
            return Float(n);
        end Snap;

        ------------------------------------------------------------------------

        pragma Unreferenced( this );
        tool : A_Tool;
    begin
        if first then
            if func = Primary then
                if context.worldX >= 0 and then context.worldX < context.world.Get_Width and then
                   context.worldY >= 0 and then context.worldY < context.world.Get_Height
                then
                    context.world.Spawn_Entity( External_Tag( Entities.Triggers.Keen.Keen_Trigger'Tag ),
                                                Snap( context.worldX ),
                                                Snap( context.worldY ),
                                                context.world.Tile_Width * 2,
                                                context.world.Tile_Width * 2 );
                end if;
            elsif context.func = Secondary or else context.func = Tertiary then
                tool := Create_Pointer;
                Queue_Set_Tool( tool );
                Delete( tool );
            end if;
        end if;
    end Apply;

    ----------------------------------------------------------------------------

    overriding
    function Get_Type( this : access Trigger_Spawner ) return Tool_Type is
        pragma Unreferenced( this );
    begin
        return Trigger_Tool;
    end Get_Type;

end Tools.Trigger_Spawners;
