--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

with Allegro.Keyboard;                  use Allegro.Keyboard;
--with Debugging;                         use Debugging;
with Events.Entities;                   use Events.Entities;
with Game_Views;                        use Game_Views;
with Support;                           use Support;
with Values.Strings;                    use Values.Strings;
with Widgets.Buttons.Pushes;            use Widgets.Buttons.Pushes;
with Widgets.Input_Boxes;               use Widgets.Input_Boxes;
with Widgets.Labels;                    use Widgets.Labels;

package body Widgets.Containers.Panels.Dialogs.Entity_Attributes is

    VALUES_WIDTH : constant := 300;   -- width of the value column
    SPACING      : constant := 5;

    ----------------------------------------------------------------------------

    procedure Do_Apply( this : not null access Entity_Attributes_Dialog'Class );

    -- END PRIVATE DECLARATIONS --
    --==========================================================================

    function Create_Entity_Attributes_Dialog( view       : not null access Game_Views.Game_View'Class;
                                              id         : String;
                                              eid        : Entity_Id;
                                              class      : String;
                                              attributes : Assoc_Ptr ) return A_Dialog is
        this : A_Dialog := new Entity_Attributes_Dialog;
    begin
        Entity_Attributes_Dialog(this.all).Construct( view, id, eid, class, attributes );
        return this;
    exception
        when others =>
            Delete( A_Widget(this) );
            raise;
    end Create_Entity_Attributes_Dialog;

    ----------------------------------------------------------------------------

    not overriding
    procedure Construct( this       : access Entity_Attributes_Dialog;
                         view       : not null access Game_Views.Game_View'Class;
                         id         : String;
                         eid        : Entity_Id;
                         class      : String;
                         attributes : Assoc_Ptr ) is
        labels      : Widget_Lists.List;
        inputs      : Widget_Lists.List;
        nextTop     : Integer := 0;
        namesWidth  : Integer := 0;
        totalHeight : Integer := 0;

        ------------------------------------------------------------------------

        procedure Add_Attribute( name : String; value : Value_Ptr ) is
            label : A_Label;
            input : A_Input_Box;
        begin
            label := Create_Label( this.Get_View, To_String( this.id & ":lbl" & name ), name & ":" );
            label.Set_Align( Align_Right );
            labels.Append( A_Widget(label) );
            if A_Widget(label).Get_Min_Width > namesWidth then
                namesWidth := A_Widget(label).Get_Min_Width;
            end if;
            totalHeight := totalHeight + SPACING + A_Widget(label).Get_Min_Height;
            this.Add_Widget( A_Widget(label) );

            input := Create_Input_Box( this.Get_View, To_String( this.id & ":ib" & name ) );
            input.Set_Text( As_String( value ).Get.To_String );
            inputs.Append( A_Widget(input) );
            this.Add_Widget( A_Widget(input) );
        exception
            when others =>
                Delete( A_Widget(label) );
                Delete( A_Widget(input) );
                raise;
        end Add_Attribute;

        ------------------------------------------------------------------------

        procedure Layout_Attributes is
            lCrsr     : Widget_Lists.Cursor := labels.First;
            iCrsr     : Widget_Lists.Cursor := inputs.First;
            iNext     : Widget_Lists.Cursor;
            label     : A_Widget;
            input     : A_Widget;
            rowHeight : Integer;
        begin
            if Widget_Lists.Has_Element( iCrsr ) then
                -- attach first input's previous to the cancel button
                Widget_Lists.Element( iCrsr ).Set_Prev( this.id & ":btnCancel" );
                this.Get_View.Get_Widget( this.id & ":btnCancel" ).Set_Next( Widget_Lists.Element( iCrsr ).Get_Id );
                iNext := Widget_Lists.Next( iCrsr );
            end if;

            while Widget_Lists.Has_Element( lCrsr ) loop
                label := Widget_Lists.Element( lCrsr );
                rowHeight := label.Get_Min_Height;
                label.Set_Layout_LTWH( SPACING, nextTop + SPACING, namesWidth, rowHeight );

                input := Widget_Lists.Element( iCrsr );
                input.Set_Layout_LTRB( SPACING + namesWidth + SPACING,
                                       nextTop + SPACING,
                                       SPACING, -(nextTop + SPACING + rowHeight) );

                if Widget_Lists.Has_Element( iNext ) then
                    -- set next to the next input
                    Widget_Lists.Element( iCrsr ).Set_Next( Widget_Lists.Element( iNext ).Get_Id );
                    Widget_Lists.Element( iNext ).Set_Prev( Widget_Lists.Element( iCrsr ).Get_Id );
                else
                    -- this is last; set next to Apply button
                    Widget_Lists.Element( iCrsr ).Set_Next( this.id & ":btnApply" );
                    this.Get_View.Get_Widget( this.id & ":btnApply" ).Set_Prev( Widget_Lists.Element( iCrsr ).Get_Id );
                end if;

                Widget_Lists.Next( lCrsr );
                Widget_Lists.Next( iCrsr );
                Widget_Lists.Next( iNext );
                nextTop := nextTop + SPACING + rowHeight;
            end loop;
        end Layout_Attributes;

        ------------------------------------------------------------------------

        function Dialog_Icon( class : String ) return String is
        begin
            if class = "trigger" then
                return "trigger_icon";
            else
                return "";
            end if;
        end Dialog_Icon;

        ------------------------------------------------------------------------

        theme  : constant A_Theme := Themes.Get;
        button : A_Button;
    begin
        Dialog(this.all).Construct( view, id,
                                    Capitalize( class ) & " Attributes",
                                    Dialog_Icon( class ) );
        this.eAttributes := As_Assoc( attributes.Clone );
        this.eid := eid;

        button := Create_Push_Button( view, this.id & ":btnApply", "Apply" );
        button.Set_Layout_LTWH( -67, -SPACING, 55, 20 );
        button.Set_Next( this.id & ":btnCancel" );
        button.Add_Listener( A_Button_Listener(this) );
        this.Add_Widget( A_Widget(button) );

        button := Create_Push_Button( view, this.id & ":btnCancel", "Cancel" );
        button.Set_Layout_LTWH( -SPACING, -SPACING, 55, 20 );
        button.Set_Prev( this.id & ":btnApply" );
        button.Add_Listener( A_Button_Listener(this) );
        this.Add_Widget( A_Widget(button) );

        this.eAttributes.Get.Iterate( Add_Attribute'Access );
        Layout_Attributes;

        labels.Clear;
        inputs.Clear;

        this.Set_Layout_Center( theme.Border_Width * 2 +
                                Integer'Max( SPACING + namesWidth + SPACING + VALUES_WIDTH + SPACING,
                                             this.titlePadding + theme.Label_Width( theme.Get_Bitmap( this.icon ),
                                                                                    this.Get_Title,
                                                                                    To_String( this.fontName ),
                                                                                    this.fontSize ) + this.titlePadding ),
                                totalHeight + this.Get_Min_Height + 30 );
    exception
        when others =>
            Delete( A_Widget(button) );
            raise;
    end Construct;

    ----------------------------------------------------------------------------

    procedure Do_Apply( this : not null access Entity_Attributes_Dialog'Class ) is
        view : constant A_Game_View := A_Game_View(this.Get_View);

        procedure Apply_Attribute( name : String; value : Value_Ptr ) is
            input : constant A_Input_Box := A_Input_Box(view.Get_Widget( To_String( this.id & ":ib" & name ) ));
        begin
            if As_String( value ) /= input.Get_Text then
                Queue_Set_Entity_Attribute( this.eid, name, Create( input.Get_Text ) );
            end if;
        end Apply_Attribute;

    begin
        this.eAttributes.Get.Iterate( Apply_Attribute'Access );
        this.Hide;
    end Do_Apply;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Action( this   : access Entity_Attributes_Dialog;
                             action : A_Button_Action ) is
    begin
        if action.Get_Id = Click then
            if action.Get_Source.Get_Id = To_String( this.id & ":btnApply" ) then
                this.Do_Apply;
            elsif action.Get_Source.Get_Id = To_String( this.id & ":btnCancel" ) then
                this.Hide;
            end if;
        end if;
    end Handle_Action;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Action( this    : access Entity_Attributes_Dialog;
                             action  : A_Key_Action;
                             handled : out Boolean ) is
    begin
        if action.Get_Key = ALLEGRO_KEY_ESCAPE and then action.Get_Id = Key_Press then
            this.Hide;
            handled := True;
        else
            Dialog(this.all).Handle_Action( action, handled );
        end if;
    end Handle_Action;

    ----------------------------------------------------------------------------

    overriding
    procedure Show( this : access Entity_Attributes_Dialog ) is
    begin
        Dialog(this.all).Show;
        this.Get_View.Get_Widget( this.id & ":btnApply" ).Request_Focus;
    end Show;

end Widgets.Containers.Panels.Dialogs.Entity_Attributes;
