--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

with Ada.Exceptions;                    use Ada.Exceptions;
with Allegro.Keyboard;                  use Allegro.Keyboard;
with Debugging;                         use Debugging;
with Engine_Debugging;                  use Engine_Debugging;
with Events.World;                      use Events.World;
with Game_Views;                        use Game_Views;
with Support;                           use Support;
with Widgets.Buttons.Pushes;            use Widgets.Buttons.Pushes;
with Widgets.Containers.Panels;         use Widgets.Containers.Panels;
with Widgets.Input_Boxes.Constraints;   use Widgets.Input_Boxes.Constraints;
with Widgets.Labels;                    use Widgets.Labels;

package body Widgets.Containers.Panels.Dialogs.New_World is

    procedure Do_Ok( this : not null access New_Dialog'Class );

    -- END PRIVATE DECLARATIONS --
    --==========================================================================

    function Create_New_Dialog( view : not null access Game_Views.Game_View'Class;
                                id   : String ) return A_Dialog is
        this : A_Dialog := new New_Dialog;
    begin
        this.Construct( view, id );
        return this;
    exception
        when others =>
            Delete( A_Widget(this) );
            raise;
    end Create_New_Dialog;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access New_Dialog;
                         view : not null access Game_Views.Game_View'Class;
                         id   : String ) is
        label     : A_Label;
        button    : A_Button;
        input_box : A_Input_Box;
    begin
        Dialog(this.all).Construct( view, id, "New World...", "" );
        this.Set_Layout_Center( 185, 165 );

        label := Create_Label( view, this.id & ":lblWidth", "Width:" );
        label.Set_Layout_LTRB( 5, 5, -60, -23 );
        label.Set_Align( Align_Right );
        this.Add_Widget( A_Widget(label) );

        input_box := Create_Input_Box( view, this.id & ":ibWidth" );
        input_box.Set_Layout_LTRB( 65, 5, -115, -23 );
        input_box.Set_Constraint( Constraints.Int_Only'Access );
        input_box.Add_Listener( A_Input_Listener(this) );
        input_box.Set_Max_Length( 3 );
        input_box.Set_Prev( this.id & ":btnCancel" );
        input_box.Set_Next( this.id & ":ibHeight" );
        this.Add_Widget( A_Widget(input_box) );

        label := Create_Label( view, this.id & ":lblHeight", "Height:" );
        label.Set_Layout_LTRB( 5, 26, -60, -44 );
        label.Set_Align( Align_Right );
        this.Add_Widget( A_Widget(label) );

        input_box := Create_Input_Box( view, this.id & ":ibHeight" );
        input_box.Set_Layout_LTRB( 65, 26, -115, -44 );
        input_box.Set_Constraint( Constraints.Int_Only'Access );
        input_box.Add_Listener( A_Input_Listener(this) );
        input_box.Set_Max_Length( 3 );
        input_box.Set_Prev( this.id & ":ibWidth" );
        input_box.Set_Next( this.id & ":ibLib" );
        this.Add_Widget( A_Widget(input_box) );

        label := Create_Label( view, this.id & ":lblLib", "Library:" );
        label.Set_Layout_LTRB( 5, 68, -60, -86 );
        label.Set_Align( Align_Right );
        this.Add_Widget( A_Widget(label) );

        input_box := Create_Input_Box( view, this.id & ":ibLib" );
        input_box.Set_Layout_LTRB( 65, 68, 5, -86 );
        input_box.Set_Constraint( Constraints.Basename_Only'Access );
        input_box.Add_Listener( A_Input_Listener(this) );
        input_box.Set_Prev( this.id & ":ibHeight" );
        input_box.Set_Next( this.id & ":ibDomain" );
        this.Add_Widget( A_Widget(input_box) );

        label := Create_Label( view, this.id & ":lblDomain", "Domain:" );
        label.Set_Layout_LTRB( 5, 89, -60, -107 );
        label.Set_Align( Align_Right );
        this.Add_Widget( A_Widget(label) );

        input_box := Create_Input_Box( view, this.id & ":ibDomain" );
        input_box.Set_Layout_LTRB( 65, 89, 5, -107 );
        input_box.Set_Constraint( Constraints.Basename_Only'Access );
        input_box.Add_Listener( A_Input_Listener(this) );
        input_box.Set_Prev( this.id & ":ibLib" );
        input_box.Set_Next( this.id & ":btnOk" );
        this.Add_Widget( A_Widget(input_box) );

        button := Create_Push_Button( view, this.id & ":btnOk", "OK" );
        button.Set_Layout_LTWH( -63, -5, 55, 20 );
        button.Add_Listener( A_Button_Listener(this) );
        button.Set_Prev( this.id & ":ibDomain" );
        button.Set_Next( this.id & ":btnCancel" );
        this.Add_Widget( A_Widget(button) );

        button := Create_Push_Button( view, this.id & ":btnCancel", "Cancel" );
        button.Set_Layout_LTWH( -5, -5, 55, 20 );
        button.Add_Listener( A_Button_Listener(this) );
        button.Set_Prev( this.id & ":btnOk" );
        button.Set_Next( this.id & ":ibWidth" );
        this.Add_Widget( A_Widget(button) );
    exception
        when others =>
            Delete( A_Widget(label) );
            Delete( A_Widget(button) );
            Delete( A_Widget(input_box) );
            raise;
    end Construct;

    ----------------------------------------------------------------------------

    procedure Do_Ok( this : not null access New_Dialog'Class ) is
        view    : constant A_Game_View := A_Game_View(this.Get_View);
        wstr    : constant String := A_Input_Box(view.Get_Widget( this.id & ":ibWidth" )).Get_Text;
        hstr    : constant String := A_Input_Box(view.Get_Widget( this.id & ":ibHeight" )).Get_Text;
        libName : constant String := A_Input_Box(view.Get_Widget( this.id & ":ibLib" )).Get_Text;
        domain  : constant String := A_Input_Box(view.Get_Widget( this.id & ":ibDomain" )).Get_Text;
    begin
        if wstr'Length    > 0 and then
           hstr'Length    > 0 and then
           libName'Length > 0 and then
           domain'Length  > 0
        then
            declare
                width  : constant Integer := Integer'Value( wstr );
                height : constant Integer := Integer'Value( hstr );
            begin
                if width > 0 and then height > 0 then
                    Trigger_Create_World( width, height, libName, domain );
                    this.Hide;
                else
                    raise Constraint_Error with "Invalid map size";
                end if;
            exception
                when e : others =>
                    Dbg( "Error creating world: " & Exception_Message( e ),
                         D_VIEW, ERROR );
            end;
        end if;
    end Do_Ok;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Action( this   : access New_Dialog;
                             action : A_Button_Action ) is
    begin
        if action.Get_Id = Click then
            if action.Get_Source.Get_Id = To_String( this.id & ":btnOk" ) then
                this.Do_Ok;
            elsif action.Get_Source.Get_Id = To_String( this.id & ":btnCancel" ) then
                this.Hide;
            end if;
        end if;
    end Handle_Action;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Action( this   : access New_Dialog;
                             action : A_Input_Action ) is
    begin
        if action.Get_Id = Entered then
            this.Do_Ok;
        end if;
    end Handle_Action;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Action( this    : access New_Dialog;
                             action  : A_Key_Action;
                             handled : out Boolean ) is
    begin
        if action.Get_Key = ALLEGRO_KEY_ESCAPE and then
           action.Get_Id = Key_Press
        then
            this.Hide;
            handled := True;
        else
            Dialog(this.all).Handle_Action( action, handled );
        end if;
    end Handle_Action;

    ----------------------------------------------------------------------------

    overriding
    procedure On_Dialog_Show( this : access New_Dialog ) is
        view : constant A_Game_View := A_Game_View(this.Get_View);
    begin
        A_Input_Box(view.Get_Widget( this.id & ":ibWidth" )).Set_Text( "" );
        A_Input_Box(view.Get_Widget( this.id & ":ibHeight" )).Set_Text( "" );
        A_Input_Box(view.Get_Widget( this.id & ":ibLib" )).Set_Text( "" );
        A_Input_Box(view.Get_Widget( this.id & ":ibDomain" )).Set_Text( "" );
    end On_Dialog_Show;

    ----------------------------------------------------------------------------

    overriding
    procedure Show( this : access New_Dialog ) is
    begin
        Dialog(this.all).Show;
        this.Get_View.Get_Widget( this.id & ":ibWidth" ).Request_Focus;
    end Show;

end Widgets.Containers.Panels.Dialogs.New_World;
