--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

with Allegro.Keyboard;                  use Allegro.Keyboard;
--with Debugging;                         use Debugging;
with Game_Views;                        use Game_Views;
with Game_Views.Ked;                    use Game_Views.Ked;
with Support;                           use Support;
with Widgets.Buttons.Pushes;            use Widgets.Buttons.Pushes;
with Widgets.Containers.Panels;         use Widgets.Containers.Panels;
with Widgets.Labels;                    use Widgets.Labels;

package body Widgets.Containers.Panels.Dialogs.Save_Changes is

    procedure Do_No( this : not null access Save_Changes_Dialog'Class );

    procedure Do_Yes( this : not null access Save_Changes_Dialog'Class );

    -- END PRIVATE DECLARATIONS --
    --==========================================================================

    function Create_Save_Changes_Dialog( view : not null access Game_Views.Game_View'Class;
                                         id   : String ) return A_Dialog is
        this : A_Dialog := new Save_Changes_Dialog;
    begin
        this.Construct( view, id );
        return this;
    exception
        when others =>
            Delete( A_Widget(this) );
            raise;
    end Create_Save_Changes_Dialog;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Save_Changes_Dialog;
                         view : not null access Game_Views.Game_View'Class;
                         id   : String ) is
        button : A_Button;
        label  : A_Label;
    begin
        Dialog(this.all).Construct( view, id, "Save Changes?", "" );
        this.Set_Layout_Center( 300, 93 );

        label := Create_Label( view, this.id & ":lbl1", "The world has been modified." );
        label.Set_Layout_CenterH( 290, 5, -23 );
        label.Set_Align( Align_Center );
        this.Add_Widget( A_Widget(label) );

        label := Create_Label( view, this.id & ":lbl2", "Do you want to save your changes?" );
        label.Set_Layout_CenterH( 290, 24, -42 );
        label.Set_Align( Align_Center );
        this.Add_Widget( A_Widget(label) );

        button := Create_Push_Button( view, this.id & ":btnYes", "Yes" );
        button.Set_Layout_LTWH( 65, -5, 55, 20 );
        button.Add_Listener( A_Button_Listener(this) );
        button.Set_Prev( this.id & ":btnCancel" );
        button.Set_Next( this.id & ":btnNo" );
        this.Add_Widget( A_Widget(button) );

        button := Create_Push_Button( view, this.id & ":btnNo", "No" );
        button.Set_Layout_LTWH( 123, -5, 55, 20 );
        button.Add_Listener( A_Button_Listener(this) );
        button.Set_Prev( this.id & ":btnYes" );
        button.Set_Next( this.id & ":btnCancel" );
        this.Add_Widget( A_Widget(button) );

        button := Create_Push_Button( view, this.id & ":btnCancel", "Cancel" );
        button.Set_Layout_LTWH( 181, -5, 55, 20 );
        button.Add_Listener( A_Button_Listener(this) );
        button.Set_Prev( this.id & ":btnNo" );
        button.Set_Next( this.id & ":btnYes" );
        this.Add_Widget( A_Widget(button) );
    exception
        when others =>
            Delete( A_Widget(button) );
            Delete( A_Widget(label) );
            raise;
    end Construct;

    ----------------------------------------------------------------------------

    procedure Do_No( this : not null access Save_Changes_Dialog'Class ) is
    begin
        this.Hide;
        A_Ked_View(this.Get_View).Do_Dialog_Action( saveChanges => False );
    end Do_No;

    ----------------------------------------------------------------------------

    procedure Do_Yes( this : not null access Save_Changes_Dialog'Class ) is
    begin
        this.Hide;
        A_Ked_View(this.Get_View).Do_Dialog_Action( saveChanges => True );
    end Do_Yes;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Action( this   : access Save_Changes_Dialog;
                             action : A_Button_Action ) is
    begin
        if action.Get_Source.Get_Id = To_String( this.id & ":btnYes" ) then
            this.Do_Yes;
        elsif action.Get_Source.Get_Id = To_String( this.id & ":btnNo" ) then
            this.Do_No;
        elsif action.Get_Source.Get_Id = To_String( this.id & ":btnCancel" ) then
            this.Hide;
        end if;
    end Handle_Action;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Action( this    : access Save_Changes_Dialog;
                             action  : A_Key_Action;
                             handled : out Boolean ) is
    begin
        handled := False;
        if action.Get_Id = Key_Press then
            if action.Get_Key = ALLEGRO_KEY_ESCAPE then
                this.Hide;
                handled := True;
            elsif action.Get_Key = ALLEGRO_KEY_N then
                this.Do_No;
                handled := True;
            elsif action.Get_Key = ALLEGRO_KEY_Y then
                this.Do_Yes;
                handled := True;
            end if;
        end if;
        if not handled then
            Dialog(this.all).Handle_Action( action, handled );
        end if;
    end Handle_Action;

    ----------------------------------------------------------------------------

    overriding
    procedure Show( this : access Save_Changes_Dialog ) is
    begin
        Dialog(this.all).Show;
        this.Get_View.Get_Widget( this.id & ":btnYes" ).Request_Focus;
    end Show;

end Widgets.Containers.Panels.Dialogs.Save_Changes;
