--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

private with Physics.Clip_Maps;

package Widgets.Containers.Scenes.Ked is

    type Ked_Scene is new Scene with private;
    type A_Ked_Scene is access all Ked_Scene'Class;

    function Create_Scene( view : not null access Game_Views.Game_View'Class;
                           id   : String ) return A_Ked_Scene;
    pragma Precondition( id'Length > 0 );
    pragma Postcondition( Create_Scene'Result /= null );

    -- Returns the layer currently active for tools.
    function Get_Active_Layer( this : not null access Ked_Scene'Class ) return Natural;

    -- Returns the scene's X focus in world coordinates.
    function Get_Focus_X( this : not null access Ked_Scene'Class ) return Float;

    -- Returns the scene's Y focus in world coordinates.
    function Get_Focus_Y( this : not null access Ked_Scene'Class ) return Float;

    -- Sets the active layer for editing tiles.
    procedure Set_Active_Layer( this  : not null access Ked_Scene'Class;
                                layer : Positive );

    -- Sets the visibility for a layer.
    procedure Set_Layer_Visible( this    : not null access Ked_Scene'Class;
                                 layer   : Positive;
                                 visible : Boolean );

    -- Moves the focus point of the scene relative to it's current location.
    procedure Shift_Focus( this : not null access Ked_Scene'Class; x,y : Float );

    -- Zooms in, keeping the scene focus centered.
    procedure Zoom_In( this : not null access Ked_Scene'Class );

    -- Zooms out, keeping the scene focus centered.
    procedure Zoom_Out( this : not null access Ked_Scene'Class );

    -- Sets the absolute zoom factor for the scene, keeping the current focus
    -- point centered.
    procedure Zoom_With_Focus( this   : not null access Ked_Scene'Class;
                               factor : Float );

private

    use Physics.Clip_Maps;

    type Ked_Scene is new Scene with
        record
            leftMouse       : Boolean := False;
            rightMouse      : Boolean := False;
            middleMouse     : Boolean := False;
            activeModifiers : Modifiers_Array := Modifiers_Array'(others=>False);
            activeLayer     : Natural := 0;
            scrollSpeed     : Float := 0.0;
            clipMap         : A_Clip_Map := null;
            dragStartX,
            dragStartY      : Integer := -1;
        end record;

    procedure Construct( this : access Ked_Scene;
                         view : not null access Game_Views.Game_View'Class;
                         id   : String );
    pragma Precondition( id'Length > 0 );

    procedure Delete( this : in out Ked_Scene );

    procedure Draw_Layer( this   : access Ked_Scene;
                          layer  : Positive;
                          startX,
                          startY : Integer;
                          tileX1,
                          tileY1,
                          tileX2,
                          tileY2 : Integer );

    procedure Draw_Layer_Overlay( this   : access Ked_Scene;
                                  startX,
                                  startY : Integer;
                                  tileX1,
                                  tileY1,
                                  tileX2,
                                  tileY2 : Integer );

    procedure Handle_Entity_Created( this : access Ked_Scene;
                                     evt  : not null A_Entity_Created_Event );

    procedure Handle_Tile_Changed( this : access Ked_Scene;
                                   evt  : not null A_Tile_Changed_Event );

    procedure Handle_World_Loaded( this : access Ked_Scene;
                                   evt  : not null A_World_Loaded_Event );

    procedure On_Exit( this : access Ked_Scene );

    procedure On_Mouse_Move( this : access Ked_Scene;
                             evt  : not null A_Mouse_Event );

    procedure On_Mouse_Press( this : access Ked_Scene;
                              evt  : not null A_Mouse_Button_Event );

    procedure On_Mouse_Release( this : access Ked_Scene;
                                evt  : not null A_Mouse_Button_Event );

    procedure On_Mouse_Scroll( this    : access Ked_Scene;
                               x, y    : Integer;
                               amount  : Integer;
                               handled : in out Boolean );

    procedure On_Resize( this : access Ked_Scene );

end Widgets.Containers.Scenes.Ked;
