--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

with Events.Listeners;                  use Events.Listeners;
with Tiles.Libraries;                   use Tiles.Libraries;

private with Ada.Containers.Vectors;
private with Events;

package Widgets.Palettes.Tiles is

    type Tile_Palette is abstract new Palette and Event_Listener with private;

    function Create_Tile_Palette( view : not null access Game_Views.Game_View'Class;
                                  id   : String ) return A_Palette;

    -- Raises an error if tile library 'name' can't be loaded.
    procedure Set_Library_Name( this : not null access Tile_Palette'Class;
                                name : String );

private

    use Events;

    package Integer_Vectors is new Ada.Containers.Vectors( Positive, Natural, "=" );

    ----------------------------------------------------------------------------

    type Tile_Palette is new Palette and Event_Listener with
        record
            lib   : A_Tile_Library := null;
            slots : Integer_Vectors.Vector;
        end record;

    procedure Construct( this : access Tile_Palette;
                         view : not null access Game_Views.Game_View'Class;
                         id   : String );

    procedure Delete( this : in out Tile_Palette );

    procedure Get_Item_Details( this   : access Tile_Palette;
                                row,
                                col    : Natural;
                                bmp    : out A_Allegro_Bitmap;
                                border : out Allegro_Color );

    function Get_Scroll_Inc_Y( this : access Tile_Palette ) return Integer;

    function Get_Item_Count( this : access Tile_Palette ) return Natural;

    procedure Handle_Event( this : access Tile_Palette;
                            evt  : in out A_Event;
                            resp : out Response_Type );

    procedure On_Mouse_Press( this : access Tile_Palette;
                              evt  : not null A_Mouse_Button_Event );

end Widgets.Palettes.Tiles;
