--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

--with Debugging;                         use Debugging;
with Resources;                         use Resources;
with Support.Paths;                     use Support.Paths;
with Themes;                            use Themes;
with Tiles.Libraries.Manager;           use Tiles.Libraries.Manager;

pragma Elaborate_All( Themes );

package body Themes.Keen is

    LIB_NAME : constant String := "interface";

    -- END PRIVATE DECLARATIONS --
    --==========================================================================

    function Create_Theme return A_Theme is
        this : A_Theme := new Keen_Theme;
    begin
        this.Construct;
        return this;
    exception
        when others =>
            Delete( this );
            raise;
    end Create_Theme;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Keen_Theme ) is
    begin
        Theme(this.all).Construct;
        this.colors(Background)    := Al_Map_RGBA( 148, 145, 140, 255 );
        this.colors(Foreground)    := Al_Map_RGBA( 212, 208, 200, 255 );
        this.colors(Selected)      := Al_Map_RGBA(  58,  95, 205, 255 );
        this.colors(Selected_Text) := Al_Map_RGBA( 255, 255, 255, 255 );
        this.colors(Text)          := Al_Map_RGBA(   0,   0,   0, 255 );
        this.Register_Font( "dejavusans.ttf", DEFAULT_FONT_NAME );  -- may raise exception
        this.Register_Font( "dosfont.ttf", "dos" );                 -- may raise exception
        this.Register_Font( "keenmenu.ttf", "menu" );
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure Delete( this : in out Keen_Theme ) is
    begin
        Unload_Library( this.lib );
        Theme(this).Delete;
    end Delete;

    ----------------------------------------------------------------------------

    overriding
    function Get_Bitmap( this : access Keen_Theme;
                         id   : Natural ) return A_Allegro_Bitmap is
    begin
        if this.lib = null then
            this.lib := Load_Library( LIB_NAME, bitmaps => True );
        end if;
        if this.lib /= null then
            return this.lib.Get_Tile( id ).Get_Bitmap;
        end if;
        return null;
    end Get_Bitmap;

    ----------------------------------------------------------------------------

    overriding
    function Get_ID( this : access Keen_Theme; name : String ) return Natural is
        id : Natural;
    begin
        if this.lib = null then
            this.lib := Load_Library( LIB_NAME, bitmaps => True );
        end if;
        if this.lib /= null then
            id := this.lib.Get_Id( name );
            if id = 0 then
                id := this.lib.Get_Id( Add_Extension( name, "png" ) );
                if id = 0 then
                    id := this.lib.Get_Id( Add_Extension( name, "bmp" ) );
                end if;
            end if;
            return id;
        else
            --return 0;
            raise RESOURCE_ERROR with "Error loading tile library <" & LIB_NAME & ">";
        end if;
    end Get_ID;

    ----------------------------------------------------------------------------

    overriding
    function Get_Tile( this : access Keen_Theme; id : Natural ) return A_Tile is
    begin
        if this.lib = null then
            this.lib := Load_Library( LIB_NAME, bitmaps => True );
        end if;
        if this.lib /= null then
            return this.lib.Get_Tile( id );
        end if;
        return null;
    end Get_Tile;

begin

    Register_Allocator( Create_Theme'Access );

end Themes.Keen;
