--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

with Allegro.Keyboard;                  use Allegro.Keyboard;
with Game_Views;                        use Game_Views;
with Palette_16;

package body Widgets.Buttons.Pushes.Menu_Buttons is

    BLINK_DELAY   : constant Time_Span := Milliseconds( 500 );

    ON_ICON       : constant String := "menu_dot_on.png";
    OFF_ICON      : constant String := "menu_dot_off.png";
    DISABLED_ICON : constant String := "menu_dot_disabled.png";

    -- END PRIVATE DECLARATIONS --
    --==========================================================================

    function Create_Menu_Button( view : not null access Game_Views.Game_View'Class;
                                 id   : String;
                                 text : String := "" ) return A_Button is
        this : A_Menu_Button := new Menu_Button;
    begin
        this.Construct( view, id, text );
        return A_Button(this);
    exception
        when others =>
            Delete( A_Widget(this) );
            raise;
    end Create_Menu_Button;

    ----------------------------------------------------------------------------

    not overriding
    procedure Construct( this : access Menu_Button;
                         view : not null access Game_Views.Game_View'Class;
                         id   : String;
                         text : String ) is
    begin
        Push_Button(this.all).Construct( view, id, text, OFF_ICON );
        this.border := Border_None;
        this.align := Align_Left;
        this.focusTime := Time_Span_Zero;
        this.Set_Color( Themes.Text, Palette_16.Green );
        this.Set_Color( Themes.Selected, Palette_16.Dark_Grey );
        this.Set_Color( Themes.Foreground, Palette_16.Dark_Grey );
        this.Set_Shade_Disabled( 1.0 );
        this.Set_Font( "menu", 10 );
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure On_Blur( this : access Menu_Button ) is
    begin
        Push_Button(this.all).On_Blur;
        this.Set_Color( Themes.Text, Palette_16.Green );
        this.Set_Icon( OFF_ICON );
    end On_Blur;

    ----------------------------------------------------------------------------

    overriding
    procedure On_Disabled( this : access Menu_Button ) is
    begin
        this.Set_Icon( DISABLED_ICON );
    end On_Disabled;

    ----------------------------------------------------------------------------

    overriding
    procedure On_Enabled( this : access Menu_Button ) is
    begin
        this.Set_Icon( OFF_ICON );
    end On_Enabled;

    ----------------------------------------------------------------------------

    overriding
    procedure On_Focus( this : access Menu_Button ) is
    begin
        this.Set_Color( Themes.Text, Palette_16.Light_Green );
    end On_Focus;

    ----------------------------------------------------------------------------

    overriding
    procedure On_Key_Press( this    : access Menu_Button;
                            evt     : not null A_Key_Event;
                            handled : in out Boolean ) is
    begin
        if evt.Get_Key = ALLEGRO_KEY_DOWN then
            if this.Focus_Next then
                handled := True;
                return;
            end if;
        elsif evt.Get_Key = ALLEGRO_KEY_UP then
            if this.Focus_Prev then
                handled := True;
                return;
            end if;
        elsif evt.Get_Key = ALLEGRO_KEY_TAB then
            -- prevent the default tab-focus-switching behavior
            handled := True;
            return;
        end if;

        Push_Button(this.all).On_Key_Press( evt, handled );
    end On_Key_Press;

    ----------------------------------------------------------------------------

    overriding
    procedure Tick( this : access Menu_Button; time : Tick_Time ) is
    begin
        if this.focused then
            if ((time.total - this.focusTime) / BLINK_DELAY) mod 2 = 0 then
                this.Set_Icon( ON_ICON );
            else
                this.Set_Icon( OFF_ICON );
            end if;
        else
            this.focusTime := time.total;
        end if;
    end Tick;

end Widgets.Buttons.Pushes.Menu_Buttons;
