--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

with Ada.Real_Time;                     use Ada.Real_Time;
with Allegro.Keyboard;                  use Allegro.Keyboard;
with Drawing;                           use Drawing;
with Palette_16;

package body Widgets.Buttons.Pushes.Retry_Buttons is

    BLINK_DELAY : constant Time_Span := Milliseconds( 250 );

    -- END PRIVATE DECLARATIONS --
    --==========================================================================

    function Create_Retry_Button( view : not null access Game_Views.Game_View'Class;
                                  id   : String;
                                  text : String := "";
                                  icon : String := "" ) return A_Button is
        this : A_Retry_Button := new Retry_Button;
    begin
        this.Construct( view, id, text, icon );
        return A_Button(this);
    exception
        when others =>
            Delete( A_Widget(this) );
            raise;
    end Create_Retry_Button;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Retry_Button;
                         view : not null access Game_Views.Game_View'Class;
                         id   : String;
                         text : String;
                         icon : String ) is
    begin
        Push_Button(this.all).Construct( view, id, text, icon );
        this.border := Border_None;
        this.Set_Color( Selected, Palette_16.Light_Red );
        this.Set_Font( "dos", 10 );
        this.Set_Shade_On( 1.0 );
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure Draw_Content( this : access Retry_Button ) is
    begin
        Push_Button(this.all).Draw_Content;

        if this.focused then
            -- draw the thick flashing border around the widget
            Rect( 0, 0, this.cwidth - 1, this.cheight - 1,
                  this.colors(Selected) );
            Rect( 1, 1, this.cwidth - 2, this.cheight - 2,
                  this.colors(Selected) );
        end if;
    end Draw_Content;

    ----------------------------------------------------------------------------

    overriding
    procedure On_Key_Press( this    : access Retry_Button;
                            evt     : not null A_Key_Event;
                            handled : in out Boolean ) is
    begin
        if evt.Get_Key = ALLEGRO_KEY_DOWN then
            if this.Focus_Next then
                handled := True;
            end if;
        elsif evt.Get_Key = ALLEGRO_KEY_UP then
            if this.Focus_Prev then
                handled := True;
            end if;
        elsif evt.Get_Key = ALLEGRO_KEY_TAB then
            -- prevent the default tab-focus-switching behavior
            handled := True;
        end if;
    end On_Key_Press;

    ----------------------------------------------------------------------------

    overriding
    procedure Tick( this : access Retry_Button; time : Tick_Time ) is
    begin
        if this.focused then
            if (time.total / BLINK_DELAY) mod 2 = 0 then
                this.Set_Color( Selected, Palette_16.Light_Red );
            else
                this.Set_Color( Selected, Palette_16.Blue );
            end if;
        end if;
    end Tick;

end Widgets.Buttons.Pushes.Retry_Buttons;
