--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

with Events.Listeners;                  use Events.Listeners;

private with Allegro.Keyboard;
private with Entities;
private with Events;

package Widgets.Containers.Game_Screens.Game_Play is

    -- The game play screen contains the main Scene widget where all game play
    -- occurs. It is responsible for input to the scene during game play,
    -- and handling events relevant to the scene, the progress board, and score
    -- board.
    type Game_Play_Screen is new Game_Screen and Event_Listener with private;

    -- Creates a new game play screen.
    function Create_Game_Play_Screen( view : not null access Game_Views.Game_View'Class;
                                      id   : String ) return A_Game_Screen;

private

    use Allegro.Keyboard;
    use Entities;
    use Events;

    type Key_Purpose is (
        MOVE_UP,        -- move/look up
        MOVE_DOWN,      -- move/look down
        MOVE_LEFT,      -- move left
        MOVE_RIGHT,     -- move right
        JUMP,           -- jump up
        POGO,           -- toggle pogostick
        SHOOT,          -- shoot stunner gun
        PROGRESS,       -- toggle progress board
        PAUSE,          -- pause/resume play
        MENU            -- exit to menu
    );

    type Key_Binding_Array is array (Key_Purpose) of Integer;

    type Boolean_Key_Array is array (1..ALLEGRO_KEY_MAX) of Boolean;

    type Game_Play_Screen is new Game_Screen and Event_Listener with
        record
            -- an array of key scancodes bound to key purposes
            key : Key_Binding_Array;

            -- events from these keys sent to the scene should be ignored until
            -- the next key press event.
            ignoreKey : Boolean_Key_Array := Boolean_Key_Array'(others => False);

            -- the id of the player entity, for sending impulses.
            playerId : Entity_Id := INVALID_ID;
        end record;

    procedure Construct( this : access Game_Play_Screen;
                         view : not null access Game_Views.Game_View'Class;
                         id   : String );

    procedure Delete( this : in out Game_Play_Screen );

    procedure Activate( this : access Game_Play_Screen );

    -- Inherited from the Event_Listener interface. Handles events that the view
    -- is registered to receive.
    procedure Handle_Event( this : access Game_Play_Screen;
                            evt  : in out A_Event;
                            resp : out Response_Type );
    pragma Precondition( evt /= null );

end Widgets.Containers.Game_Screens.Game_Play;
