--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

with Allegro.Keyboard;                  use Allegro.Keyboard;
--with Debugging;                         use Debugging;
with Drawing;                           use Drawing;
with Game_Views;                        use Game_Views;
with Preferences;                       use Preferences;
with Support;                           use Support;
with Widgets.Simple_Key_Listeners;
with Widgets.Menu_Enumerations;          use Widgets.Menu_Enumerations;
with Widgets.Menu_Enumerations.Simple_Listeners;

pragma Elaborate_All( Widgets.Menu_Enumerations.Simple_Listeners );
pragma Elaborate_All( Widgets.Simple_Key_Listeners );

package body Widgets.Containers.Game_Screens.Menus.Config is

    package Enum_Listeners is new Widgets.Menu_Enumerations.Simple_Listeners(Config_Screen);
    use Enum_Listeners;

    package Key_Listeners is new Widgets.Simple_Key_Listeners(Config_Screen);
    use Key_Listeners;

    ----------------------------------------------------------------------------

--    procedure Action_Do_Nothing( this : not null access Config_Screen'Class );

    procedure Action_Escape( this   : not null access Config_Screen'Class;
                             action : A_Key_Action );

    procedure Action_Filter( this   : not null access Config_Screen'Class;
                             action : A_Enum_Action );

    -- END PRIVATE DECLARATIONS --
    --==========================================================================

    function Create_Config_Screen( view : not null access Game_Views.Game_View'Class;
                                   id   : String ) return A_Game_Screen is
        this : A_Game_Screen := new Config_Screen;
    begin
        this.Construct( view, id );
        return this;
    exception
        when others =>
            Delete( A_Widget(this) );
            raise;
    end Create_Config_Screen;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Config_Screen;
                         view : not null access Game_Views.Game_View'Class;
                         id   : String ) is

        enum : A_Menu_Enumeration;
    begin
        Menu_Screen(this.all).Construct( view, id, popup => False );
        this.Set_Background( "config_menu" );

        this.Add_Listener( Listener( ALLEGRO_KEY_ESCAPE, Key_Press, this, Action_Escape'Access ) );

        -- Set scaling filter
        enum := Create_Menu_Enum( view, this.id & ":enFilter" );
        enum.Add_Listener( Listener( this, Action_Filter'Access ) );
        for f in Filter_Type'Range loop
            enum.Add_Option( "Scaling: " & To_Name( f ) );
        end loop;
        enum.Set_Index( Filter_Type'Pos( To_Filter( Get_Pref( "scaling_filter" ),
                                                    Filter_xBR ) ) + 1,
                        fireAction => False );
        this.Add_Item( enum );
    exception
        when others =>
            Delete( A_Widget(enum) );
            raise;
    end Construct;

    ----------------------------------------------------------------------------

--    procedure Action_Do_Nothing( this : not null access Config_Screen'Class ) is
--        pragma Unreferenced( this );
--    begin
--        null;
--    end Action_Do_Nothing;

    ----------------------------------------------------------------------------

    procedure Action_Escape( this   : not null access Config_Screen'Class;
                             action : A_Key_Action ) is
        pragma Unreferenced( action );
    begin
        this.Exit_Screen;
    end Action_Escape;

    ----------------------------------------------------------------------------

    procedure Action_Filter( this   : not null access Config_Screen'Class;
                             action : A_Enum_Action ) is
        filter : constant Filter_Type := Filter_Type'Val( action.Get_Index - 1 );
    begin
        this.Get_Window.Set_Filter( filter );
        Set_Pref( "scaling_filter", To_Name( filter ) );
    end Action_Filter;

    ----------------------------------------------------------------------------

    overriding
    procedure Activate( this : access Config_Screen ) is
    begin
        Menu_Screen(this.all).Activate;
        this.Get_View.Get_Widget( this.id & ":enFilter" ).Request_Focus;
    end Activate;

end Widgets.Containers.Game_Screens.Menus.Config;
