--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

--with Debugging;                         use Debugging;
with Game_Views.Keen;                   use Game_Views.Keen;
with Support;                           use Support;
with Widgets.Buttons.Simple_Listeners;
with Widgets.Buttons.Pushes.Menu_Buttons;
 use Widgets.Buttons.Pushes.Menu_Buttons;
with Widgets.Containers.Game_Screens.Menus.Config;
 use Widgets.Containers.Game_Screens.Menus.Config;
with Widgets.Containers.Game_Screens.Paddle_War;
 use Widgets.Containers.Game_Screens.Paddle_War;
with Widgets.Containers.Screen_Managers;
 use Widgets.Containers.Screen_Managers;

pragma Elaborate_All( Widgets.Buttons.Simple_Listeners );

package body Widgets.Containers.Game_Screens.Menus.Main is

    package Button_Listeners is new Widgets.Buttons.Simple_Listeners(Main_Menu_Screen);
    use Button_Listeners;

    ----------------------------------------------------------------------------

    -- Called when the user attempts to open the configure menu.
    procedure Action_Configure( this : not null access Main_Menu_Screen'Class );

    -- Called when the user attempts to start a new game. This procedure
    -- ensures that a new game may be started and then queues a New_Game event.
    procedure Action_New_Game( this : not null access Main_Menu_Screen'Class );

    -- Called when the user attempts to open the paddle war game.
    procedure Action_Paddle_War( this : not null access Main_Menu_Screen'Class );

   -- Called when the user attempts to quit the game.
    procedure Action_Quit( this : not null access Main_Menu_Screen'Class );

    -- Called when the user attempts to resume a game in progress.
    procedure Action_Resume_Game( this : not null access Main_Menu_Screen'Class );

    -- END PRIVATE DECLARATIONS --
    --==========================================================================

    function Create_Main_Menu_Screen( view           : not null access Game_Views.Game_View'Class;
                                      id             : String;
                                      gameInProgress : Boolean ) return A_Game_Screen is
        this : A_Game_Screen := new Main_Menu_Screen;
    begin
        Main_Menu_Screen(this.all).Construct( view, id, gameInProgress );
        return this;
    exception
        when others =>
            Delete( A_Widget(this) );
            raise;
    end Create_Main_Menu_Screen;

    ----------------------------------------------------------------------------

    overriding   -- overriding because last parameters differ in name only
    procedure Construct( this           : access Main_Menu_Screen;
                         view           : not null access Game_Views.Game_View'Class;
                         id             : String;
                         gameInProgress : Boolean ) is
        button : A_Button;
    begin
        Menu_Screen(this.all).Construct( view, id, popup => False );
        this.gameInProgress := gameInProgress;
        this.Set_Background( "main_menu" );

        -- New Game button
        button := Create_Menu_Button( view, this.id & ":btnNewGame", "NEW GAME" );
        button.Add_Listener( Listener( Press, this, Action_New_Game'Access ) );
        this.Add_Item( button );

        -- Load Game button
        button := Create_Menu_Button( view, this.id & ":btnLoadGame", "LOAD GAME" );
        button.Set_Enabled( False );
        this.Add_Item( button );

        -- Save Game button
        button := Create_Menu_Button( view, this.id & ":btnSaveGame", "SAVE GAME" );
        button.Set_Enabled( False );
        this.Add_Item( button );

        -- Configure button
        button := Create_Menu_Button( view, this.id & ":btnConfigure", "CONFIGURE" );
        button.Add_Listener( Listener( Press, this, Action_Configure'Access ) );
        this.Add_Item( button );

        -- Resume Game button
        button := Create_Menu_Button( view, this.id & ":btnResumeGame", "BACK TO GAME" );
        button.Set_Enabled( this.gameInProgress );
        button.Add_Listener( Listener( Press, this, Action_Resume_Game'Access ) );
        this.Add_Item( button );

        -- End Game button
        button := Create_Menu_Button( view, this.id & ":btnEndGame", "END GAME" );
        button.Set_Enabled( False );
        this.Add_Item( button );

        -- Paddle War button
        button := Create_Menu_Button( view, this.id & ":btnPaddleWar", "PADDLE WAR" );
        button.Add_Listener( Listener( Press, this, Action_Paddle_War'Access ) );
        this.Add_Item( button );

        -- Quit button
        button := Create_Menu_Button( view, this.id & ":btnQuitGame", "QUIT" );
        button.Add_Listener( Listener( Press, this, Action_Quit'Access ) );
        this.Add_Item( button );
    exception
        when others =>
            Delete( A_Widget(button) );
            raise;
    end Construct;

    ----------------------------------------------------------------------------

    procedure Action_Configure( this : not null access Main_Menu_Screen'Class ) is
        screen : A_Game_Screen;
    begin
        screen := Create_Config_Screen( this.Get_View, "scrConfigMenu" );
        A_Screen_Manager(this.parent).Add_Screen( screen );
    end Action_Configure;

    ----------------------------------------------------------------------------

    procedure Action_New_Game( this : not null access Main_Menu_Screen'Class ) is
    begin
        this.Exit_Screen;
        this.Get_View.New_Game;
    end Action_New_Game;

    ----------------------------------------------------------------------------

    procedure Action_Paddle_War( this : not null access Main_Menu_Screen'Class ) is
        screen : A_Game_Screen;
    begin
        screen := Create_Paddle_War_Screen( this.Get_View, "scrPaddleWar" );
        A_Screen_Manager(this.parent).Add_Screen( screen );
    end Action_Paddle_War;

    ----------------------------------------------------------------------------

    procedure Action_Quit( this : not null access Main_Menu_Screen'Class ) is
    begin
        this.Get_View.Quit;
    end Action_Quit;

    ----------------------------------------------------------------------------

    procedure Action_Resume_Game( this : not null access Main_Menu_Screen'Class ) is
    begin
        this.Exit_Screen;
        A_Keen_View(this.Get_View).Pause_By_Menu( enabled => False );
    end Action_Resume_Game;

    ----------------------------------------------------------------------------

    overriding
    procedure Activate( this : access Main_Menu_Screen ) is
    begin
        Menu_Screen(this.all).Activate;
        if this.gameInProgress then
            this.Get_View.Get_Widget( this.id & ":btnResumeGame" ).Request_Focus;
        else
            this.Get_View.Get_Widget( this.id & ":btnNewGame" ).Request_Focus;
        end if;
    end Activate;

end Widgets.Containers.Game_Screens.Menus.Main;
