--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

--with Debugging;                         use Debugging;
with Game_Views;                        use Game_Views;
with Palette_16;
with Support;                           use Support;
with Widgets.Containers.Panels;         use Widgets.Containers.Panels;

package body Widgets.Containers.Game_Screens.Menus is

    ITEM_HEIGHT : constant := 8;       -- in pixels

    OPTIONS_LEFT   : constant := 109;    -- (40 from left/right edges)
    OPTIONS_RIGHT  : constant := -199;   --
    OPTIONS_TOP    : constant := 64;
    OPTIONS_BOTTOM : constant := -127;

    procedure Add_Item( this : not null access Menu_Screen'Class;
                        item : not null A_Widget );

    -- END PRIVATE DECLARATIONS --
    --==========================================================================

    overriding
    procedure Construct( this  : access Menu_Screen;
                         view  : not null access Game_Views.Game_View'Class;
                         id    : String;
                         popup : Boolean ) is
        options : A_Panel;
    begin
        Game_Screen(this.all).Construct( view, id, popup );

        options := Create_Panel( view, this.id & ":pnlOptions" );
        options.Set_Layout_LTRB( OPTIONS_LEFT, OPTIONS_TOP,
                                 OPTIONS_RIGHT, OPTIONS_BOTTOM );
        options.Set_Color( Foreground, Palette_16.Dark_Grey );
        options.Set_Border( Border_None );

        this.Add_Child( A_Widget(options), consume => False );
    end Construct;

    ----------------------------------------------------------------------------

    procedure Add_Item( this : not null access Menu_Screen'Class;
                        item : not null A_Widget ) is
    begin
        -- stack the buttons vertically
        item.Set_Layout_LTRB( 0, this.nextItemTop,
                              0, -(this.nextItemTop + ITEM_HEIGHT - 1) );
        this.nextItemTop := this.nextItemTop + ITEM_HEIGHT;

        if this.lastItem /= null then
            -- connect the next/prev focus behavior
            item.Set_Prev( this.lastItem.Get_Id );
            this.lastItem.Set_Next( item.Get_Id );
        end if;

        declare
            tmp : A_Widget;
        begin
            this.lastItem := item;
            tmp := this.lastItem;
            A_Panel(this.Get_View.Get_Widget( this.id & ":pnlOptions" )).Add_Widget( tmp );
        end;
    end Add_Item;

    ----------------------------------------------------------------------------

    procedure Add_Item( this   : not null access Menu_Screen'Class;
                        button : not null A_Button ) is
    begin
        this.Add_Item( A_Widget(button) );
    end Add_Item;

    ----------------------------------------------------------------------------

    procedure Add_Item( this   : not null access Menu_Screen'Class;
                        enum : not null A_Menu_Enumeration ) is
    begin
        this.Add_Item( A_Widget(enum) );
    end Add_Item;

end Widgets.Containers.Game_Screens.Menus;
