--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

with Allegro.Keyboard;                  use Allegro.Keyboard;
--with Debugging;                         use Debugging;
with Game_Views;                        use Game_Views;
with Support;                           use Support;
with Widgets.Containers;                use Widgets.Containers;
with Widgets.Paddle_Wars;               use Widgets.Paddle_Wars;
with Widgets.Simple_Key_Listeners;

pragma Elaborate_All( Widgets.Simple_Key_Listeners );

package body Widgets.Containers.Game_Screens.Paddle_War is

    package Key_Listeners is new Widgets.Simple_Key_Listeners(Paddle_War_Screen);
    use Key_Listeners;

    procedure Action_Escape( this   : not null access Paddle_War_Screen'Class;
                             action : A_Key_Action );

    -- END PRIVATE DECLARATIONS --
    --==========================================================================

    function Create_Paddle_War_Screen( view : not null access Game_Views.Game_View'Class;
                                       id   : String ) return A_Game_Screen is
        this : A_Game_Screen := new Paddle_War_Screen;
    begin
        this.Construct( view, id );
        return this;
    exception
        when others =>
            Delete( A_Widget(this) );
            raise;
    end Create_Paddle_War_Screen;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Paddle_War_Screen;
                         view : not null access Game_Views.Game_View'Class;
                         id   : String ) is
        paddleWar : A_Paddle_War;
    begin
        Game_Screen(this.all).Construct( view, id, popup => False );
        this.Set_Background( "paddlewar_menu" );

        this.Add_Listener( Listener( ALLEGRO_KEY_ESCAPE, Key_Press, this, Action_Escape'Access ) );

        -- paddle war widget
        paddleWar := Create_Paddle_War( view, this.id & ":paddleWar", "interface" );
        paddleWar.Set_Layout_LTWH( 75, 49, 0, 0 );
        this.Add_Child( A_Widget(paddleWar) );
    exception
        when others =>
            Delete( A_Widget(paddleWar) );
            raise;
    end Construct;

    ----------------------------------------------------------------------------

    procedure Action_Escape( this   : not null access Paddle_War_Screen'Class;
                             action : A_Key_Action ) is
        pragma Unreferenced( action );
    begin
        this.Exit_Screen;
    end Action_Escape;

    ----------------------------------------------------------------------------

    overriding
    procedure Activate( this : access Paddle_War_Screen ) is
    begin
        Game_Screen(this.all).Activate;
        this.Get_View.Get_Widget( this.id & ":paddleWar" ).Set_Visible( False );
        this.Get_View.Get_Widget( this.id & ":paddleWar" ).Set_Visible( True );    -- hack: forces it to handle ancestor unhidden and reset the game
        this.Get_View.Get_Widget( this.id & ":paddleWar" ).Request_Focus;
    end Activate;

end Widgets.Containers.Game_Screens.Paddle_War;
