--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

--with Debugging;                         use Debugging;
with Events.Keen;                       use Events.Keen;
with Game_Views;                        use Game_Views;
with Support;                           use Support;
with Widgets.Labels;                    use Widgets.Labels;
with Widgets.Containers.Keen_Panels;    use Widgets.Containers.Keen_Panels;
with Widgets.Buttons;                   use Widgets.Buttons;
with Widgets.Buttons.Pushes.Retry_Buttons;
 use Widgets.Buttons.Pushes.Retry_Buttons;
with Widgets.Buttons.Simple_Listeners;

pragma Elaborate_All( Widgets.Buttons.Simple_Listeners );

package body Widgets.Containers.Game_Screens.Retry is

    package Button_Listeners is new Widgets.Buttons.Simple_Listeners(Retry_Screen);
    use Button_Listeners;

    DIALOG_WIDTH  : constant := 210;
    DIALOG_HEIGHT : constant := 92;

    -- Called when the user responds with No to the retry level dialog.
    procedure Action_Retry_No( this : not null access Retry_Screen'Class );

    -- Called when the user responds with Yes to the retry level dialog.
    procedure Action_Retry_Yes( this : not null access Retry_Screen'Class );

    -- END PRIVATE DECLARATIONS --
    --==========================================================================

    function Create_Retry_Screen( view      : not null access Game_Views.Game_View'Class;
                                  id        : String;
                                  dialogId  : Integer;
                                  levelName : String;
                                  yesText   : String;
                                  noText    : String ) return A_Game_Screen is
        this : A_Game_Screen := new Retry_Screen;
    begin
        Retry_Screen(this.all).Construct( view, id, dialogId, levelName, yesText, noText );
        return this;
    exception
        when others =>
            Delete( A_Widget(this) );
            raise;
    end Create_Retry_Screen;

    ----------------------------------------------------------------------------

    not overriding
    procedure Construct( this      : access Retry_Screen;
                         view      : not null access Game_Views.Game_View'Class;
                         id        : String;
                         dialogId  : Integer;
                         levelName : String;
                         yesText   : String;
                         noText    : String ) is
        kpanel : A_Keen_Panel;
        label  : A_Label;
        button : A_Button;
    begin
        Game_Screen(this.all).Construct( view, id, popup => True );
        this.dialogId := dialogId;

        -- retry level dialog
        kpanel := Create_Panel( view, this.id & ":pnlRetry", "interface" );
        kpanel.Set_Layout_Center( DIALOG_WIDTH, DIALOG_HEIGHT );

            -- retry level text
            label := Create_Label( view, this.id & ":lblRetryText" );
            label.Set_Layout_LTRB( 0, 8, 0, -17 );
            label.Set_Text( "You didn't make it past" );
            label.Set_Align( Align_Center );
            label.Set_Color( Foreground, kpanel.Get_Color(Background) );
            label.Set_Font( "dos", 10 );
            kpanel.Add_Widget( A_Widget(label) );

            -- level name
            label := Create_Label( view, this.id & ":lblRetryLevel" );
            label.Set_Layout_LTRB( 0, 18, 0, -27 );
            label.Set_Text( levelName );
            label.Set_Align( Align_Center );
            label.Set_Color( Foreground, kpanel.Get_Color(Background) );
            label.Set_Font( "dos", 10 );
            kpanel.Add_Widget( A_Widget(label) );

            -- option 1 (yes, retry)
            button := Create_Retry_Button( view, this.id & ":btnRetry1" );
            button.Set_Layout_LTRB( 2, -41, 2, 22 );
            button.Set_Text( yesText );
            button.Set_Color( Foreground, kpanel.Get_Color(Background) );
            button.Set_Next( this.id & ":btnRetry2" );
            button.Add_Listener( Listener( this, Action_Retry_Yes'Access ) );
            kpanel.Add_Widget( A_Widget(button) );

            -- option 2 (no, quit)
            button := Create_Retry_Button( view, this.id & ":btnRetry2" );
            button.Set_Layout_LTRB( 2, -21, 2, 2 );
            button.Set_Text( noText );
            button.Set_Color( Foreground, kpanel.Get_Color(Background) );
            button.Set_Prev( this.id & ":btnRetry1" );
            button.Add_Listener( Listener( this, Action_Retry_No'Access ) );
            kpanel.Add_Widget( A_Widget(button) );

        this.Add_Child( A_Widget(kpanel) );
    exception
        when others =>
            Delete( A_Widget(button) );
            Delete( A_Widget(label) );
            Delete( A_Widget(kpanel) );
            raise;
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure Activate( this : access Retry_Screen ) is
    begin
        Game_Screen(this.all).Activate;
        this.Get_View.Get_Widget( this.id & ":btnRetry1" ).Request_Focus;
    end Activate;

    ----------------------------------------------------------------------------

    procedure Action_Retry_No( this : not null access Retry_Screen'Class ) is
    begin
        this.Exit_Screen;
        Queue_Dialog_Response( this.dialogId, 2 );   -- no is the second option
    end Action_Retry_No;

    ----------------------------------------------------------------------------

    procedure Action_Retry_Yes( this : not null access Retry_Screen'Class ) is
    begin
        this.Exit_Screen;
        Queue_Dialog_Response( this.dialogId, 1 );   -- yes is the first option
    end Action_Retry_Yes;

end Widgets.Containers.Game_Screens.Retry;
