--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

with Debugging;                         use Debugging;
with Drawing;                           use Drawing;
with Engine_Debugging;                  use Engine_Debugging;
with Game_Views;                        use Game_Views;
with Widgets.Containers;                use Widgets.Containers;

package body Widgets.Containers.Game_Screens is

    not overriding
    procedure Construct( this  : access Game_Screen;
                         view  : not null access Game_Views.Game_View'Class;
                         id    : String;
                         popup : Boolean ) is
    begin
        Container(this.all).Construct( view, id );
        this.popup := popup;
        if this.popup then
            this.transparent := True;
        end if;

        -- game screens fill their parent completely (the parent is expected to
        -- be a Screen_Manager object.)
        this.Set_Layout_LTRB( 0, 0, 0, 0 );
    end Construct;

    ----------------------------------------------------------------------------

    procedure Activate( this : access Game_Screen ) is
    begin
        pragma Debug( Dbg( "Activated screen " & this.To_String, D_GUI, INFO ) );
        this.active := True;
        this.Set_Visible( True );
        this.Request_Focus;
    end Activate;

    ----------------------------------------------------------------------------

    procedure Deactivate( this : access Game_Screen ) is
    begin
        pragma Debug( Dbg( "Deactivated screen " & this.To_String, D_GUI, INFO ) );
        this.Set_Visible( False );
        this.active := False;
    end Deactivate;

    ----------------------------------------------------------------------------

    overriding
    procedure Draw_Content( this : access Game_Screen ) is
        theme   : constant A_Theme := Themes.Get;
        bgImage : constant A_Allegro_Bitmap := theme.Get_Bitmap( this.background );
    begin
        if not this.transparent then
            Rectfill( 0, 0,
                      this.Get_Content_Width - 1, this.Get_Content_Height - 1,
                      this.colors(Background) );
        end if;
        if bgImage /= null then
            Draw_Bitmap( bgImage, 0, 0 );
        end if;
    end Draw_Content;

    ----------------------------------------------------------------------------

    procedure Exit_Screen( this : not null access Game_Screen'Class ) is
    begin
        this.exiting := True;
    end Exit_Screen;

    ----------------------------------------------------------------------------

    function Is_Active( this : not null access Game_Screen'Class ) return Boolean is
    begin
        return this.active;
    end Is_Active;

    ----------------------------------------------------------------------------

    function Is_Exiting( this : not null access Game_Screen'Class ) return Boolean is
    begin
        return this.exiting;
    end Is_Exiting;

    ----------------------------------------------------------------------------

    function Is_Popup( this : not null access Game_Screen'Class ) return Boolean is
    begin
        return this.popup;
    end Is_Popup;

    ----------------------------------------------------------------------------

    procedure Set_Background( this       : not null access Game_Screen'Class;
                              background : String ) is
    begin
        if background'Length > 0 then
            this.background := Themes.Get.Get_ID( background );
            pragma Debug( this.background = 0,
                          Dbg( "Warning: Background <" & background & "> " &
                               "not found for Game_Screen " & To_String( this ),
                               D_GUI, WARNING ) );
            this.transparent := this.background /= 0;
        else
            this.background := 0;
            this.transparent := False;
        end if;
    end Set_Background;

end Widgets.Containers.Game_Screens;
