--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

with Allegro.Displays;                  use Allegro.Displays;
--with Debugging;                         use Debugging;
with Drawing;                           use Drawing;
with Game_Views;                        use Game_Views;
--with Physics.Drawing;                   use Physics.Drawing;
with Preferences;
with Themes.Palette;                    use Themes.Palette;
with Tiles;                             use Tiles;
with Values.Casting;                    use Values.Casting;

pragma Elaborate_All( Preferences );

package body Widgets.Containers.Scenes.Keen is

    function Create_Scene( view : not null access Game_Views.Game_View'Class;
                           id   : String ) return A_Scene is
        this : A_Scene := new Keen_Scene;
    begin
        this.Construct( view, id );
        return this;
    exception
        when others =>
            Delete( A_Widget(this) );
            raise;
    end Create_Scene;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Keen_Scene;
                         view : not null access Game_Views.Game_View'Class;
                         id   : String ) is
    begin
        Scene(this.all).Construct( view, id );
        this.colors(Background) := base_colors(Black);
        this.Get_View.Get_Corral.Add_Listener( A_Event_Listener(this), "Follow_Entity" );
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    procedure Delete( this : in out Keen_Scene ) is
    begin
        this.Get_View.Get_Corral.Remove_Listener( A_Event_Listener(this), "Follow_Entity" );
--      Delete( this.clipMap );
        Scene(this).Delete;
    end Delete;

    ----------------------------------------------------------------------------

    overriding
    procedure Draw_Layer( this   : access Keen_Scene;
                          layer  : Positive;
                          startX,
                          startY : Integer;
                          tileX1,
                          tileY1,
                          tileX2,
                          tileY2 : Integer ) is
        drawY     : Integer := startY;
        drawX     : Integer := startX;
        mapOffset : Integer := this.mapLayers(layer).data'First + tileY1 * this.mapWidth;
        mapPos    : Integer;
        tileId    : Integer;     -- id of the tile to draw
        bmp       : A_Allegro_Bitmap;
    begin
        Al_Hold_Bitmap_Drawing( True );
        for y in tileY1..tileY2 loop
            drawX := startX;
            for x in tileX1..tileX2 loop
                mapPos := mapOffset + x;

                tileId := this.mapLayers(layer).data(mapPos);
                if tileId > 0 then
                    -- look up the bitmap in the cache
                    if tileId = this.bitmapCache(layer)(mapPos).tileId then
                        bmp := this.bitmapCache(layer)(mapPos).bmp;
                    else
                        -- bitmap not cached, find it
                        bmp := this.tileLib.Get_Tile( tileId ).Get_Bitmap;
                        if bmp /= null then
                            -- found the bitmap, put it in the cache
                            this.bitmapCache(layer)(mapPos) := (tileId, bmp);
                        end if;
                    end if;

                    if bmp /= null then
                        -- draw the tile to the scene
                        Draw_Bitmap( bmp, drawX, drawY );
                    end if;
                end if;

                drawX := drawX + this.tileWidth;
            end loop;
            drawY := drawY + this.tileWidth;
            mapOffset := mapOffset + this.mapWidth;
        end loop;
        Al_Hold_Bitmap_Drawing( False );
    end Draw_Layer;

    ----------------------------------------------------------------------------

--  overriding
--  procedure Draw_Layer_Overlay( this   : access Keen_Scene;
--                                startX,
--                                startY : Integer;
--                                tileX1,
--                                tileY1,
--                                tileX2,
--                                tileY2 : Integer ) is
--      drawY : Integer := startY;
--      drawX : Integer := startX;
--  begin
--      for y in tileY1..tileY2 loop
--          drawX := startX;
--          for x in tileX1..tileX2 loop
--              Draw( this.clipMap.Get( x, y ),
--                    drawX, drawY, this.tileWidth, base_colors(White) );
--              drawX := drawX + this.tileWidth;
--          end loop;
--          drawY := drawY + this.tileWidth;
--      end loop;
--  end Draw_Layer_Overlay;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_Event( this : access Keen_Scene;
                            evt  : in out A_Event;
                            resp : out Response_Type ) is
        pragma Unreferenced( resp );

        ------------------------------------------------------------------------

        procedure Do_Follow_Entity( evt : A_Entity_Event ) is
        begin
            this.Set_Target( evt.Get_Id );
        end Do_Follow_Entity;

        ------------------------------------------------------------------------

    begin
        if evt.Get_Id = FOLLOW_ENTITY_ID then
            Do_Follow_Entity( A_Entity_Event(evt) );
        else
            Scene(this.all).Handle_Event( evt, resp );
        end if;
    end Handle_Event;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_World_Loaded( this : access Keen_Scene;
                                   evt  : not null A_World_Loaded_Event ) is
    begin
        Scene(this.all).Handle_World_Loaded( evt );
        this.spriteLayer := this.mapLayers'Last - 1;
--      Delete( this.clipMap );
--      this.clipMap := Create_Clip_Map( this.mapWidth, this.mapHeight,
--                                       this.tileWidth, this.mapLayers,
--                                       this.tileLib.Get_Name );
    end Handle_World_Loaded;

    ----------------------------------------------------------------------------

    overriding
    procedure Handle_World_Property_Changed( this : access Keen_Scene;
                                             evt  : not null A_World_Property_Event ) is
    begin
        if evt.Get_Property_Name = "domain" then
            this.wSlackX := Preferences.Get_Pref( "development", "scene.keen." & Cast_String( evt.Get_Value ) & ".slack_x" );
            this.wSlackY := Preferences.Get_Pref( "development", "scene.keen." & Cast_String( evt.Get_Value ) & ".slack_y" );
        end if;
    end Handle_World_Property_Changed;

begin

    Preferences.Set_Default( "development", "scene.keen.Platform.slack_x", 44.0 );
    Preferences.Set_Default( "development", "scene.keen.Platform.slack_y", 50.0 );

    Preferences.Set_Default( "development", "scene.keen.Map.slack_x", 24.0 );
    Preferences.Set_Default( "development", "scene.keen.Map.slack_y", 24.0 );

end Widgets.Containers.Scenes.Keen;
