--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

with Ada.Tags;                          use Ada.Tags;
with Entities.Enemies;
with Resources;                         use Resources;

pragma Elaborate_All( Entities.Enemies );

package body Entities.Enemies.Keen4 is

    function Create_Arachnut return A_Entity is
        this : A_Entity := new Arachnut;
    begin
        this.Construct;
        return this;
    exception
        when RESOURCE_ERROR =>
            Delete( this );
            raise;
    end Create_Arachnut;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Arachnut ) is
    begin
        Enemy(this.all).Construct( 40, 40, "arachnut", "arachnut_run1" );
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    function Object_Input( stream : access Root_Stream_Type'Class ) return Arachnut is
        this : aliased Arachnut;
    begin
        this.Construct;
        Arachnut'Read( stream, this );
        return this;
    end Object_Input;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Read( stream : access Root_Stream_Type'Class; obj : out Arachnut ) is
    begin
        Enemy'Read( stream, Enemy(obj) );
    end Object_Read;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Write( stream : access Root_Stream_Type'Class; obj : Arachnut ) is
    begin
        Enemy'Write( stream, Enemy(obj) );
    end Object_Write;

    --==========================================================================

    function Create_Bounder return A_Entity is
        this : A_Entity := new Bounder;
    begin
        this.Construct;
        return this;
    exception
        when RESOURCE_ERROR =>
            Delete( this );
            raise;
    end Create_Bounder;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Bounder ) is
    begin
        Enemy(this.all).Construct( 24, 21, "bounder", "bounder_up1" );
        this.deadlyTouch := False;
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    function Object_Input( stream : access Root_Stream_Type'Class ) return Bounder is
        this : aliased Bounder;
    begin
        this.Construct;
        Bounder'Read( stream, this );
        return this;
    end Object_Input;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Read( stream : access Root_Stream_Type'Class; obj : out Bounder ) is
    begin
        Enemy'Read( stream, Enemy(obj) );
    end Object_Read;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Write( stream : access Root_Stream_Type'Class; obj : Bounder ) is
    begin
        Enemy'Write( stream, Enemy(obj) );
    end Object_Write;

    --==========================================================================

    function Create_Cloud return A_Entity is
        this : A_Entity := new Cloud;
    begin
        this.Construct;
        return this;
    exception
        when RESOURCE_ERROR =>
            Delete( this );
            raise;
    end Create_Cloud;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Cloud ) is
    begin
        Enemy(this.all).Construct( 64, 24, "cloud", "cloud_aware" );
        this.physical := False;
        this.deadlyTouch := False;
        this.shootable := False;
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    function Object_Input( stream : access Root_Stream_Type'Class ) return Cloud is
        this : aliased Cloud;
    begin
        this.Construct;
        Cloud'Read( stream, this );
        return this;
    end Object_Input;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Read( stream : access Root_Stream_Type'Class; obj : out Cloud ) is
    begin
        Enemy'Read( stream, Enemy(obj) );
    end Object_Read;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Write( stream : access Root_Stream_Type'Class; obj : Cloud ) is
    begin
        Enemy'Write( stream, Enemy(obj) );
    end Object_Write;

    --==========================================================================

    function Create_Inchworm return A_Entity is
        this : A_Entity := new Inchworm;
    begin
        this.Construct;
        return this;
    exception
        when RESOURCE_ERROR =>
            Delete( this );
            raise;
    end Create_Inchworm;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Inchworm ) is
    begin
        Enemy(this.all).Construct( 24, 16, "inchworm", "inchworm_left1" );
        this.deadlyTouch := False;
        this.shootable := False;
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    function Object_Input( stream : access Root_Stream_Type'Class ) return Inchworm is
        this : aliased Inchworm;
    begin
        this.Construct;
        Inchworm'Read( stream, this );
        return this;
    end Object_Input;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Read( stream : access Root_Stream_Type'Class; obj : out Inchworm ) is
    begin
        Enemy'Read( stream, Enemy(obj) );
    end Object_Read;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Write( stream : access Root_Stream_Type'Class; obj : Inchworm ) is
    begin
        Enemy'Write( stream, Enemy(obj) );
    end Object_Write;

    --==========================================================================

    function Create_Lick return A_Entity is
        this : A_Entity := new Lick;
    begin
        this.Construct;
        return this;
    exception
        when RESOURCE_ERROR =>
            Delete( this );
            raise;
    end Create_Lick;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Lick ) is
    begin
        Enemy(this.all).Construct( 22, 16, "lick", "lick_left2" );
        this.deadlyTouch := False;
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    function Object_Input( stream : access Root_Stream_Type'Class ) return Lick is
        this : aliased Lick;
    begin
        this.Construct;
        Lick'Read( stream, this );
        return this;
    end Object_Input;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Read( stream : access Root_Stream_Type'Class; obj : out Lick ) is
    begin
        Enemy'Read( stream, Enemy(obj) );
    end Object_Read;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Write( stream : access Root_Stream_Type'Class; obj : Lick ) is
    begin
        Enemy'Write( stream, Enemy(obj) );
    end Object_Write;

    --==========================================================================

    function Create_Mimrock return A_Entity is
        this : A_Entity := new Mimrock;
    begin
        this.Construct;
        return this;
    exception
        when RESOURCE_ERROR =>
            Delete( this );
            raise;
    end Create_Mimrock;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Mimrock ) is
    begin
        Enemy(this.all).Construct( 24, 24, "mimrock", "mimrock_sit" );
        this.deadlyTouch := False;
        this.shootable := False;
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    function Object_Input( stream : access Root_Stream_Type'Class ) return Mimrock is
        this : aliased Mimrock;
    begin
        this.Construct;
        Mimrock'Read( stream, this );
        return this;
    end Object_Input;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Read( stream : access Root_Stream_Type'Class; obj : out Mimrock ) is
    begin
        Enemy'Read( stream, Enemy(obj) );
    end Object_Read;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Write( stream : access Root_Stream_Type'Class; obj : Mimrock ) is
    begin
        Enemy'Write( stream, Enemy(obj) );
    end Object_Write;

    --==========================================================================

    function Create_Mushroom return A_Entity is
        this : A_Entity := new Mushroom;
    begin
        this.Construct;
        return this;
    exception
        when RESOURCE_ERROR =>
            Delete( this );
            raise;
    end Create_Mushroom;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Mushroom ) is
    begin
        Enemy(this.all).Construct( 32, 32, "mushroom", "mushroom_left1" );
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    function Object_Input( stream : access Root_Stream_Type'Class ) return Mushroom is
        this : aliased Mushroom;
    begin
        this.Construct;
        Mushroom'Read( stream, this );
        return this;
    end Object_Input;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Read( stream : access Root_Stream_Type'Class; obj : out Mushroom ) is
    begin
        Enemy'Read( stream, Enemy(obj) );
    end Object_Read;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Write( stream : access Root_Stream_Type'Class; obj : Mushroom ) is
    begin
        Enemy'Write( stream, Enemy(obj) );
    end Object_Write;

    --==========================================================================

    function Create_Slug return A_Entity is
        this : A_Entity := new Slug;
    begin
        this.Construct;
        return this;
    exception
        when RESOURCE_ERROR =>
            Delete( this );
            raise;
    end Create_Slug;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Slug ) is
    begin
        Enemy(this.all).Construct( 16, 24, "slug", "slug_left1" );
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    function Object_Input( stream : access Root_Stream_Type'Class ) return Slug is
        this : aliased Slug;
    begin
        this.Construct;
        Slug'Read( stream, this );
        return this;
    end Object_Input;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Read( stream : access Root_Stream_Type'Class; obj : out Slug ) is
    begin
        Enemy'Read( stream, Enemy(obj) );
    end Object_Read;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Write( stream : access Root_Stream_Type'Class; obj : Slug ) is
    begin
        Enemy'Write( stream, Enemy(obj) );
    end Object_Write;

    --==========================================================================

    function Create_Snake return A_Entity is
        this : A_Entity := new Snake;
    begin
        this.Construct;
        return this;
    exception
        when RESOURCE_ERROR =>
            Delete( this );
            raise;
    end Create_Snake;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Snake ) is
    begin
        Enemy(this.all).Construct( 16, 24, "snake", "snake_attack_left2" );
        this.deadlyTouch := False;
        this.shootable := False;
    end Construct;

    ----------------------------------------------------------------------------

    overriding
    function Object_Input( stream : access Root_Stream_Type'Class ) return Snake is
        this : aliased Snake;
    begin
        this.Construct;
        Snake'Read( stream, this );
        return this;
    end Object_Input;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Read( stream : access Root_Stream_Type'Class; obj : out Snake ) is
    begin
        Enemy'Read( stream, Enemy(obj) );
    end Object_Read;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Write( stream : access Root_Stream_Type'Class; obj : Snake ) is
    begin
        Enemy'Write( stream, Enemy(obj) );
    end Object_Write;

begin

    Entities.Factory.Register_Class( External_Tag( Arachnut'Tag ), Create_Arachnut'Access );
    Entities.Factory.Register_Class( External_Tag( Bounder'Tag ),  Create_Bounder'Access );
    Entities.Factory.Register_Class( External_Tag( Cloud'Tag ),    Create_Cloud'Access );
    Entities.Factory.Register_Class( External_Tag( Inchworm'Tag ), Create_Inchworm'Access );
    Entities.Factory.Register_Class( External_Tag( Lick'Tag ),     Create_Lick'Access );
    Entities.Factory.Register_Class( External_Tag( Mimrock'Tag ),  Create_Mimrock'Access );
    Entities.Factory.Register_Class( External_Tag( Mushroom'Tag ), Create_Mushroom'Access );
    Entities.Factory.Register_Class( External_Tag( Slug'Tag ),     Create_Slug'Access );
    Entities.Factory.Register_Class( External_Tag( Snake'Tag ),    Create_Snake'Access );

end Entities.Enemies.Keen4;
