--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

with Ada.Strings.Unbounded;             use Ada.Strings.Unbounded;
with Events.Entities;                   use Events.Entities;
with Keen_Directives;

package body Entities.Players.Little_Keen.States is

    IDLE_DELAY    : constant Time_Span := Milliseconds( 4500 );
    IDLE_DURATION : constant Time_Span := Milliseconds( 1800 );

    --==========================================================================

    type Frame_Type is
        record
            filename : String_Access := null;  -- todo: store a config path instead that contains the filename
            id       : Integer := -1;
        end record;

    type Frame_Array is array(Integer range <>) of Frame_Type;
    type A_Frame_Array is access all Frame_Array;

    type Direction_Array is array (Direction_8) of A_Frame_Array;
    type A_Direction_Array is access all Direction_Array;

    type Animation( directional : Boolean ) is
        record
            looping    : Boolean;
            frameDelay : Time_Span;
            case directional is
                when True =>
                    dir : A_Direction_Array := null;
                when False =>
                    frames : A_Frame_Array := null;
            end case;
        end record;
    type A_Animation is access all Animation;

    ----------------------------------------------------------------------------

    type Visible_Action is
    (
        Standing_Frames,
        Walking_Frames,
        Swimming_Frames,
        Waving_Frames
    );

    type Action_Frames is array (Visible_Action) of A_Animation;

    frames : constant Action_Frames := Action_Frames'(
        Standing_Frames => new Animation'(
            directional => True,
            looping     => True,
            frameDelay  => Milliseconds( 115 ),
            dir         => new Direction_Array'(
                D8_Left       => new Frame_Array'(1 => (filename => new String'("map_walk_left1" ), others => <>)),
                D8_Right      => new Frame_Array'(1 => (filename => new String'("map_walk_right1" ), others => <>)),
                D8_Up         => new Frame_Array'(1 => (filename => new String'("map_walk_up1" ), others => <>)),
                D8_Down       => new Frame_Array'(1 => (filename => new String'("map_walk_down1" ), others => <>)),
                D8_Up_Left    => new Frame_Array'(1 => (filename => new String'("map_walk_up_left1" ), others => <>)),
                D8_Up_Right   => new Frame_Array'(1 => (filename => new String'("map_walk_up_right1" ), others => <>)),
                D8_Down_Left  => new Frame_Array'(1 => (filename => new String'("map_walk_down_left1" ), others => <>)),
                D8_Down_Right => new Frame_Array'(1 => (filename => new String'("map_walk_down_right1" ), others => <>)))),
        Walking_Frames => new Animation'(
            directional => True,
            looping     => True,
            frameDelay  => Milliseconds( 115 ),
            dir         => new Direction_Array'(
                D8_Left => new Frame_Array'(
                    (filename => new String'("map_walk_left1" ), others => <>),
                    (filename => new String'("map_walk_left2" ), others => <>),
                    (filename => new String'("map_walk_left1" ), others => <>),
                    (filename => new String'("map_walk_left3" ), others => <>)),
                D8_Right => new Frame_Array'(
                    (filename => new String'("map_walk_right1" ), others => <>),
                    (filename => new String'("map_walk_right2" ), others => <>),
                    (filename => new String'("map_walk_right1" ), others => <>),
                    (filename => new String'("map_walk_right3" ), others => <>)),
                D8_Up => new Frame_Array'(
                    (filename => new String'("map_walk_up1" ), others => <>),
                    (filename => new String'("map_walk_up2" ), others => <>),
                    (filename => new String'("map_walk_up1" ), others => <>),
                    (filename => new String'("map_walk_up3" ), others => <>)),
                D8_Down => new Frame_Array'(
                    (filename => new String'("map_walk_down1" ), others => <>),
                    (filename => new String'("map_walk_down2" ), others => <>),
                    (filename => new String'("map_walk_down1" ), others => <>),
                    (filename => new String'("map_walk_down3" ), others => <>)),
                D8_Up_Left => new Frame_Array'(
                    (filename => new String'("map_walk_up_left1" ), others => <>),
                    (filename => new String'("map_walk_up_left2" ), others => <>),
                    (filename => new String'("map_walk_up_left1" ), others => <>),
                    (filename => new String'("map_walk_up_left3" ), others => <>)),
                D8_Up_Right => new Frame_Array'(
                    (filename => new String'("map_walk_up_right1" ), others => <>),
                    (filename => new String'("map_walk_up_right2" ), others => <>),
                    (filename => new String'("map_walk_up_right1" ), others => <>),
                    (filename => new String'("map_walk_up_right3" ), others => <>)),
                D8_Down_Left => new Frame_Array'(
                    (filename => new String'("map_walk_down_left1" ), others => <>),
                    (filename => new String'("map_walk_down_left2" ), others => <>),
                    (filename => new String'("map_walk_down_left1" ), others => <>),
                    (filename => new String'("map_walk_down_left3" ), others => <>)),
                D8_Down_Right => new Frame_Array'(
                    (filename => new String'("map_walk_down_right1" ), others => <>),
                    (filename => new String'("map_walk_down_right2" ), others => <>),
                    (filename => new String'("map_walk_down_right1" ), others => <>),
                    (filename => new String'("map_walk_down_right3" ), others => <>)))),
        Swimming_Frames => new Animation'(
            directional => True,
            looping     => True,
            frameDelay  => Milliseconds( 115 ),
            dir         => new Direction_Array'(
                D8_Left => new Frame_Array'(
                    (filename => new String'("map_swim_left1" ), others => <>),
                    (filename => new String'("map_swim_left2" ), others => <>),
                    (filename => new String'("map_swim_left1" ), others => <>),
                    (filename => new String'("map_swim_left3" ), others => <>)),
                D8_Right => new Frame_Array'(
                    (filename => new String'("map_swim_right1" ), others => <>),
                    (filename => new String'("map_swim_right2" ), others => <>),
                    (filename => new String'("map_swim_right1" ), others => <>),
                    (filename => new String'("map_swim_right3" ), others => <>)),
                D8_Up => new Frame_Array'(
                    (filename => new String'("map_swim_up1" ), others => <>),
                    (filename => new String'("map_swim_up2" ), others => <>),
                    (filename => new String'("map_swim_up1" ), others => <>),
                    (filename => new String'("map_swim_up3" ), others => <>)),
                D8_Down => new Frame_Array'(
                    (filename => new String'("map_swim_down1" ), others => <>),
                    (filename => new String'("map_swim_down2" ), others => <>),
                    (filename => new String'("map_swim_down1" ), others => <>),
                    (filename => new String'("map_swim_down3" ), others => <>)),
                D8_Up_Left => new Frame_Array'(
                    (filename => new String'("map_swim_up_left1" ), others => <>),
                    (filename => new String'("map_swim_up_left2" ), others => <>),
                    (filename => new String'("map_swim_up_left1" ), others => <>),
                    (filename => new String'("map_swim_up_left3" ), others => <>)),
                D8_Up_Right => new Frame_Array'(
                    (filename => new String'("map_swim_up_right1" ), others => <>),
                    (filename => new String'("map_swim_up_right2" ), others => <>),
                    (filename => new String'("map_swim_up_right1" ), others => <>),
                    (filename => new String'("map_swim_up_right3" ), others => <>)),
                D8_Down_Left => new Frame_Array'(
                    (filename => new String'("map_swim_down_left1" ), others => <>),
                    (filename => new String'("map_swim_down_left2" ), others => <>),
                    (filename => new String'("map_swim_down_left1" ), others => <>),
                    (filename => new String'("map_swim_down_left3" ), others => <>)),
                D8_Down_Right => new Frame_Array'(
                    (filename => new String'("map_swim_down_right1" ), others => <>),
                    (filename => new String'("map_swim_down_right2" ), others => <>),
                    (filename => new String'("map_swim_down_right1" ), others => <>),
                    (filename => new String'("map_swim_down_right3" ), others => <>)))),
        Waving_Frames => new Animation'(
            directional => False,
            looping     => True,
            frameDelay  => IDLE_DURATION / 5,
            frames      => new Frame_Array'(
                (filename => new String'("map_wave1" ), others => <>),
                (filename => new String'("map_wave2" ), others => <>)))
    );

    ----------------------------------------------------------------------------

    function Get_Frame_Id( lib       : not null A_Tile_Library;
                           action    : Visible_Action;
                           actionAge : Time_Span;
                           dir       : Direction_8 := D8_Up;
                           reversed  : Boolean := False ) return Natural is

        ------------------------------------------------------------------------

        procedure Resolve_Id( id : out Natural; frame : in out Frame_Type ) is
        begin
            if frame.id < 0 then
                frame.id := lib.Get_Id( frame.filename.all );
            end if;
            id := frame.id;
        end Resolve_Id;

        ------------------------------------------------------------------------

        frameArray : access Frame_Array;
        frameNum   : Natural := 0;
        id         : Natural := 0;
    begin
        if frames(action).directional then
            frameArray := frames(action).dir(dir);
        else
            frameArray := frames(action).frames;
        end if;

        if frameArray'Length = 1 then
            frameNum := 0;
        else
            if frames(action).looping then
                frameNum := (actionAge / frames(action).frameDelay) mod frameArray'Length;
            else
                frameNum := Natural'Min( (actionAge / frames(action).frameDelay), frameArray'Length - 1 );
            end if;
        end if;

        if reversed then
            Resolve_Id( id, frameArray(frameArray'Last - frameNum) );
        else
            Resolve_Id( id, frameArray(frameArray'First + frameNum) );
        end if;

        return id;
    end Get_Frame_Id;

    ----------------------------------------------------------------------------

    -- singleton state instances
    MOVING : constant A_State := new Moving_State'(action => Act_Moving);
    BORED  : constant A_State := new Bored_State'(action => Act_Bored);

    --==========================================================================

    function Get_State( action : Action_Type ) return A_State is
    begin
        case action is
            when Act_Moving => return MOVING;
            when Act_Bored  => return BORED;
        end case;
    end Get_State;

    --==========================================================================

    overriding
    function Get_Frame( this : Moving_State; k : Little_Keen'Class ) return Natural is
        pragma Unreferenced( this );
    begin
        if k.swimming then
            return Get_Frame_Id( k.lib,
                                 Swimming_Frames, k.age - k.stateStart,
                                 To_D8( k.dir ) );
        else
            if not k.moving then
                return Get_Frame_Id( k.lib,
                                     Standing_Frames, k.age - k.stateStart,
                                     To_D8( k.dir ) );
            else
                return Get_Frame_Id( k.lib,
                                     Walking_Frames, k.age - k.stateStart,
                                     To_D8( k.dir ) );
            end if;
        end if;
    end Get_Frame;

    ----------------------------------------------------------------------------

    overriding
    function Get_Frame( this : Bored_State; k : Little_Keen'Class ) return Natural is
        pragma Unreferenced( this );
    begin
        return Get_Frame_Id( k.lib,
                             Waving_Frames, k.age - k.stateStart );
    end Get_Frame;

    --==========================================================================

    overriding
    procedure Update( this : Moving_State; k : A_Little_Keen ) is
        pragma Unreferenced( this );

        ------------------------------------------------------------------------

        procedure Start_Thrusting( dir : Cardinal_Direction ) is
        begin
            k.Reset_Idle;

            -- if Keen isn't already moving in this direction and he isn't
            -- moving in the opposite direction, then start moving in this
            -- direction.
            if not k.moving(dir) and then not k.moving(Opposite( dir )) then
                -- update the facing and moving directions
                if not k.moving then
                    k.dir := To_Direction( dir );
                else
                    k.dir := k.dir + dir;
                end if;
                k.moving(dir) := True;

                -- accelerate to walking speed in this direction
                Queue_Accelerate( k.id, dir, k.walkSpeed, k.walkAcceleration );
            end if;
        end Start_Thrusting;

        ------------------------------------------------------------------------

        procedure Stop_Thrusting( dir : Cardinal_Direction ) is
        begin
            if k.moving(dir) then
                -- update the facing and moving directions
                k.dir := k.dir - dir;
                k.moving(dir) := False;

                -- accelerate to a stop in this direction
                Queue_Accelerate( k.id, dir, 0.0, 0.0 );
            end if;
        end Stop_Thrusting;

        ------------------------------------------------------------------------

        -- activates all entities that Keen is touching
        procedure Activate is
            procedure Activate( e : A_Entity ) is
            begin
                e.Activate( activator => A_Entity(k) );
            end Activate;
        begin
            k.Iterate_Touching( Activate'Access );
        end Activate;

        ------------------------------------------------------------------------

    begin
        if k.Directive( Keen_Directives.LEFT ) then
            Start_Thrusting( Left );
        else
            Stop_Thrusting( Left );
        end if;
        if k.Directive( Keen_Directives.RIGHT ) then
            Start_Thrusting( Right );
        else
            Stop_Thrusting( Right );
        end if;
        if k.Directive( Keen_Directives.UP ) then
            Start_Thrusting( Up );
        else
            Stop_Thrusting( Up );
        end if;
        if k.Directive( Keen_Directives.DOWN ) then
            Start_Thrusting( Down );
        else
            Stop_Thrusting( Down );
        end if;

        if k.Directive( Keen_Directives.JUMP ) or else
           k.Directive( Keen_Directives.POGO )
        then
            Activate;
        end if;

        -- these are given by Ked to change Keen's orientation
        if k.Directive( Keen_Directives.FACE_LEFT ) then
            k.dir := Dir_Left;
        elsif k.Directive( Keen_Directives.FACE_RIGHT ) then
            k.dir := Dir_Right;
        elsif k.Directive( Keen_Directives.FACE_UP ) then
            k.dir := Dir_Up;
        elsif k.Directive( Keen_Directives.FACE_DOWN ) then
            k.dir := Dir_Down;
        end if;

        -- check if Keen is bored yet
        if k.age >= k.idleStart + IDLE_DELAY then
            k.Change_State( BORED, IDLE_DURATION );
            return;
        end if;
    end Update;

    ----------------------------------------------------------------------------

    overriding
    procedure Update( this : Bored_State; k : A_Little_Keen ) is
        pragma Unreferenced( this );

        ------------------------------------------------------------------------

        -- activates all entities that Keen is touching
        procedure Activate is
            procedure Activate( e : A_Entity ) is
            begin
                e.Activate( activator => A_Entity(k) );
            end Activate;
        begin
            k.Iterate_Touching( Activate'Access );
        end Activate;

        ------------------------------------------------------------------------

    begin
        if k.Directive( Keen_Directives.JUMP ) or else
           k.Directive( Keen_Directives.POGO )
        then
            Activate;
        end if;

        -- if keen is done being bored (waving) or he gets a directive, then go
        -- back to the moving state.
        if k.Directive( Keen_Directives.LEFT ) or else
           k.Directive( Keen_Directives.RIGHT ) or else
           k.Directive( Keen_Directives.UP ) or else
           k.Directive( Keen_Directives.DOWN ) or else
           not k.Is_Busy
        then
            k.Change_State( MOVING );
            k.Reset_Idle;
            return;
        end if;
    end Update;

end Entities.Players.Little_Keen.States;
