--
-- Copyright (c) 2012 Kevin Wellwood
-- All rights reserved.
--
-- This source code is distributed under the Modified BSD License. For terms and
-- conditions, see license.txt.
--

--with Debugging;                         use Debugging;
with Entities.Players;
with Entities.Players.Little_Keen.States;
 use Entities.Players.Little_Keen.States;
with Preferences;

pragma Elaborate_All( Entities.Players );
pragma Elaborate_All( Preferences );

package body Entities.Players.Little_Keen is

    PREF_PREFIX : constant String := "entity.player.little_keen.";

    function Get_Pref( shortName : String ) return Integer;

    function Get_Pref( shortName : String ) return Float;

    function Get_Pref( shortName : String ) return String;

    -- END PRIVATE DECLARATIONS --
    --==========================================================================

    function Get_Pref( shortName : String ) return Integer is
    begin
        return Preferences.Get_Pref( "development", PREF_PREFIX & shortName );
    end Get_Pref;
    pragma Inline( Get_Pref );

    ----------------------------------------------------------------------------

    function Get_Pref( shortName : String ) return Float is
    begin
        return Preferences.Get_Pref( "development", PREF_PREFIX & shortName );
    end Get_Pref;
    pragma Inline( Get_Pref );

    ----------------------------------------------------------------------------

    function Get_Pref( shortName : String ) return String is
    begin
        return Preferences.Get_Pref( "development", PREF_PREFIX & shortName );
    end Get_Pref;
    pragma Inline( Get_Pref );

    --==========================================================================

    function Create_Player return A_Entity is
        this : constant A_Entity := new Little_Keen;
    begin
        this.Construct;
        return this;
    end Create_Player;

    ----------------------------------------------------------------------------

    overriding
    procedure Construct( this : access Little_Keen ) is
    begin
        Player(this.all).Construct( Get_Pref( "width" ),
                                    Get_Pref( "height" ),
                                    Get_Pref( "library" ) );
        this.state := Get_State( Act_Moving );
        this.walkSpeed := Get_Pref( "walk_speed" );
        this.walkAcceleration := Get_Pref( "walk_acceleration" );
        this.Update_Frame( notify => False );
        this.Reset_Idle;
    end Construct;

    ----------------------------------------------------------------------------

    procedure Change_State( this   : not null access Little_Keen'Class;
                            state  : not null A_State;
                            length : Time_Span := Time_Span_Zero ) is
    begin
        -- change keen's state, if 'state' is different. begin tracking how long
        -- he's been in this state (via .stateStart) and how long he must be
        -- in it (as .stateDuration).
        if this.state /= state then
            this.state := state;
            this.stateStart := this.age;
            this.stateDuration := length;
        end if;
    end Change_State;

    ----------------------------------------------------------------------------

    procedure Die( this : access Little_Keen ) is
        pragma Unreferenced( this );
    begin
        null;
    end Die;

    ----------------------------------------------------------------------------

    function Is_Busy( this : not null access Little_Keen'Class ) return Boolean is
    begin
        if this.stateDuration > Time_Span_Zero then
            return this.stateStart + this.stateDuration > this.age;
        end if;
        return False;
    end Is_Busy;

    ----------------------------------------------------------------------------

    overriding
    function Object_Input( stream : access Root_Stream_Type'Class ) return Little_Keen is
        this : aliased Little_Keen;
    begin
        this.Construct;
        Little_Keen'Read( stream, this );
        return this;
    end Object_Input;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Read( stream : access Root_Stream_Type'Class; obj : out Little_Keen ) is
    begin
        Player'Read( stream, Player(obj) );
        obj.state := Get_State( Action_Type'Input( stream ) );
        obj.stateStart := Time_Span'Input( stream );
        obj.stateDuration := Time_Span'Input( stream );
        obj.swimming := Boolean'Input( stream );
        obj.Reset_Idle;
    end Object_Read;

    ----------------------------------------------------------------------------

    overriding
    procedure Object_Write( stream : access Root_Stream_Type'Class; obj : Little_Keen ) is
    begin
        Player'Write( stream, Player(obj) );
        Action_Type'Output( stream, obj.state.action );
        Time_Span'Output( stream, obj.stateStart );
        Time_Span'Output( stream, obj.stateDuration );
        Boolean'Output( stream, obj.swimming );
    end Object_Write;

    ----------------------------------------------------------------------------

    procedure Reset_Idle( this : not null access Little_Keen'Class ) is
    begin
        -- reset the idle time tracking (a lack of directives)
        this.idleStart := this.age;
    end Reset_Idle;

    ----------------------------------------------------------------------------

    overriding
    procedure Update( this : access Little_Keen; time : Tick_Time ) is
        pragma Unreferenced( time );
    begin
        this.state.Update( A_Little_Keen(this) );
    end Update;

    ----------------------------------------------------------------------------

    overriding
    procedure Update_Frame( this : access Little_Keen; notify : Boolean := True ) is
    begin
        -- determine keen's current visible action
        this.Set_Frame( this.state.Get_Frame( this.all ), notify );
    end Update_Frame;

begin

    Entities.Factory.Register_Class( "player.Map", Create_Player'Access );

    Preferences.Set_Default( "development", "entity.player.little_keen.library",           "keen" );
    Preferences.Set_Default( "development", "entity.player.little_keen.height",            16     );
    Preferences.Set_Default( "development", "entity.player.little_keen.width",             16     );
    Preferences.Set_Default( "development", "entity.player.little_keen.walk_speed",        92.0   ); -- measured at 80 px/s with keen4e
    Preferences.Set_Default( "development", "entity.player.little_keen.walk_acceleration", 700.0  );

end Entities.Players.Little_Keen;
